<?php

namespace App\Controllers;
use App\Models\MemberModel;
use CodeIgniter\API\ResponseTrait;
use CodeIgniter\Log\Logger;
use App\Util\CodeUtil;
use App\Util\accessLogRedis;
use App\Models\MainModel;

class AccessLogController extends BaseController {

    use ResponseTrait;
   
    public function index() {
        if (false == session()->has('member_idx')) {
            return;
        }

        $chkMobile = CodeUtil::rtn_mobile_chk();

        $current_page = isset($_POST['current_page']) ? $_POST['current_page'] : $_SERVER["PHP_SELF"];
        $viewRoot = "PC" == $chkMobile ? 'PC' : 'Mobile';

        $memberIdx = session()->get('member_idx');
        $memberModel = new MemberModel();
        $member = $memberModel->getMemberWhereIdx($memberIdx);

        if ($member == null) {
            $response['messages'] = '조회되는 유저가 없습니다.';
            return $this->fail($response);
        }
        
        // 전체 충환전
        $sql = "SELECT charge_total_money, exchange_total_money FROM total_member_cash WHERE member_idx = ?";
        $resultTotalCash = $memberModel->db->query($sql, [$memberIdx])->getResultArray();
        
        // 오늘의 충환전
        $start_dt = date('Y-m-d 00:00:00');
        $end_dt = date('Y-m-d 23:59:59');
        $sql = "SELECT ifnull(sum(money), 0) as money FROM member_money_charge_history WHERE member_idx = ? and status = 3 and update_dt >= ? and ? <= update_dt";
        $resultTodayCharge = $memberModel->db->query($sql, [$memberIdx, $start_dt, $end_dt])->getResultArray();
        
        $sql = "SELECT ifnull(sum(money), 0) as money FROM member_money_exchange_history WHERE member_idx = ? and status = 3 and update_dt >= ? and ? <= update_dt";
        $resultTodayExchange = $memberModel->db->query($sql, [$memberIdx, $start_dt, $end_dt])->getResultArray();
        
        $level = $member->getLevel();
        $id = $member->getId();
        $nickName = $member->getNickName();
        $cal = ($resultTotalCash[0]['charge_total_money'] + $resultTodayCharge[0]['money']) - ($resultTotalCash[0]['exchange_total_money'] + $resultTodayExchange[0]['money']);
        //$page = basename($_SERVER["PHP_SELF"]);
        $hostname = $_SERVER["HTTP_HOST"];
        $client_ip = CodeUtil::get_client_ip();
        $now_time = date("Y-m-d H:i:s");
        $arrRedisData = array(
            'level' => $level,
            'id' => $id,
            'nickName' => $nickName,
            'cal' => $cal,
            'access_type' => $viewRoot,
            'page' => $current_page,
            'path' => $_POST['path'],
            'hostname' => $hostname,
            'client_ip' => $client_ip,
            'now_time' => $now_time
        );
        
        //print_r($arrRedisData);($host, $post, $password, $database, $expire, $logger)
        //return;
        $accessLogRedis = new accessLogRedis(config(App::class)->redis_ip, config(App::class)->redis_port, 
                config(App::class)->redis_password, config(App::class)->redis_database, config(App::class)->redis_expire, $this->logger);
        if(!$accessLogRedis->connect()){
            $response['messages'] = '레디스 연결 실패.';
            return $this->fail($response);
        }

        // 일정갯수 이상이면 오래된 순으로 하나 삭제
        $count = $accessLogRedis->llen($memberIdx);
        if ($count > 500) {
            $accessLogRedis->lpop($memberIdx, 1);
        }

        $accessLogRedis->rpush($memberIdx, json_encode($arrRedisData));

        if ('/web/sports/ajax_add_fixtures' != $_POST['path'] && 9 != $member->getLevel() && ('sports' == $current_page || 'classic' == $current_page)) {
            $bet_type = 1;
            $mainModel = new MainModel();

            $sql = "select * from tb_both_bet_user_list where member_idx = ? order by create_dt desc limit 1";

            $result_list = $memberModel->db->query($sql, [$memberIdx])->getResultArray();
            $this->logger->info(' current_page sql :' . $memberModel->getLastQuery());
            if (!isset($result_list) || true == empty($result_list)) {
                $this->logger->info(' current_page 1');
                $this->doCheckBothBetUser($memberIdx, $bet_type, $result_list, $mainModel, $memberModel);
            } else {
                $this->logger->info(' current_page 2' . json_encode($result_list));
                $currentTime = date("Y-m-d H:i:s");
                if ('N' == $result_list[0]['is_display'] && $result_list[0]['display_dt'] < $currentTime) {
                    $this->doCheckBothBetUser($memberIdx, $bet_type, $result_list, $mainModel, $memberModel);
                }
            }
            //$sql_delete = 'delete from tb_user_site_call_static where member_idx = ? and create_dt < date_sub(NOW(),interval 30 MINUTE)';
            //$mainModel->db->query($sql_delete, [$memberIdx]);
        }
        echo 'success';
    }

    private function doCheckBothBetUser($memberIdx, $bet_type, $result_list, $mainModel, $memberModel) {
        $sql = 'INSERT INTO tb_user_site_call_static (member_idx,bet_type) VALUES (?,?)';
        $mainModel->db->query($sql, [$memberIdx, $bet_type]);
        $sql_sl = 'select count(*) as cnt from tb_user_site_call_static where member_idx = ? and bet_type = 1 and  date_sub(NOW(),interval 30 MINUTE) < create_dt group by member_idx,bet_type';
        $result_1 = $mainModel->db->query($sql_sl, [$memberIdx])->getResultArray();
        if (true === isset($result_1) && 45 < $result_1[0]['cnt']) {
            //$this->logger->info('test1');
            //$is_monitor_bet_both = 'Y';
            //$sql_update = 'update member set is_monitor_bet_both = ? where idx = ? ';
            //$memberModel->db->query($sql_update, [$is_monitor_bet_both, $memberIdx]);
            $this->logger->info(' current_page 3' . json_encode($result_1));
            //if (!isset($result_list) || true == empty($result_list)) {
            $sql = 'INSERT INTO tb_both_bet_user_list (member_idx,is_display,is_sound) VALUES (?,?,?)';
            $memberModel->db->query($sql, [$memberIdx, 'Y', 'Y']);
            //} else {
            // 만약 대기 상태이면 데이터를 
            //     $sql = "update tb_both_bet_user_list set is_display = 'Y', is_sound = 'Y',update_dt = now(),display_dt = now() where member_idx = ? ";
            //     $mainModel->db->query($sql, [$memberIdx]);
            //}

            $sql_delete = 'delete from tb_user_site_call_static where member_idx = ?';
            $mainModel->db->query($sql_delete, [$memberIdx]);
        }
    }

    public function doDeleteBothBetUser() {
        $member_idx = isset($_POST['member_idx']) ? $_POST['member_idx'] : 0;
        $memberModel = new MemberModel();

        $sql_delete = 'delete from tb_both_bet_user_list where member_idx = ?';
        $memberModel->db->query($sql_delete, [$memberIdx]);
        $response = [
            'result_code' => 200,
            'messages' => '삭제 성공',
            'data' => [
                'del_idx' => $member_idx
            ]
        ];
        return $this->respond($response, 200);
    }

}
