<?php

namespace App\Controllers;

use App\Entities\MemberBet;
use App\Models\GameModel;
use App\Models\MemberBetModel;
use App\Models\MemberModel;
use App\Models\TGameConfigModel;
use App\Models\TLogCashModel;
use App\Models\TotalMemberCashModel;
use App\Models\MiniGameMemberBetModel;
use App\Models\LSportsSportsModel;
use CodeIgniter\API\ResponseTrait;
use App\Util\BetDataUtil;
use App\Util\CodeUtil;
use App\Util\Calculate;
use App\Util\AbuseIPDB;
use App\GamblePatch\GmPatch;
use App\GamblePatch\NoneGmPatch;
use App\RollingComps\UserPayBack;
use App\RollingComps\UserBettingLose;
use App\Util\accessLogRedis;

class BetController extends BaseController {

    use ResponseTrait;

    // API
    protected $gmPt; // 겜블패치 

    public function __construct() {
        if ('ON' == config(App::class)->IS_GM_PATCH) {
            $this->gmPt = new GmPatch();
        } else {
            $this->gmPt = new NoneGmPatch();
        }
        
        helper('form');
        helper('security');
    }

    public function addMiniBet() {
        $memberIdx = session()->get('member_idx');
        $betList = isset($_POST['betList']) ? $_POST['betList'] : NULL;
        $betType = isset($_POST['betType']) ? $_POST['betType'] : NULL;
        $totalMoney = isset($_POST['totalMoney']) ? $_POST['totalMoney'] : NULL;
        $keep_login_access_token = isset($_POST['keep_login_access_token']) ? $_POST['keep_login_access_token'] : '';
        $response['keep_login_access_token'] = '';
        $chkMobile = CodeUtil::rtn_mobile_chk();
        $viewRoot = "PC" == $chkMobile ? 'web' : 'web';

        if (false == CodeUtil::only_number($memberIdx) || false == CodeUtil::only_number($betType) || false == CodeUtil::only_number($totalMoney)) {
            $response['messages'] = '인자값이 잘못되었습니다.';
            return $this->fail($response);
        }

        if (0 < session()->get('tm_unread_cnt')) {
            $url = base_url("/web/note");
            echo "<script>
            alert('미확인 쪽지를 확인 바랍니다.');
            window.location.href='$url';
            </script>";
            return;
        }
        $this->logger->info("test :: start@@@");

        $memberModel = new MemberModel();
        try {
            $memberModel->db->transStart();
            list($revtal, $message) = $this->checkBettingAccessToken($keep_login_access_token, $memberModel);
            if (false == $revtal) {
                $memberModel->db->transRollback();
                $response['messages'] = $message;
                return $this->fail($response);
            }

            list($revtal, $message, $member, $memberIdx) = $this->checkMemberStatus(session()->get('member_idx'), $memberModel);
            if (false == $revtal) {
                $memberModel->db->transRollback();
                $response['messages'] = $message;
                return $this->fail($response);
            }

            if ($member->getMoney() < $totalMoney) {
                $memberModel->db->transRollback();
                $response['messages'] = '배팅 금액이 부족합니다.';
                return $this->fail($response);
            }

            // Commercial ip check
            list($retVal, $message) = CodeUtil::checkCommercialIp($member, $this->logger);
            if (false == $retVal) {
                $memberModel->db->transRollback();
                $response['messages'] = $message;
                return $this->fail($response);
            }

            // 중복배팅
            $lastBetInfo = $this->getLastBettingInfo($memberModel, $member->getIdx());
            $checkStartTime = date("Y-m-d H:i:s", strtotime($lastBetInfo[0]['betting_dt'] . "+" . 3 . "seconds"));
            $currentTime = date('Y-m-d H:i:s');

            if (0 < count($lastBetInfo) && $currentTime < $checkStartTime) {
                $sql = "UPDATE member set is_monitor_bet = 'Y' WHERE idx = ? ";
                $memberModel->db->query($sql, [$memberIdx]);

                $response['messages'] = '정상배팅이 아닙니다.!';
                $memberModel->db->transComplete();
                return $this->fail($response);
            }

            $gameType = 5;
            /*if (3 == $betType) {
                $gameType = 5;
            } else if (4 == $betType) {
                $gameType = 6;
            } if (5 == $betType || 57 == $betType || 58 == $betType || 59 == $betType || 60 == $betType || 61 == $betType) {
                $gameType = 7;
            } if (6 == $betType || 50 == $betType || 51 == $betType) {
                $gameType = 8;
            } if (15 == $betType || 52 == $betType || 53 == $betType || 54 == $betType || 55 == $betType || 56 == $betType) {
                $gameType = 15;
            } if (63 == $betType || 64 == $betType || 65 == $betType) {
                $gameType = 8;
            } if (62 == $betType) {
                $gameType = 8;
            }*/
            if (3 == $betType) {
                $gameType = 5;
            } else if (4 == $betType) {
                $gameType = 6;
            } else if (5 == $betType) {
                $gameType = 7;
            } else if (6 == $betType) {
                $gameType = 8;
            } else {
                $gameType = $betType;
            }
            
            // 가상축구 bet type 6번으로 들어가게 수정
            if (66 == $betType || 67 == $betType || 68 == $betType) {
                $betType = 6;
            }
            
            // 경마는 bet type 50
            if (70 == $betType || 71 == $betType) {
                $betType = 50;
            }
            
            // 개경주는 bet type 51
            if (69 == $betType) {
                $betType = 51;
            }
            
            $game_type_sql = "SELECT status FROM member_game_type where member_idx = ? and game_type = ?";
            $arr_db_result_game_type = $memberModel->db->query($game_type_sql, [$memberIdx, $gameType])->getResultArray();

            if ('OFF' == $arr_db_result_game_type[0]['status']) {
                $memberModel->db->transRollback();
                $response['messages'] = '해당 게임 배팅이 금지되어있습니다.';
                return $this->fail($response);
            }

            $tgcModel = new TGameConfigModel();
            $this->logger->info("test :: 2");
            // 배팅 금지 상태인지 확인하자.(레벨9 유저는 베팅가능)
            if (9 != $member->getLevel()) {
                $str_sql_config = "SELECT set_type, set_type_val 
                                FROM t_game_config 
                                WHERE set_type 
                                    IN('mini_service_powerball',
                                        'mini_service_eos_powerball',
                                        'mini_service_power_ladder',
                                        'mini_service_kino_ladder',
                                        'mini_service_v_soccer',
                                        'mini_service_v_horse',
                                        'mini_service_v_dog',
                                        'mini_service_n_rpowerball',
                                        'mini_service_nxt_powerball',
                                        'mini_service_sh_powerball',
                                        'mini_service_n_rpladder',
                                        'mini_service_nxt_ladder',
                                        'mini_service_sh_ladder',
                                        'mini_service_sh_race',
                                        'mini_service_nxt_bac'
                                    )";
                $arr_config_result = $tgcModel->db->query($str_sql_config)->getResultArray();

                $arr_config = array();
                foreach ($arr_config_result as $key => $value) {
                    $arr_config[$value['set_type']] = $value['set_type_val'];
                }

                if (3 == $betType && 'N' == $arr_config['mini_service_eos_powerball']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = 'EOS 파워볼 배팅은 점검중입니다.';
                    return $this->fail($response);
                } else if (4 == $betType && 'N' == $arr_config['mini_service_power_ladder']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '파워사다리 점검중입니다.';
                    return $this->fail($response);
                } else if (5 == $betType && 'N' == $arr_config['mini_service_kino_ladder']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '키노사다리 점검중입니다.';
                    return $this->fail($response);
                } else if ((6 == $betType || 66 == $betType || 67 == $betType || 68 == $betType) && 'N' == $arr_config['mini_service_v_soccer']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '가상축구 점검중입니다.';
                    return $this->fail($response);
                } else if (15 == $betType && 'N' == $arr_config['mini_service_powerball']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '파워볼 배팅은 점검중입니다.';
                    return $this->fail($response);
                } else if ((50 == $betType || 70 == $betType || 71 == $betType) && 'N' == $arr_config['mini_service_v_horse']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '가상경마 점검중입니다.';
                    return $this->fail($response);
                } else if ((51 == $betType || 69 == $betType) && 'N' == $arr_config['mini_service_v_dog']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '가상개경주 점검중입니다.';
                    return $this->fail($response);
                } else if (52 == $betType && 'N' == $arr_config['mini_service_n_rpowerball']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '네임드 레드파워볼 점검중입니다.';
                    return $this->fail($response);
                } else if (53 == $betType && 'N' == $arr_config['mini_service_nxt_powerball']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '넥스트게임 파워볼 점검중입니다.';
                    return $this->fail($response);
                } else if ((54 == $betType || 55 == $betType || 56 == $betType) && 'N' == $arr_config['mini_service_sh_powerball']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 파워볼 점검중입니다.';
                    return $this->fail($response);
                } else if (57 == $betType && 'N' == $arr_config['mini_service_n_rpladder']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '네임드 레드사다리 점검중입니다.';
                    return $this->fail($response);
                } else if (58 == $betType && 'N' == $arr_config['mini_service_nxt_ladder']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '넥스트게임 사다리 점검중입니다.';
                    return $this->fail($response);
                } else if ((59 == $betType || 60 == $betType || 61 == $betType) && 'N' == $arr_config['mini_service_sh_ladder']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 사다리 점검중입니다.';
                    return $this->fail($response);
                } else if ((63 == $betType || 64 == $betType || 65 == $betType) && 'N' == $arr_config['mini_service_sh_race']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 레이싱 점검중입니다.';
                    return $this->fail($response);
                } else if (62 == $betType && 'N' == $arr_config['mini_service_nxt_bac']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '넥스트게임 바카라 점검중입니다.';
                    return $this->fail($response);
                }

                $str_sql_config_menu = "SELECT set_type, set_type_val 
                                    FROM t_game_config 
                                    WHERE set_type 
                                        IN('mini_menu_powerball',
                                            'mini_menu_eos_powerball',
                                            'mini_menu_pladder',
                                            'mini_menu_kladder',
                                            'mini_menu_v_soccer',
                                            'mini_menu_v_horse',
                                            'mini_service_v_dog',
                                            'mini_menu_n_rpowerball5',
                                            'mini_menu_nxt_powerball',
                                            'mini_service_sh_powerball',
                                            'mini_menu_sh_powerball1',
                                            'mini_menu_sh_powerball2',
                                            'mini_menu_sh_powerball3',
                                            'mini_menu_n_rpladder5',
                                            'mini_menu_nxt_ladder',
                                            'mini_menu_sh_ladder1',
                                            'mini_menu_sh_ladder2',
                                            'mini_menu_sh_ladder3',
                                            'mini_menu_nxt_bac',
                                            'mini_menu_sh_race1',
                                            'mini_menu_sh_race2',
                                            'mini_menu_sh_race3'
                                        )";
                $arr_config_menu_result = $tgcModel->db->query($str_sql_config_menu)->getResultArray();

                $arr_config_menu = array();
                foreach ($arr_config_menu_result as $key => $value) {
                    $arr_config_menu[$value['set_type']] = $value['set_type_val'];
                }

                if (3 == $betType && 'N' == $arr_config_menu['mini_menu_eos_powerball']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = 'EOS 파워볼 배팅은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (4 == $betType && 'N' == $arr_config_menu['mini_menu_pladder']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '파워사다리은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (5 == $betType && 'N' == $arr_config_menu['mini_menu_kladder']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '키노사다리는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (6 == $betType && 'N' == $arr_config_menu['mini_menu_v_soccer']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '가상축구는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (15 == $betType && 'N' == $arr_config_menu['mini_menu_powerball']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '파워볼 배팅은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (50 == $betType && 'N' == $arr_config_menu['mini_menu_v_horse']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '가상경마는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (51 == $betType && 'N' == $arr_config_menu['mini_menu_v_dog']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '가상개경주는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (52 == $betType && 'N' == $arr_config_menu['mini_menu_n_rpowerball5']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '네임드 레드파워볼은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (53 == $betType && 'N' == $arr_config_menu['mini_menu_nxt_powerball']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '넥스트게임 파워볼은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if ((54 == $betType) && 'N' == $arr_config_menu['mini_menu_sh_powerball1']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 파워볼1은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if ( 55 == $betType && 'N' == $arr_config_menu['mini_menu_sh_powerball2']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 파워볼2는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (56 == $betType && 'N' == $arr_config_menu['mini_menu_sh_powerball3']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 파워볼3은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (57 == $betType && 'N' == $arr_config_menu['mini_menu_n_rpladder5']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '네임드 레드사다리는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (58 == $betType && 'N' == $arr_config_menu['mini_menu_nxt_ladder']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '넥스트게임 사다리는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if ((59 == $betType) && 'N' == $arr_config_menu['mini_menu_sh_ladder1']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 사다리1은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (60 == $betType && 'N' == $arr_config_menu['mini_menu_sh_ladder2']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 사다리2는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (61 == $betType && 'N' == $arr_config_menu['mini_menu_sh_ladder3']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 사다리3은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if ((63 == $betType) && 'N' == $arr_config_menu['mini_menu_sh_race1']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 레이싱1은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (64 == $betType && 'N' == $arr_config_menu['mini_menu_sh_race2']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 레이싱2는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (65 == $betType && 'N' == $arr_config_menu['mini_menu_sh_race3']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '슈어맨 레이싱3은 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                } else if (62 == $betType && 'N' == $arr_config_menu['mini_menu_nxt_bac']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '넥스트게임 바카라는 현재 사용하실수 없습니다.';
                    return $this->fail($response);
                }
            }

            // 미니게임 설정
            $m_level = session()->get('level');
            $sql_config = "SELECT * FROM mini_game_bet_config 
                        WHERE level = $m_level 
                        AND bet_type = ?"; // 가상축구 설정값
            $result_config = $tgcModel->db->query($sql_config, [$betType])->getResult();
            foreach ($result_config as $key => $value) {
                $game_config = $value;
            }
            $fixture_id = $betList[0]['fixtureId'];
            $round = $betList[0]['round'];
            $marketsId = $betList[0]['marketsId'];
            if (!is_int((int) $fixture_id) || !is_int((int) $round) || !is_int((int) $marketsId) || !is_double((double) $betList[0]['betPrice'])) {
                $this->logger->critical("!!!!!!!!!!!!!!!!!! ***************** addMiniBet: die ");
                die();
            }
            $dt_current = date("Y-m-d H:i:s");

            $deadline = 'mini_powerball_deadline';

            // 가상축구이면 동일경기에 이미 베팅을 했는지 확인
            if (6 == $betType) {
                // 배팅가능 시간을 체크한다.
                $sql = "SELECT start_dt, end_dt, result, league FROM mini_game WHERE bet_type = 6 AND id = ?";
                $gameInfo = $tgcModel->db->query($sql, [$fixture_id])->getResultArray();
                $result = json_decode($gameInfo[0]['result'], true);

                // 현재 경기가져온다.
                $sql = "SELECT * FROM mini_game where bet_type = ? and start_dt > now() and league = ? order by start_dt asc limit 2;";
                $db_result = $memberModel->db->query($sql, [$betType, $gameInfo[0]['league']])->getResultArray();

                $deadline = 'mini_v_soccer_deadline';
                // 데드라인 시간 체크
                $sql = "SELECT * FROM t_game_config where set_type = ?;";
                $db_deadline = $memberModel->db->query($sql, [$deadline])->getResultArray();

                if (0 == count($db_result)) {
                    $response['messages'] = '경기정보가 없습니다.';
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }

                $current_game = array();
                foreach ($db_result as $key => $value) {
                    $current_game[] = $value['id'];
                }

                // 보내준 경기번호가 현재진행중인 경기인지 체크
                if (!in_array($fixture_id, $current_game)) {
                    $response['messages'] = '잘못된 경기입니다.';
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }

                // 경기시작 deadline 초전까지 가능
                $checkStartTime = date("Y-m-d H:i:s", strtotime($gameInfo[0]['start_dt'] . "-" . intval($db_deadline[0]['set_type_val']) . "seconds"));
                if ($checkStartTime < $dt_current) {
                    $response['messages'] = '베팅가능 시간을 초과하였습니다.';
                    $response['code'] = 1001;
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }

                $sql = "SELECT ls_markets_id, total_bet_money
                    FROM mini_game_member_bet 
                    WHERE member_idx = ?
                        AND round = ? 
                        AND bet_type = ? "; //for update";
                $betInfo = $tgcModel->db->query($sql, [$memberIdx, $round, $betType])->getResultArray();

                // 잘못된 마켓아이디인지 체크한다.
                if ($marketsId < 13001 || $marketsId > 13005) {
                    $response['messages'] = '잘못 된 배팅입니다.';
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }

                // 배당률을 디비걸로 셋팅한다.
                if ($marketsId == 13001) {
                    $betList[0]['betPrice'] = $result['win'];
                } else if ($marketsId == 13002) {
                    $betList[0]['betPrice'] = $result['draw'];
                } else if ($marketsId == 13003) {
                    $betList[0]['betPrice'] = $result['lose'];
                } else if ($marketsId == 13004) {
                    $betList[0]['betPrice'] = $result['lose'];
                } else {
                    $betList[0]['betPrice'] = $result['win'];
                }

                if (0 < count($betInfo)) {
                    $sql = "SELECT total_bet_money FROM mini_game_member_bet WHERE round = ? AND bet_type = ? AND ls_fixture_id = ? ";
                    $totalBetInfo = $tgcModel->db->query($sql, [$round, $betType, $fixture_id])->getResultArray();

                    $total_money = 0;
                    foreach ($totalBetInfo as $value) {
                        $total_money += $value['total_bet_money'];
                    }
                    // 같은 타입 배팅은 최대한도만 체크한다.(100만)
                    if ($marketsId == $betInfo[0]['ls_markets_id']) {
                        if ($total_money + $totalMoney > $game_config->max) {
                            $response['messages'] = '최대 배팅 및 당첨 상한이 초과되었습니다.';
                            $memberModel->db->transRollback();
                            return $this->fail($response);
                        }
                    }
                    
                    $response['messages'] = '이미 베팅한 경기입니다.';
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                } else { // 첫배팅도 한도 체크
                    if ($totalMoney > $game_config->max) {
                        $response['messages'] = '최대 배팅 및 당첨 상한이 초과되었습니다.';
                        $memberModel->db->transRollback();
                        return $this->fail($response);
                    }
                }
            } else if (50 == $betType || 51 == $betType){
                // 배팅가능 시간을 체크한다.
                $sql = "SELECT start_dt, end_dt, result, league FROM mini_game WHERE bet_type = ? AND id = ?";
                $gameInfo = $tgcModel->db->query($sql, [$betType, $fixture_id])->getResultArray();
                $result = json_decode($gameInfo[0]['result'], true);
                // 현재 경기가져온다.
                $sql = "SELECT * FROM mini_game where bet_type = ? and start_dt > now() and league = ? order by start_dt asc limit 1;";
                $db_result = $memberModel->db->query($sql, [$betType, $gameInfo[0]['league']])->getResultArray();
                if (0 == count($db_result)) {
                    $response['messages'] = '경기정보가 없습니다.';
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }
                $current_game = array();
                foreach ($db_result as $key => $value) {
                    $current_game[] = $value['id'];
                }

                if(50 == $betType){
                    $deadline = 'mini_v_horse_deadline';
                } else {
                    $deadline = 'mini_v_dog_deadline';
                }
                
                // 데드라인 시간 체크
                $sql = "SELECT * FROM t_game_config where set_type = ?;";
                $db_deadline = $memberModel->db->query($sql, [$deadline])->getResultArray();

                // 보내준 경기번호가 현재진행중인 경기인지 체크
                if (!in_array($fixture_id, $current_game)) {
                    $response['messages'] = '잘못된 경기입니다.';
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }

                // 경기시작 deadline초전까지 가능
                $checkStartTime = date("Y-m-d H:i:s", strtotime($gameInfo[0]['start_dt'] . "-" . intval($db_deadline[0]['set_type_val']) . "seconds"));
                if ($checkStartTime < $dt_current) {
                    $response['messages'] = '베팅가능 시간을 초과하였습니다.';
                    $response['code'] = 1001;
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }

                $sql = "SELECT ls_markets_id, total_bet_money,round, ls_fixture_id
                    FROM mini_game_member_bet 
                    WHERE member_idx = ?
                        AND round = ? 
                        AND bet_type = ? 
                        AND ls_fixture_id = ? "; //for update";
                $betInfo = $tgcModel->db->query($sql, [$memberIdx, $round, $betType, $fixture_id])->getResultArray();

                // 잘못된 마켓아이디인지 체크한다.
                if ($marketsId > 15) {
                    if(($marketsId < 15001 || $marketsId > 16005)){
                        $response['messages'] = '잘못 된 배팅입니다.';
                        $memberModel->db->transRollback();
                        return $this->fail($response);
                    } 
                    $sql = "SELECT bet_price FROM mini_game_bet WHERE markets_id = ?";
                    $betInfos = $tgcModel->db->query($sql, [$marketsId])->getResultArray();
                    // 배당률을 디비걸로 셋팅한다.
                
                    if($betInfos['bet_price'] > 0){
                        if ($marketsId == 15001) {
                            $betList[0]['betPrice'] = $betInfos['bet_price'];
                        } else if ($marketsId == 15002) {
                            $betList[0]['betPrice'] = $betInfos['bet_price'];
                        } else if ($marketsId == 15003) {
                            $betList[0]['betPrice'] = $betInfos['bet_price'];
                        } else if ($marketsId == 15004) {
                            $betList[0]['betPrice'] = $betInfos['bet_price'];
                        } else if ($marketsId == 15005) {
                            $betList[0]['betPrice'] = $betInfos['bet_price'];
                        }
                    }
                } 

                // $this->logger->info($round);
                // $this->logger->info($betInfo);
                if (0 < count($betInfo)) {
                    $sql = "SELECT total_bet_money FROM mini_game_member_bet WHERE round = ? AND bet_type = ? AND ls_fixture_id = ? ";
                    $totalBetInfo = $tgcModel->db->query($sql, [$round, $betType, $fixture_id])->getResultArray();

                    $total_money = 0;
                    foreach ($totalBetInfo as $value) {
                        $total_money += $value['total_bet_money'];
                    }
                    // 같은 타입 배팅은 최대한도만 체크한다.(100만)
                    if ($fixture_id != $betInfo[0]['ls_fixture_id']) {
                        if ($total_money + $totalMoney > $game_config->max) {
                            $response['messages'] = '최대 배팅 및 당첨 상한이 초과되었습니다.';
                            $memberModel->db->transRollback();
                            return $this->fail($response);
                        }
                    }
                    
                    $response['messages'] = '이미 베팅한 경기입니다.';
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                } else { // 첫배팅도 한도 체크
                    if ($totalMoney > $game_config->max) {
                        $response['messages'] = '최대 배팅 및 당첨 상한이 초과되었습니다.';
                        $memberModel->db->transRollback();
                        return $this->fail($response);
                    }
                }
            } else {
                // 배팅가능 시간을 체크한다.
                $sql = "SELECT end_dt, cnt FROM mini_game WHERE bet_type = ? AND id = ?";
                $gameInfo = $tgcModel->db->query($sql, [$betType, $fixture_id])->getResultArray();
                $cnt = $gameInfo[0]['cnt'];

                if ($round != $cnt) {
                    $response['messages'] = '배팅 정보가 잘못되었습니다. 새로고침해주세요.';
                    $response['code'] = 1001;
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }

                if(15 == $betType || 3 == $betType){
                    $deadline = 'mini_powerball_deadline';
                } else if(4 == $betType){
                    $deadline = 'mini_power_ladder_deadline';
                } else if(5 == $betType){
                    $deadline = 'mini_kino_ladder_deadline';
                } else if(52 == $betType){
                    $deadline = 'mini_n_rpowerball5_deadline';
                } else if(53 == $betType){
                    $deadline = 'mini_nxt_powerball_deadline';
                } else if(54 == $betType){
                    $deadline = 'mini_sh_powerball1_deadline';
                } else if(55 == $betType){
                    $deadline = 'mini_sh_powerball2_deadline';
                } else if(56 == $betType){
                    $deadline = 'mini_sh_powerball3_deadline';
                } else if(57 == $betType){
                    $deadline = 'mini_n_rpladder5_deadline';
                } else if(58 == $betType){
                    $deadline = 'mini_nxt_ladder_deadline';
                } else if(59 == $betType){
                    $deadline = 'mini_sh_ladder1_deadline';
                } else if(60 == $betType){
                    $deadline = 'mini_sh_ladder2_deadline';
                } else if(61 == $betType){
                    $deadline = 'mini_sh_ladder3_deadline';
                } else if(62 == $betType){
                    $deadline = 'mini_nxt_bac_deadline';
                } else if(63 == $betType){
                    $deadline = 'mini_sh_race1_deadline';
                } else if(64 == $betType){
                    $deadline = 'mini_sh_race2_deadline';
                } else if(65 == $betType){
                    $deadline = 'mini_sh_race3_deadline';
                }


                // 데드라인 시간 체크
                $sql = "SELECT * FROM t_game_config where set_type = ?;";
                $db_deadline = $memberModel->db->query($sql, [$deadline])->getResultArray();

                $checkStartTime = date("Y-m-d H:i:s", strtotime($gameInfo[0]['end_dt'] . "-" . intval($db_deadline[0]['set_type_val']) . "seconds"));

                if ($checkStartTime < $dt_current) {
                    $response['messages'] = '배팅가능 시간을 초과하였습니다.';
                    $response['code'] = 1001;
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }

                $sql = "SELECT ls_markets_id, total_bet_money,round, ls_fixture_id
                    FROM mini_game_member_bet 
                    WHERE member_idx = ?
                        AND round = ? 
                        AND bet_type = ? 
                        AND ls_fixture_id = ? "; //for update";
                $betInfo = $tgcModel->db->query($sql, [$memberIdx, $round, $betType, $fixture_id])->getResultArray();
                
                $sql = "SELECT total_bet_money FROM mini_game_member_bet WHERE round = ? AND bet_type = ? AND ls_fixture_id = ? ";
                $totalBetInfo = $tgcModel->db->query($sql, [$round, $betType, $fixture_id])->getResultArray();

                $total_money = 0;
                foreach ($totalBetInfo as $value) {
                    $total_money += $value['total_bet_money'];
                }
                if ($fixture_id != $betInfo[0]['ls_fixture_id']) {
                    if ($total_money + $totalMoney > $game_config->max) {
                        $response['messages'] = '최대 배팅 및 당첨 상한이 초과되었습니다.';
                        $memberModel->db->transRollback();
                        return $this->fail($response);
                    }
                } else {
                    $response['messages'] = '이미 베팅한 경기입니다.';
                    $memberModel->db->transRollback();
                    return $this->fail($response);
                }

                /* if ($round != $cnt) {

                  $response['messages'] = '배팅 정보가 잘못되었습니다. 새로고침해주세요.';

                  $response['code'] = 1001;
                  $memberModel->db->transRollback();
                  return $this->fail($response);
                  } */

                /* if ($gameInfo[0]['end_dt'] < $dt_current) {
                  $response['messages'] = '배팅가능 시간을 초과하였습니다.';
                  $response['code'] = 1001;
                  $memberModel->db->transRollback();
                  return $this->fail($response);
                  } */

                $sql = "SELECT ls_markets_id, total_bet_money

                    FROM mini_game_member_bet 
                    WHERE member_idx = ?
                        AND round = ? 
                        AND bet_type = ?
                        AND date(create_dt) >= date_format(now(), '%Y-%m-%d')";
                /* $betInfo = $tgcModel->db->query($sql, [$memberIdx, $round, $betType])->getResultArray();
                  if (0 < count($betInfo)) {
                  $response['messages'] = '이미 베팅한 경기입니다.';
                  return $this->fail($response);
                  } */
                // 배당률을 디비걸로 셋팅한다.(가상축구는 제외)
                $sql = "SELECT bet_price FROM mini_game_bet WHERE markets_id = ?";
                $betInfo = $tgcModel->db->query($sql, [$marketsId])->getResultArray();
                if ($betInfo[0]['bet_price'] > 0)
                    $betList[0]['betPrice'] = $betInfo[0]['bet_price'];
            }

            if($betType == 50 || $betType == 51){
                if($marketsId > 15000){
                    // 베팅정보를 가져온다.
                    $marketsId = $betList[0]['marketsId'];
                    $sql = "SELECT game, markets_name, bet_price FROM mini_game_bet WHERE markets_id = ?";
                    $miniBet = $tgcModel->db->query($sql, [$marketsId])->getResultArray();
                    if (0 == count($miniBet)) {
                        $memberModel->db->transRollback();
                        $response['messages'] = '배팅 정보가 잘못되었습니다. ';
                        return $this->fail($response);
                    }
                }
            } else {
                // 베팅정보를 가져온다.
                $marketsId = $betList[0]['marketsId'];
                $sql = "SELECT game, markets_name, bet_price FROM mini_game_bet WHERE markets_id = ?";
                $miniBet = $tgcModel->db->query($sql, [$marketsId])->getResultArray();
                if (0 == count($miniBet)) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '배팅 정보가 잘못되었습니다. ';
                    return $this->fail($response);
                }
            }

            $array_league_tag_id = [];

            $bonus_dividend = 1;
            $u_level = $member->getLevel();
            $sql = "SELECT * 
                    FROM mini_game_bet_config 
                    WHERE bet_type = ? 
                    AND level = ?";
            $mini_config_result = $tgcModel->db->query($sql, [$betType, $u_level])->getResultArray();

            if ($totalMoney > $mini_config_result[0]['max']) {
                $memberModel->db->transRollback();
                $response['messages'] = '최대 배팅 가능 금액은 ' . number_format($mini_config_result[0]['max']) . '원 입니다.';
                return $this->fail($response);
            }
            if ($totalMoney < $mini_config_result[0]['min']) {
                $memberModel->db->transRollback();
                $response['messages'] = '최소 배팅 금액은 ' . number_format($mini_config_result[0]['min']) . '원 입니다.';
                return $this->fail($response);
            }
            if ($mini_config_result[0]['limit'] < $totalMoney * $betList[0]['betPrice']) {
                $memberModel->db->transRollback();
                $response['messages'] = ' 배팅 상한 제한 금액은 ' . number_format($mini_config_result[0]['limit']) . '원 입니다.';
                return $this->fail($response);
            }

            // $this->logger->info($betType);
            // $this->logger->info($betList);
            $af_money = $member->getMoney() - $totalMoney;
            $miniGameMemberBetModel = new MiniGameMemberBetModel();
            $result = $miniGameMemberBetModel->addMemberMiniGameBet($member, $dt_current, $totalMoney, 0, $betType, $betList);

            if (0 == $result) {
                $memberModel->db->transRollback();
                return $this->fail('미니게임베팅 실패');
            }

            Calculate::updateChargeBetMoney($memberIdx, 'MINI', $totalMoney, $memberModel, $this->logger);

            UserPayBack::AddBetting($memberIdx, $totalMoney, $miniGameMemberBetModel);

            $keep_login_access_token = $this->tokenRefresh($member, $memberModel);
            $response['keep_login_access_token'] = $keep_login_access_token;

            $memberModel->db->transComplete();
            session()->set('money', $af_money);
        } catch (\mysqli_sql_exception $e) {
            $this->logger->error(':::::::::::::::  addMiniBet error : ' . $e->getMessage());
            $this->logger->error(':::::::::::::::  addMiniBet query : ' . $memberModel->getLastQuery());
            $memberModel->db->transRollback();
            return $this->fail('디비처리 실패로 인한 배팅 실패');
        }

        $response = [
            'code' => 200,
            'messages' => '배팅 성공',
            'data' => [
                'bet_id' => $result,
                'total_money' => $af_money,
                'total_bet_money' => $totalMoney,
                'total_point' => $af_point,
                'keep_login_access_token' => $keep_login_access_token
            ]
        ];

        return $this->respond($response, 200);
    }

    //ON/OFF 설정시 호출한다.
    public function setBettingSlip() {
        $is_betting_slip = isset($_POST['is_betting_slip']) ? $_POST['is_betting_slip'] : NULL; // 베팅슬립 on/off 값이다 .

        if (false == CodeUtil::only_alpha_number($is_betting_slip) || ('ON' != $is_betting_slip && 'OFF' != $is_betting_slip)) {
            $response['messages'] = '인자값이 잘못되었습니다.';
            $this->logger->error('is_betting_slip : ' . $is_betting_slip);
            return $this->fail($response);
        }

        $chkMobile = CodeUtil::rtn_mobile_chk();
        $viewRoot = "PC" == $chkMobile ? 'web' : 'web';

        $memberIdx = session()->get('member_idx');

        $memberModel = new MemberModel();
        $member = $memberModel->getMemberWhereIdx($memberIdx);

        if ($member == null) {
            $response['messages'] = '조회되는 유저가 없습니다.';
            return $this->fail($response);
        }

        $memberModel->set('is_betting_slip', $is_betting_slip)->where('idx', $memberIdx)->update();

        $response = [
            'code' => 200,
            'messages' => '배팅 성공',
            'is_betting_slip' => $is_betting_slip
        ];
        return $this->respond($response, 200);
    }

    private function checkBetListData($isBettingSlip, $totalOdds, $arr_config, $betType, $betList, $isClassic, $isSpecial, $memberModel) {

        $array_league_tag_id = [];
        $array_fixture = [];
        $betList_renew = []; // bet_name 추가 용도
        $overlap = []; // 조합중복 체크 용도
        $fDetail = '';
        $arr_bet_price = array();

        foreach ($betList as $value) {
            $fixtureId = $value['fixtureId'];
            $marketsId = $value['marketsId'];

            $betBaseLine = $value['betBaseLine'];
            $betName = $value['betName'];
            $fixtureStartDate = $value['fixture_start_date'];
            $leagueTagId = $value['leagueTagId'];

            $str_bet_query = Calculate::getMadeAddBetQuery();
            $bet_result = $memberModel->db->query($str_bet_query, [$betType, $fixtureId, $marketsId, $betBaseLine, $betName, $betType, $fixtureStartDate, $betType, $betType])->getResultArray();
            if (!isset($bet_result) || 0 == count($bet_result)) {
                //$memberModel->db->transRollback();
                $this->logger->error('query : ' . $str_bet_query . ' bet_result : ' . json_encode([$betType, $fixtureId, $marketsId, $betBaseLine, $betName, $betType, $fixtureStartDate, $betType, $betType]));
                $messages = '배팅 정보가 잘못되었습니다. ';
                //return $this->fail($response);

                return [false, $messages, $max_dividend, $totalOdds, $bet_count, $arr_bet_price, $array_fixture, $array_league_tag_id, $fDetail, $betList_renew];
            }

            $bet = $bet_result[0];
            $max_dividend = $bet['max_dividend'];
            $value['betId'] = $bet['bet_id'];
            $value['bet_name'] = $bet['bet_name'];
            $value['fixture_sport_id'] = $bet['fixture_sport_id'];
            $value['fixture_location_id'] = $bet['fixture_location_id'];
            $value['fixture_league_id'] = $bet['fixture_league_id'];
            $value['fixture_participants_1_id'] = $bet['fixture_participants_1_id'];
            $value['fixture_participants_2_id'] = $bet['fixture_participants_2_id'];
            $value['fixture_start_date'] = $fixtureStartDate;
            if (1 == $betType) {
                list($other_ls_bet_id, $other_bet_price, $find_bet_name) = $this->getOtherBetInfo($memberModel, $value['fixtureId'], $betType, $value['marketsId'], $value['bet_name'], $value['betBaseLine']);
                $value['other_ls_bet_id'] = $other_ls_bet_id;
                $value['other_bet_price'] = $other_bet_price;
                $value['other_bet_name'] = $find_bet_name;
                $this->logger->error('addMemberBet other_ls_bet_id :' . $other_ls_bet_id . ' other_bet_price : ' . $other_bet_price . ' other_bet_name : ' . $find_bet_name);
            }
            $betList_renew[] = $value;
            $overlap[$fixtureId][] = $value;

            if ('ON' == $isClassic && SOCCER == $value['fixture_sport_id'] && (WDL != $marketsId && OVER_UNDER != $marketsId && HANDICAP != $marketsId) ||
                    'ON' == $isClassic && BASKETBALL == $value['fixture_sport_id'] && (OVER_UNDER_OVERTIME != $marketsId && M_12_OVERTIME != $marketsId && HANDICAP_OVERTIME != $marketsId) ||
                    'ON' == $isClassic && BASEBALL == $value['fixture_sport_id'] && (OVER_UNDER_OVERTIME != $marketsId && M_12_OVERTIME != $marketsId && HANDICAP_OVERTIME != $marketsId) ||
                    'ON' == $isClassic && VOLLEYBALL == $value['fixture_sport_id'] && (OVER_UNDER != $marketsId && WL != $marketsId) ||
                    'ON' == $isClassic && UFC == $value['fixture_sport_id'] && (WL != $marketsId) ||
                    'ON' == $isClassic && ICEHOCKEY == $value['fixture_sport_id'] && (WDL != $marketsId && OVER_UNDER != $marketsId && HANDICAP != $marketsId) ||
                    'ON' == $isClassic && ESPORTS == $value['fixture_sport_id'] && (WL != $marketsId) ||
                    'ON' == $isSpecial && SOCCER == $value['fixture_sport_id'] && (FIRST_HALF_1X2 != $marketsId && FIRST_HALF_OVER_UNDER != $marketsId && FIRST_HALF_HANDICAP != $marketsId && Q1_1X2 != $marketsId) ||
                    'ON' == $isSpecial && ICEHOCKEY == $value['fixture_sport_id'] && (Q1_1X2 != $marketsId && Q1_OVER_UNDER != $marketsId) ||
                    'ON' == $isSpecial && BASKETBALL == $value['fixture_sport_id'] && (Q1_HANDICAP != $marketsId && Q1_OVER_UNDER != $marketsId && FIRST_HALF_HANDICAP != $marketsId && FIRST_HALF_OVER_UNDER != $marketsId) ||
                    'ON' == $isSpecial && BASEBALL == $value['fixture_sport_id'] && (Q5_HANDICAP != $marketsId && Q5_OVER_UNDER != $marketsId && Q1_OVER_UNDER != $marketsId && Q1_1X2 != $marketsId)
            ) {


                $this->logger->critical("!!!!!!!!!!!!!!!!!! ***************** checkBetListData: marketsId " . $marketsId);
                return [false, '배팅정보오류', $max_dividend, $totalOdds, $bet_count, $arr_bet_price, $array_fixture, $array_league_tag_id, $fDetail, $betList_renew, $overlap];
            }

            if (false == in_array($fixtureId, $array_fixture)) {
                $array_fixture[] = $fixtureId;
                $array_league_tag_id[] = $leagueTagId;
            }

            list($retval_bet, $message_bet, $fDetail, $totalOdds, $bet_count, $arr_bet_price) = $this->checkPerBettingData($bet, $arr_config, $totalOdds, $bet_count, $arr_bet_price, $isBettingSlip, $betType, $this->logger);
            if (false == $retval_bet) {
                //$memberModel->db->transRollback();
                //$messages = $message_bet;
                //return $this->fail($response);
                return [false, $message_bet, $max_dividend, $totalOdds, $bet_count, $arr_bet_price, $array_fixture, $array_league_tag_id, $fDetail, $betList_renew, $overlap];
            }
        } // end foreach betList

        return [true, 'success', $max_dividend, $totalOdds, $bet_count, $arr_bet_price, $array_fixture, $array_league_tag_id, $fDetail, $betList_renew, $overlap];
    }

    private function checkItemIdx($memberIdx, $itemId, $itemValue, $memberModel, $logger) {
        $item = $this->gmPt->getAvailableOneItemAtOrderbyCreate($memberModel, $memberIdx, $itemId, $itemValue);
        $itemIdx = 0;
        $comment = '';
        if (null != $item) {
            $itemIdx = $item[0]['idx'];
            if (1 == $item[0]['type']) {
                $comment = '환급패치 아이템 사용';
            } else if (2 == $item[0]['type']) {
                $comment = '배당패치 아이템 사용';
            } else {

                return [false, '아이템 정보 오류입니다.'];
            }
        }

        if (0 < $itemIdx) {
            $ukey = md5($memberIdx . strtotime('now'));
            $this->gmPt->useItem($ukey, $memberIdx, $itemId, $itemIdx, $comment, $memberModel, $logger);
        }

        return [true, 'success'];
    }

    private function checkMinMaxBetMoney($totalOdds, $bonus_dividend, $totalMoney, $betType, $isClassic, $isSpecial, $member, $tgcModel) {
        $config = $tgcModel->getMemberLevelConfig($betType, $member->getLevel());

        //$betGroup = $isClassic == 'ON' ? 'pre' : 'real';
        if (1 == $betType) {
            $betGroup = 'pre';
        } else if (2 == $betType) {
            $betGroup = 'real';
        }

        foreach ($config as $key => $item) {
            if ($item['set_type'] == $betGroup . '_max_money') {
                if ($totalMoney > $item['set_type_val']) {
                    //$memberModel->db->transRollback();
                    $messages = '최대 배팅 가능 금액은 ' . number_format($item['set_type_val']) . '원 입니다.';
                    //return $this->fail($response);
                    return [false, $messages];
                }
            }
            if ($item['set_type'] == $betGroup . '_min_money') {
                if ($totalMoney < $item['set_type_val']) {
                    //$memberModel->db->transRollback();
                    $messages = '최소 배팅 금액은 ' . number_format($item['set_type_val']) . '원 입니다.';
                    //return $this->fail($response);
                    return [false, $messages];
                }
            }

            if ($item['set_type'] == $betGroup . '_limit_money') {
                if ($item['set_type_val'] < $totalMoney * round($totalOdds * $bonus_dividend, 2)) {
                    //$memberModel->db->transRollback();
                    $messages = '최대 당첨 가능 금액은 ' . number_format($item['set_type_val']) . '원 입니다.';
                    //return $this->fail($response);
                    return [false, $messages];
                }
            }
        }

        return [true, 'success'];
    }

    private function checkLimitBetMoney($totalMoney, $betType, $betList, $array_fixture, $member, $isClassic, $isSpecial, $memberModel) {

        $plcBetType = $betType;
        if (GT_SPORTS == $betType && 'ON' == $isClassic) {
            $plcBetType = GT_CLASSIC;
        } else if (GT_SPORTS == $betType && 'ON' == $isSpecial) {
            $plcBetType = GT_SPECIAL;
        }

        $str_fixture = implode(',', $array_fixture);
        $str_league_query = "   SELECT 
                                        fix.fixture_id,
                                        IFNULL(fix_bet.sum_bet_money, 0) AS sum_bet_money,
                                        d_plc.amount,
                                        leagues.id as league_id 
                                    FROM lsports_fixtures as fix
                                    LEFT JOIN fixtures_bet_sum as fix_bet 
                                        ON fix_bet.fixture_id = fix.fixture_id 
                                        AND fix_bet.bet_type = fix.bet_type 
                                        AND fix_bet.member_idx = ?
                                    LEFT JOIN lsports_leagues as leagues 
                                        ON fix.fixture_league_id = leagues.id
                                        AND leagues.bet_type = ?
                                    LEFT JOIN dividend_policy as d_plc 
                                        ON leagues.dividend_rank = d_plc.rank 
                                        AND d_plc.type = ? AND d_plc.level = ? 
                                    WHERE fix.fixture_id IN ($str_fixture) 
                                    AND   fix.bet_type = ? ;";
        $arr_fix_bet_sum_amount = $memberModel->db->query($str_league_query, [$member->getIdx(), $betType, $plcBetType, $member->getLevel(), $betType])->getResultArray();

        $sql = array();

        if (0 < count($arr_fix_bet_sum_amount)) {
            foreach ($arr_fix_bet_sum_amount as $bet_sum_value) {
                if ($bet_sum_value['sum_bet_money'] + $totalMoney <= $bet_sum_value['amount']) {
                    $insertSql = '('
                            . $member->getIdx() . ', '
                            . $bet_sum_value['fixture_id'] . ', "'
                            . $plcBetType . '", '
                            . $totalMoney . ')';
                    array_push($sql, $insertSql);
                    continue;
                }
                //$memberModel->db->transRollback();
                $messages = '경기당 최대 배팅 가능 금액은 ' . number_format($bet_sum_value['amount']) . '원 입니다.';
                //return $this->fail($response);
                return [false, $messages, []];
            }
        } else {
            foreach ($betList as $value) {
                $insertSql = '('
                        . $member->getIdx() . ', '
                        . $value['fixtureId'] . ', "'
                        . $plcBetType . '", '
                        . $totalMoney . ')';
                array_push($sql, $insertSql);
            }
        }

        return [true, 'success', $sql];
    }

    private function checkOverLap($overlap) {
        foreach ($overlap as $item) {
            $fixture_sport_id = $item[0]['fixture_sport_id'];
            if ($fixture_sport_id == BASEBALL || $fixture_sport_id == BASKETBALL || $fixture_sport_id == ICEHOCKEY) {
                $count = count($item);
                if ($count == 2) {
                    if ($item[0]['marketsId'] == $item[1]['marketsId']) {
                        $this->logger->debug('combine count error sport : ' . $fixture_sport_id . ' count : ' . $count . 'marketsId : ' . $item[0]['marketsId'] . 'marketsId_2 : ' . $item[1]['marketsId']);
                        //$memberModel->db->transRollback();
                        $messages = '동일한 베팅 조합입니다.';
                        //return $this->fail($response);
                        return [false, $messages];
                    }
                }

                if (($count > 1 && $count % 2 != 0) || 2 < $count) {
                    $this->logger->debug('combine count error sport : ' . $fixture_sport_id . ' count : ' . $count);
                    //$memberModel->db->transRollback();
                    $messages = '조합이 잘못되었습니다.';
                    //return $this->fail($response);
                    return [false, $messages];
                }
            }
        }

        return [true, 'success'];
    }

    private function checkPerBettingData($bet, $arr_config, $totalOdds, $bet_count, $arr_bet_price, $isBettingSlip, $betType, $logger) {
        $isLimitFolder = false;
        if ('OFF' == $bet['admin_bet_status'] || 'OFF' == $bet['fix_admin_bet_status']) {
            $messages = '선택 경기중 마감 된 경기가 있습니다. 재 배팅 부탁드립니다.';
            return [false, $messages, '', $totalOdds, $bet_count, $arr_bet_price];
        }

        if (4 == $bet['fixture_status']) {
            $messages = '해당 경기는 취소 된 경기입니다.';
            return [false, $messages, '', $totalOdds, $bet_count, $arr_bet_price];
        }

        $totalOdds *= $bet['bet_price'];
        // 1.15이하의 배당은 보너스배당 카운트 제외
        if ($arr_config['limit_folder_bonus'] < $bet['bet_price']) {
            $bet_count += 1;
        }

        $fDetail = '';
        if (1 == $betType) {
            $fDetail = 'prematch ==>';
        } else if (2 == $betType) {
            $fDetail = 'inplay ==>';
        }
        $fDetail .= ' [' . $bet['fixture_sport_name'] . '] ' . $bet['fixture_location_name'] . ' ' . $bet['fixture_league_name'] . ' ' . $bet['fixture_participants_1_name'] . ' VS ' . $bet['fixture_participants_2_name'];

        // 배당률을 넣어준다.
        $arr_bet_price[$bet['bet_id']] = $bet['bet_price'];

        //if ('OFF' == $isBettingSlip) {
            if (0 < $bet['limit_bet_price'] && $bet['bet_price'] < $bet['limit_bet_price']) {
                $messages = '선택 경기중 마감 된 경기가 있습니다. 재 배팅 부탁드립니다.';
                return [false, $messages, $fDetail, $totalOdds, $bet_count, $arr_bet_price];
            }

            if (0 < $bet['max_bet_price'] && $bet['max_bet_price'] < $bet['bet_price']) {
                $messages = '선택 경기중 마감 된 경기가 있습니다. 재 배팅 부탁드립니다.';
                return [false, $messages, $fDetail, $totalOdds, $bet_count, $arr_bet_price];
            }
        //}

        $startTime = date("Y-m-d H:i:s", strtotime($bet['fixture_start_date'] . "-" . 10 . " minutes"));
        $currentTime = date('Y-m-d H:i:s');

        if (1 != $bet['bet_status']) {
            if (2 == $betType || (1 == $betType && $currentTime < $startTime)) {
                $messages = '해당 유형의 배팅은 현재 불가합니다.';
                return [false, $messages, $fDetail, $totalOdds, $bet_count, $arr_bet_price];
            }
        }

        if (1 == $betType) {
            $dt_current = date("Y-m-d H:i:s");
            $end_time = '-' . $bet['end_time'];

            $dt_start = $bet['fixture_start_date'];
            $dec_time = $end_time . " minutes";
            $dt_end = date("Y-m-d H:i:s", strtotime($dt_start . $dec_time));

            if ($dt_end < $dt_current || $dt_start <= $dt_current) {
                $messages = '배팅 시간을 초과하였습니다.';
                return [false, $messages, $fDetail, $totalOdds, $bet_count, $arr_bet_price];
            }
        } else if (2 == $betType) {

            $data_object = json_decode($bet['livescore']);
            $period_id = $bet['not_display_period'];

            if (BET_OPEN != BetDataUtil::exp_score_filter($period_id, $data_object, $bet, $logger)) {
                $messages = '해당 유형의 배팅은 현재 불가합니다.!!!';
                return [false, $messages, $fDetail, $totalOdds, $bet_count, $arr_bet_price];
            }

            //$retval = BetDataUtil::checkDisplayMarkets($mergeBet, $logger);
            if (BET_OPEN != BetDataUtil::checkDisplayMarkets($bet, $logger)) {
                $messages = '해당 유형의 배팅은 현재 불가합니다.!!!';
                return [false, $messages, $fDetail, $totalOdds, $bet_count, $arr_bet_price];
            }
        }

        return [true, 'success', $fDetail, $totalOdds, $bet_count, $arr_bet_price];
    }

    private function checkIndividual($memberIdx, $betType, $betList, $isClassic, $isSpecial, $memberModel) {
        $betCount = count($betList);
        $folderType = 1 < $betCount ? 'D' : 'S';

        $game_type = array();
        if (1 == $betType) {

            if ('ON' == $isClassic) {
                $game_type[] = LGTB_CLASSIC;
            } else if ('ON' == $isSpecial) {
                $game_type[] = LGTB_SPECIAL;
            } else {
                $game_type[] = LGTB_SPORTS;

                if ('S' == $folderType) {
                    $game_type[] = LGTB_SPORTS_SINGLE_FOLDER;
                } else if (2 == $betCount) {
                    $game_type[] = LGTB_SPORTS_TWO_FOLDER;
                }
            }
        } else {
            $game_type[] = LGTB_REALTIME;
            if ('S' == $folderType) {
                $game_type[] = LGTB_REALTIME_SINGLE_FOLDER;
            }
        }

        $game_type = implode(',', $game_type);
        $game_type_sql = "SELECT game_type, status FROM member_game_type where member_idx = ? and game_type in ($game_type)";
        $arr_db_result_game_type = $memberModel->db->query($game_type_sql, [$memberIdx])->getResultArray();
        $this->logger->debug('checkIndividual : ' . $game_type_sql);
        foreach ($arr_db_result_game_type as $result_game_type) {
            if ('OFF' == $result_game_type['status']) {
                if (LGTB_SPORTS_TWO_FOLDER == $result_game_type['game_type']) {
                    $messages = '2폴더 게임 배팅은 금지되어있습니다.';
                } else {
                    $messages = '해당 베팅 이용불가합니다.' . PHP_EOL . '자세한 안내는 고객센터로 문의바랍니다.';
                }

                return [false, $messages, $folderType];
            }
        }

        return [true, 'success', $folderType];
    }

    private function checkSameTimeBetting($memberIdx, $memberModel) {
        $currentDate = date("Y-m-d H:i:s");
        $bet_sql = "SELECT count(*) as cnt FROM member_bet where member_idx = ? and create_dt = ? for update";
        $result = $memberModel->db->query($bet_sql, [$memberIdx, $currentDate])->getResultArray();

        if ($result[0]['cnt'] > 0) {

            return false;
        }
        return true;
    }

    private function checkMemberStatus($memberIdx, $memberModel) {
        //$memberModel = new MemberModel();
        $member = $memberModel->getMemberWhereIdx($memberIdx);

        if ($member == null) {
            $messages = '조회되는 유저가 없습니다.';
            return [false, $messages, $member, $memberIdx, $memberModel];
        }

        if ($member->getStatus() == 2 || $member->getStatus() == 3) {
            $messages = '사용이 불가능 한 계정으로 관리자에게 문의바랍니다.';
            return [false, $messages, $member, $memberIdx, $memberModel];
        }

        if ($member->getStatus() == 11) {
            $messages = '관리자 승인이 필요합니다.';
            return [false, $messages, $member, $memberIdx, $memberModel];
        }

        if ($member->getUBusiness() != 1) {
            $messages = '총판은 배팅 이용이 불가능합니다.';
            return [false, $messages, $member, $memberIdx, $memberModel];
        }

        return [true, 'success', $member, $memberIdx];
    }

    // 배팅한 반대 배팅정보를 구한다.(승패, 핸디캡, 오버언더)
    public function getOtherBetInfo($model, $fixture_id, $bet_type, $markets_id, $bet_name, $ls_markets_base_line) {
        $arr_winlose = array(52, 226);
        $arr_handicap = array(3, 342);
        $arr_overunder = array(2, 28);
        /* $arr_winlose = array(52,63,226);
          $arr_handicap = array(3,53,64,95,281,342);
          $arr_overunder = array(2,11,21,28,30
          ,31,77,101,102,153
          ,155,220,221,236); */

        $find_bet_name = '';
        $find_data = false;
        $other_ls_bet_id = 0;
        $other_bet_price = 0.0;
        if (true == in_array($markets_id, $arr_winlose) || true == in_array($markets_id, $arr_handicap)) {
            if ($bet_name == '1') {
                $find_bet_name = '2';
            } else {
                $find_bet_name = '1';
            }
            $find_data = true;
        } else if (true == in_array($markets_id, $arr_overunder)) {
            if ($bet_name == 'Over') {
                $find_bet_name = 'Under';
            } else {
                $find_bet_name = 'Over';
            }
            $find_data = true;
        }

        if (true == $find_data) {
            if (true == in_array($markets_id, $arr_winlose)) {
                $sql = "SELECT bet_id, bet_price FROM lsports_bet "
                        . "where fixture_id = ? and bet_type = ? and markets_id = ? and bet_name = ?";
                $other_bet_result = $model->db->query($sql, [$fixture_id, $bet_type, $markets_id, $find_bet_name])->getResultArray();
            } else {
                $sql = "SELECT bet_id, bet_price FROM lsports_bet "
                        . "where fixture_id = ? and bet_type = ? and markets_id = ? and bet_name = ? and bet_base_line = ?;";
                $other_bet_result = $model->db->query($sql, [$fixture_id, $bet_type, $markets_id, $find_bet_name, $ls_markets_base_line])->getResultArray();
            }

            if ($other_bet_result == null) {
                $this->logger->error('getOtherBetInfo fixture_id : ' . $fixture_id .
                        ' bet_type : ' . $bet_type . ' markets_id : ' . $markets_id .
                        ' bet_name : ' . $bet_name . ' ls_markets_base_line: ' . $ls_markets_base_line);
            }

            $other_ls_bet_id = $other_bet_result[0]['bet_id'];
            $other_bet_price = $other_bet_result[0]['bet_price'];
        }

        return array($other_ls_bet_id, $other_bet_price, $find_bet_name);
    }

    public function getLastBettingInfo($memberModel, $memberIdx) {
        $sql = "SELECT betting_dt FROM member_extend where member_idx = ?";

        return $memberModel->db->query($sql, [$memberIdx])->getResultArray();
    }

    public function tokenRefresh($findMember, $memberModel) {
        $keep_login_access_token = CodeUtil::uuidgen4();

        session()->set('keep_login_access_token', $keep_login_access_token);

        $memberModel->set('keep_login_access_token', $keep_login_access_token)
                ->where('idx', $findMember->getIdx())
                ->update();

        return $keep_login_access_token;
    }

    private function transProcess($member, $totalOdds, $totalMoney, $betType
        , $folderType, $bonus_dividend, $betList_renew, $arr_bet_price, $fDetail
        , $isBettingSlip, $itemIdx, $isClassic, $betDetailKey, $memberBetModel) {

        try {
            $memberBetModel->db->transStart();

            // 동일시간 베팅한 내용체크
            $revtal_same_time = $this->checkSameTimeBetting($member->getIdx(), $memberBetModel);
            if (false == $revtal_same_time) {
                $memberBetModel->db->transRollback();

                return [false, '동일시간 베팅이 있습니다.'];
            }

            $currentDate = date("Y-m-d H:i:s");
            $add_index = $memberBetModel->addMemberBet($currentDate, $member, $totalOdds, $totalMoney, $betType
                    , $folderType, $bonus_dividend, $betList_renew, $arr_bet_price, $fDetail, $isBettingSlip
                    , $itemIdx, $isClassic, $betDetailKey);

            if (0 == $add_index) {
                $memberBetModel->db->transRollback();
                return [false, '배팅 정보 삽입 실패'];
            }

            $memberBetModel->db->transComplete();
            return [true, 'success'];
        } catch (\mysqli_sql_exception $e) {
            $memberBetModel->db->transRollback();
            $this->logger->error(':::::::::::::::  transProcess error : ' . $e->getMessage());
            $this->logger->error(':::::::::::::::  transProcess query : ' . $memberBetModel->getLastQuery());
            return [false, '디비처리 실패로 인한 배팅 실패.'];
        }
    }

    // is_betting_slip OFF 일때 호출한다.
    public function doAddBet($betList, $betType, $totalMoney, $itemId, $itemIdx, $itemValue, $isBettingSlip, $isClassic, $betGroup, $folderType, $keep_login_access_token, $memberModel) {
        try {
            // 회원 상태값 체크
            list($revtal, $message) = $this->checkBettingAccessToken($keep_login_access_token, $memberModel);
            if (false == $revtal) {
                $response['messages'] = $message;
                return [false, $response['messages']];
            }

            list($revtal, $message, $member, $memberIdx) = $this->checkMemberStatus(session()->get('member_idx'), $memberModel);
            if (false == $revtal) {
                $response['messages'] = $message;
                return [false, $response['messages']];
            }

            if ($member->getMoney() < $totalMoney) {
                $response['messages'] = '배팅 금액이 부족합니다.';
                return $this->fail($response);
            }

            // Commercial ip check
            list($retVal, $message) = CodeUtil::checkCommercialIp($member, $this->logger);
            if (false == $retVal) {
                $this->logger->error('checkCommercialIp');
                $response['messages'] = $message;
                return [false, $response['messages']];
            }

            // 개인별 베팅제한 체크
            list($revtal_in, $message_in, $folderType) = $this->checkIndividual($memberIdx, $betType, $betList, $isClassic, $isSpecial, $memberModel);
            if (false == $revtal_in) {
                $response['messages'] = $message_in;
                return [false, $response['messages']];
            }

            // 배팅 슬립 데이터를 가져온다.
            if ($isBettingSlip != $member->getIsBettingSlip()) {
                $response['messages'] = '배팅슬립 설정이 잘못 되어있습니다.';
                return [false, $response['messages']];
            }
            // 점검,보너스 배당률 정보도 가져온다.
            $tgcModel = new TGameConfigModel();
            $arr_config = $this->gmPt->getConfigData($tgcModel);

            // 스포츠 점검
            if ('Y' == $arr_config['service_sports'] && GT_SPORTS == $betType && 'OFF' == $isClassic && GT_SPORTS == $betType && 'OFF' == $isSpecial && 9 != $member->getLevel()) {
                $response['messages'] = '스포츠 점검중입니다.';
                return [false, $response['messages']];
            }// 실시간 점검
            else if ('Y' == $arr_config['service_real'] && GT_REALTIME == $betType && 9 != $member->getLevel()) {
                $response['messages'] = '실시간 점검중입니다.';
                return [false, $response['messages']];
            }// 클래식 점검
            else if ('Y' == $arr_config['service_classic'] && GT_SPORTS == $betType && 'ON' == $isClassic && 9 != $member->getLevel()) {
                $response['messages'] = '클래식 점검중입니다.';
                return [false, $response['messages']];
            } //  스페셜 점검
            else if ('Y' == $arr_config['service_special'] && GT_SPORTS == $betType && 'ON' == $isSpecial && 9 != $member->getLevel()) {
                $response['messages'] = '스페셜 점검중입니다.';
                return [false, $response['messages']];
            }

            // 중복베팅 체크
            $lastBetInfo = $this->getLastBettingInfo($memberModel, $member->getIdx()); // 
            $checkStartTime = date("Y-m-d H:i:s", strtotime($lastBetInfo[0]['betting_dt'] . "+" . 5 . "seconds"));
            $currentTime = date('Y-m-d H:i:s');

            if (0 < count($lastBetInfo) && $currentTime < $checkStartTime) {
                //$sql = "UPDATE member set status = 11 WHERE idx = ? ";
                $sql = "UPDATE member set is_monitor_bet = 'Y' WHERE idx = ? ";
                $memberModel->db->query($sql, [$memberIdx]);

                $response['messages'] = '정상배팅이 아닙니다.!';
                return [false, $response['messages']];
            }


            $arr_bet_price = null;
            $max_dividend = 100;
            $bet_count = 0;

            // $betList 에서 fixtureId 값들을 추출한다.
            $totalOdds = 1;

            list($retval_betlist, $message_betlist, $max_dividend, $totalOdds, $bet_count, $arr_bet_price, $array_fixture, $array_league_tag_id, $fDetail, $betList_renew, $overlap) = $this->checkBetListData($isBettingSlip, $totalOdds, $arr_config, $betType, $betList, $isClassic, $isSpecial, $memberModel);

            if (false == $retval_betlist) {
                $response['messages'] = $message_betlist;
                return [false, $response['messages']];
            }

            if (count($betList_renew) != count($betList)) {
                //$memberModel->db->transRollback();
                $response['messages'] = '데이터 검증이 잘못되었습니다.';
                return [false, $response['messages']];
            }

            $bonus_dividend = 1;
            $total_bet_price = 1;
            list($total_bet_price, $bonus_dividend) = $this->gmPt->calBonusPrice($total_bet_price, $bonus_dividend, $folderType, $bet_count, $arr_config, $betType, $isClassic);

            // 제한베당이 하나라도 있으면 보너스 적용을 하지 않는다.(조선)
            if ($this->gmPt->isLimitFolder($betList_renew, $arr_config['limit_folder_bonus'])) {
                $bonus_dividend = 1;
            }

            // 야구, 농구, 아이스하키면 조합체크
            list($retval_overlap, $message_overlap) = $this->checkOverLap($overlap);
            if (false == $retval_overlap) {
                //$memberModel->db->transRollback();
                $response['messages'] = $message_overlap;
                return [false, $response['messages']];
            }

            // 배당제한 체크
            if ($max_dividend < $totalOdds * $bonus_dividend) {
                //$memberModel->db->transRollback();
                $this->logger->debug('max_dividend : ' . $max_dividend . ' totalOdds : ' . $totalOdds . ' bonus_dividend : ' . $bonus_dividend);
                $response['messages'] = '최고 배당 이상을 가져갈수 없습니다. ';
                return [false, $response['messages']];
            }

            // 리그 최대 배팅금액도 가져온다.

            list($retval_limit, $message_limit, $sql) = $this->checkLimitBetMoney($totalMoney, $betType, $betList, $array_fixture, $member, $isClassic, $isSpecial, $memberModel);

            if (false == $retval_limit) {
                //$memberModel->db->transRollback();
                $response['messages'] = $message_limit;
                return [false, $response['messages']];
            }

            if (0 < $itemIdx) {
                $ukey = md5($memberIdx . strtotime('now'));
                $this->gmPt->useItem($ukey, $memberIdx, $itemId, $itemIdx, $comment, $memberModel, $this->logger);
            }

            $memberModel->db->query(
                    'INSERT INTO `fixtures_bet_sum` ('
                    . 'member_idx, '
                    . 'fixture_id, '
                    . 'bet_type, '
                    . 'sum_bet_money) VALUES '
                    . implode(',', $sql)
                    . ' ON DUPLICATE KEY UPDATE '
                    . 'sum_bet_money = sum_bet_money + VALUES(sum_bet_money)'
            );

            // 유저 레벨별 배팅 최소,최대 금액을 체크한다.
            list($retval_min_max, $message_min_max) = $this->checkMinMaxBetMoney($totalOdds, $bonus_dividend, $totalMoney, $betType, $isClassic, $isSpecial, $member, $tgcModel);
            if (false == $retval_min_max) {
                //$memberModel->db->transRollback();
                $response['messages'] = $message_min_max;
                return [false, $response['messages']];
            }

            // 해당 아이템 번호가 유효하면 
            list($retval_item, $message_item) = $this->checkItemIdx($memberIdx, $itemId, $itemValue, $memberModel, $this->logger);
            if (false == $retval_item) {
                //$memberModel->db->transRollback();
                $response['messages'] = $message_item;
                return [false, $response['messages']];
            }

            $memberBetModel = new MemberBetModel();
            list($result, $errorMessage) = $this->transProcess($member, $totalOdds, $totalMoney, $betType
                    , $folderType, $bonus_dividend, $betList_renew, $arr_bet_price, $fDetail, $isBettingSlip
                    , $itemIdx, $isClassic, $arr_config['bet_detail_key'], $memberBetModel);

            if (false == $result) {
                $response['messages'] = $errorMessage;
                return [false, $response['messages']];
            }

            $type = '';
            $bet_type_string = '';
            if (GT_SPORTS == $betType && 'OFF' == $isClassic && 'S' == $folderType) {
                $type = 'SPORTS_S';
                $bet_type_string = 'prematch_s';
            } else if (GT_SPORTS == $betType && 'OFF' == $isClassic && 'D' == $folderType) {
                $type = 'SPORTS_D';
                $bet_type_string = 'prematch_d';
            } else if (GT_SPORTS == $betType && 'OFF' == $isSpecial && 'S' == $folderType) {
                $type = 'SPORTS_S';
                $bet_type_string = 'prematch_s';
            } else if (GT_SPORTS == $betType && 'OFF' == $isSpecial && 'D' == $folderType) {
                $type = 'SPORTS_D';
                $bet_type_string = 'prematch_d';
            } else if (GT_REALTIME == $betType && 'S' == $folderType) {
                $type = 'REAL_S';
                $bet_type_string = 'inplay_s';
            } else if (GT_REALTIME == $betType && 'D' == $folderType) {
                $type = 'REAL_D';
                $bet_type_string = 'inplay_d';
            } else if (GT_SPORTS == $betType && 'ON' == $isClassic && 'S' == $folderType) {
                $type = 'CLASSIC_S';
                $bet_type_string = 'classic_s';
            } else if (GT_SPORTS == $betType && 'ON' == $isClassic && 'D' == $folderType) {
                $type = 'CLASSIC_D';
                $bet_type_string = 'classic_d';
            } else if (GT_SPORTS == $betType && 'ON' == $isSpecial && 'S' == $folderType) {
                $type = 'SPECIAL_S';
                $bet_type_string = 'special_s';
            } else if (GT_SPORTS == $betType && 'ON' == $isSpecial && 'D' == $folderType) {
                $type = 'SPECIAL_D';
                $bet_type_string = 'special_d';
            }

            Calculate::updateChargeBetMoney($memberIdx, $type, $totalMoney, $memberModel, $this->logger);
            UserPayBack::AddBetting($memberIdx, $totalMoney, $memberModel);

            $this->logger->error(':::::::::::::::  addBet 1 error : ' . json_encode($response));
            //UserBettingLose::RewardBettingComps($memberIdx, $bet_type_string, $totalMoney, $this->logger, $memberModel);
            $af_money = $member->getMoney() - $totalMoney;
            session()->set('money', $af_money);

            // 실시간이면 키갱신을 하지 않는다.
            if(1 == $betType){
                $keep_login_access_token = $this->tokenRefresh($member, $memberModel);
                $response['keep_login_access_token'] = $keep_login_access_token;
            }else{
                $response['keep_login_access_token'] = session()->get('member_idx');
            }
        } catch (\mysqli_sql_exception $e) {
            $this->logger->error(':::::::::::::::  addMemberBet error : ' . $e->getMessage());
            $this->logger->error(':::::::::::::::  addMemberBet query : ' . $memberModel->getLastQuery());
            //$memberModel->db->transRollback();
            $response['messages'] = '디비처리 실패로 인한 배팅 실패';
            return [false, $response['messages']];
        }

        return [true, $response['messages'], $add_index, $af_money, $totalMoney, $array_league_tag_id, $keep_login_access_token];
    }

    // is_betting_slip OFF 일때만 호출한다. 
    public function doCheckBet($betType, $betList, $member, $memberModel) {

        try {
            $array_league_tag_id = [];
            $arr_bet_price = null;
            $max_dividend = 100;
            //$memberModel->db->transStart();

            $return_data = array();
            foreach ($betList as $value) {
                // bet_id 값으로 배팅 
                $str_bet_query = "SELECT    bet.bet_id,
                                        IF('ON' = bet.passivity_flag AND bet.bet_price_passivity is NOT NULL ,bet.bet_price_passivity,bet.bet_price) as bet_price,
                                        IF('ON' = bet.passivity_flag AND bet.bet_status_passivity is NOT NULL ,bet.bet_status_passivity,bet.bet_status) as bet_status,
                                        IF('ON' = fix.passivity_flag AND fix.fixture_start_date_passivity is NOT NULL ,fix.fixture_start_date_passivity,fix.fixture_start_date) as fixture_start_date,
                                        markets.limit_bet_price,
                                        markets.max_bet_price
                                    FROM lsports_bet as bet 
                                    LEFT JOIN lsports_markets as markets 
                                        ON bet.markets_id = markets.id
                                    LEFT JOIN lsports_fixtures as fix 
                                        ON bet.fixture_id = fix.fixture_id
                                    WHERE bet.bet_id = ? 
                                    AND bet.bet_type = ?
                                    AND markets.bet_group = ? 
                                    AND markets.is_delete = 0
                                    AND markets.sport_id = ? ;";

                $bet = $memberModel->db->query($str_bet_query, [$value['betId'], $betType, $betType, $value['betSportId']])->getResultArray();

                if (0 == count($bet)) {
                    $retval = array('retval' => -1, 'bet_id' => $value['betId']);
                    $return_data[$value['betId']] = $retval;
                    return [false, '배팅정보오류', $return_data];
                }
                $bet = $bet[0];

                $startTime = date("Y-m-d H:i:s", strtotime($bet['fixture_start_date'] . "-" . 10 . " minutes"));
                $currentTime = date('Y-m-d H:i:s');
                if (1 != $bet['bet_status']) {
                    if (2 == $betType || (1 == $betType && $currentTime < $startTime)) {
                        //$memberModel->db->transRollback();
                        $response['messages'] = '선택 경기중 마감 된 경기가 있습니다. 재 배팅 부탁드립니다.';
                        //return $this->fail($response);
                        $retval = array('retval' => -2, 'bet_id' => $value['betId']);
                        $return_data[$value['betId']] = $retval;
                        return [false, $response['messages'], $return_data];
                    }
                }

                if (0 < $bet['limit_bet_price'] && $bet['bet_price'] < $bet['limit_bet_price']) {
                    $memberModel->db->transRollback();
                    $response['messages'] = '선택 경기중 마감 된 경기가 있습니다. 재 배팅 부탁드립니다.';
                    //return $this->fail($response);
                    $retval = array('retval' => -2, 'bet_id' => $value['betId']);
                    $return_data[$value['betId']] = $retval;
                    return [false, $response['messages'], $return_data];
                }

                if (0 < $bet['max_bet_price'] && $bet['max_bet_price'] < $bet['bet_price']) {
                    //$memberModel->db->transRollback();
                    $response['messages'] = '선택 경기중 마감 된 경기가 있습니다. 재 배팅 부탁드립니다.';
                    //return $this->fail($response);

                    $retval = array('retval' => -2, 'bet_id' => $value['betId']);
                    $return_data[$value['betId']] = $retval;
                    return [false, $response['messages'], $return_data];
                }

                $retval = array('retval' => 1, 'bet_id' => $value['betId'], 'betPrice' => $bet['bet_price']);
                $return_data[$value['betId']] = $retval;
            }

            //$memberModel->db->transComplete();
            //session()->set('money', $af_money);
        } catch (\mysqli_sql_exception $e) {
            $this->logger->error(':::::::::::::::  checkBet error : ' . $e->getMessage());
            $this->logger->error(':::::::::::::::  checkBet query : ' . $memberModel->getLastQuery());
            //$memberModel->db->transRollback();
            $response['messages'] = '디비처리 실패로 인한 배팅 실패';
            return [false, $response['messages'], null];
        }

        return [true, 'success', $return_data];
    }

    public function checkBetting() {

        $betType = isset($_POST['betType']) ? $_POST['betType'] : NULL;
        $betList = isset($_POST['betList']) ? $_POST['betList'] : NULL;
        $arr_fixture = isset($_POST['fixture_id']) ? $_POST['fixture_id'] : '';

        $arr_check_fix = explode(',', $arr_fixture);
        foreach ($arr_check_fix as $fix) {
            $this->logger->error($fix);
            if (false == CodeUtil::only_number($fix)) {
                $response['messages'] = '경기번호가 잘못되었습니다.';
                return $this->fail($response);
            }
        }

        if (false == CodeUtil::only_number($betType) /* || false == CodeUtil::only_number($betId) || false == CodeUtil::only_number($betSportId) */ || $betList == NULL) {
            $response['messages'] = '경기 형식 인자값이 잘못되었습니다.';
            return $this->fail($response);
        }

        $betList = json_decode($betList, true);

        $memberIdx = session()->get('member_idx');
        $memberModel = new MemberModel();
        $member = $memberModel->getMemberWhereIdx($memberIdx);

        if ($member == null) {
            $response['messages'] = '조회되는 유저가 없습니다.';
            return $this->fail($response);
        }

        list($retval, $message, $return_data) = $this->doCheckBet($betType, $betList, $member, $memberModel);
        if (false == $retval) {
            $response['messages'] = $message;
            return $this->fail($response);
        }

        if (2 == $betType) {
            list($retval, $message) = $this->setRealTimeScore($arr_fixture);
            if (false == $retval) {
                $response['messages'] = $message;
                return $this->fail($response);
            }
        }

        $response = [
            'code' => 200,
            'messages' => '배팅 성공',
            'data' => [
                'betId' => $betId,
                'betData' => $return_data
            ]
        ];

        return $this->respond($response, 200);
    }
    
    public function checkBetting_renew() {
        $betType = isset($_POST['betType']) ? $_POST['betType'] : NULL;
        $betList = isset($_POST['betList']) ? $_POST['betList'] : NULL;
        $arr_fixture = isset($_POST['fixture_id']) ? $_POST['fixture_id'] : '';

        $this->logger->error('checkBetting_renew start : ');
        $arr_check_fix = explode(',', $arr_fixture);
        foreach ($arr_check_fix as $fix) {
            //$this->logger->error($fix);
            if (false == CodeUtil::only_number($fix)) {
                $response['messages'] = '경기번호가 잘못되었습니다.';
                $this->logger->error($response['messages']);
                return $this->fail($response);
            }
        }

        if (false == CodeUtil::only_number($betType) /* || false == CodeUtil::only_number($betId) || false == CodeUtil::only_number($betSportId) */ || $betList == NULL) {
            $response['messages'] = '경기 형식 인자값이 잘못되었습니다.';
            $this->logger->error($response['messages']);
            return $this->fail($response);
        }

        $betList = json_decode($betList, true);

        $memberIdx = session()->get('member_idx');
        $memberModel = new MemberModel();
        $member = $memberModel->getMemberWhereIdx($memberIdx);

        if ($member == null) {
            $response['messages'] = '조회되는 유저가 없습니다.';
            $this->logger->error($response['messages']);
            return $this->fail($response);
        }

        list($retval, $message, $return_data) = $this->doCheckBet($betType, $betList, $member, $memberModel);
        if (false == $retval) {
            $response['messages'] = $message;
            $this->logger->error($response['messages']);
            return $this->fail($response);
        }

        /*if (2 == $betType) {
            list($retval, $message) = $this->setRealTimeScore($arr_fixture);
            if (false == $retval) {
                $response['messages'] = $message;
                $this->logger->error($response['messages']);
                return $this->fail($response);
            }
        }*/
        
        if(1 == $betType){
            $response = [
                'code' => 200,
                'messages' => '배팅 성공',
                'data' => [
                    'betId' => $betId,
                    'betData' => $return_data
                ]
            ];
        }else{
            list($retval, $message) = $this->setRealTimeScore($arr_fixture);
            if (false == $retval) {
                $response['messages'] = $message;
                return $this->fail($response);
            }
            
            // betting delay
            $start_time = time(); // get the current time
            $duration = $this->getDelayTime($betList);
            $this->logger->error('delay time : '.$duration);
            $this->logger->error('start betiing time : '.date("Y-m-d H:i:s"));
            while (time() < $start_time + $duration) {
                // code to be executed goes here
            }
            //$end_time = time();
            $this->logger->error('end betiing time : '.date("Y-m-d H:i:s"));
            //$this->logger->error('betting gap : '.$end_time - $start_time);

            list($resultAddBet, $returnData) = $this->addBetting();
            if(!$resultAddBet){
                //$this->logger->error($returnData);
                $response['messages'] = $returnData;
                return $this->fail($response);
            }

            $response = $returnData;
        }

        return $this->respond($response, 200);
    }
    
    public function getDelayTime($betList) {
        // 베팅지연시간
        // 축구(6046), 아이스하키(35232), 농구(48242), 미식축구(131506), 배구(154830), 야구(154914)
        $time = $betSportId = 0;
        foreach ($betList as $value) {
            $betSportId = $value['betSportId'];
        }
        
        if(SOCCER == $betSportId){
            $time = config(App::class)->betDelayTime[SOCCER] / 1000;
        }else if(ICEHOCKEY == $betSportId){
            $time = config(App::class)->betDelayTime[ICEHOCKEY] / 1000;
        }else if(BASKETBALL == $betSportId){
            $time = config(App::class)->betDelayTime[BASKETBALL] / 1000;
        }else if(AMERICAN_FOOTBALL == $betSportId){
            $time = config(App::class)->betDelayTime[AMERICAN_FOOTBALL] / 1000;
        }else if(ESPORTS == $betSportId){
            $time = config(App::class)->betDelayTime[ESPORTS] / 1000;
        }else if(VOLLEYBALL == $betSportId){
            $time = config(App::class)->betDelayTime[VOLLEYBALL] / 1000;
        }else{
            $time = config(App::class)->betDelayTime[BASEBALL] / 1000;
        }
        
        return $time;
    }

    public function addBetting() {
        $betList = isset($_POST['betList']) ? $_POST['betList'] : NULL;
        $betType = isset($_POST['betType']) ? $_POST['betType'] : NULL;

        $totalMoney = isset($_POST['totalMoney']) ? $_POST['totalMoney'] : NULL;
        $itemId = isset($_POST['itemId']) ? $_POST['itemId'] : 0;
        $itemIdx = isset($_POST['itemId']) ? $_POST['itemId'] : 0;
        $itemValue = isset($_POST['itemValue']) ? $_POST['itemValue'] : 0;
        $isBettingSlip = isset($_POST['isBettingSlip']) ? $_POST['isBettingSlip'] : 'OFF';
        $isClassic = isset($_POST['isClassic']) ? $_POST['isClassic'] : 'OFF';
        $betGroup = isset($_POST['betGroup']) ? $_POST['betGroup'] : NULL;
        $folderType = isset($_POST['folderType']) ? $_POST['folderType'] : 'S';
        $keep_login_access_token = isset($_POST['keep_login_access_token']) ? $_POST['keep_login_access_token'] : '';
        
        if(2 == $betType)
            $betList = json_decode($betList, true);
        
        if (!CodeUtil::only_number($betType) ||
                (GT_SPORTS != $betType && GT_REALTIME != $betType ) ||
                !isset($betList) ||
                !CodeUtil::only_number($totalMoney) ||
                $totalMoney < 0 ||
                !CodeUtil::only_hangul($betGroup) ||
                !is_numeric($itemId) ||
                !is_numeric($itemValue) ||
                ('ON' != $isBettingSlip && 'OFF' != $isBettingSlip) ||
                ('ON' != $isClassic && 'OFF' != $isClassic) ||
                ('S' != $folderType && 'D' != $folderType)
        ) {
            $response['messages'] = '인자값이 잘못되었습니다.';

            $this->logger->error('$betType : ' . CodeUtil::only_number($betType) . ' $totalMoney : ' . CodeUtil::only_number($totalMoney) . ' $betGroup : ' . CodeUtil::only_hangul($betGroup));
            $this->logger->error('$isBettingSlip : ' . $isBettingSlip . ' $isClassic : ' . $isClassic . ' $folderTypev : ' . $folderType);

            if(1 == $betType)
                return $this->fail($response);
            else
                return array(false, $response['messages']);
        }

        if (false == session()->has('member_idx')) {
            $this->logger->error(__FUNCTION__ . ' not find member_idx : ' . session()->get('member_idx'));
            $response['messages'] = '유저를 찾을수 없습니다.';
            if(1 == $betType)
                return $this->fail($response);
            else
                return array(false, $response['messages']);
        }

        $member_idx = session()->get('member_idx');
        $memberModel = new MemberModel();

        // 실시간이면 스코어 체크
        if (2 == $betType) {
            list($retval, $message) = $this->checkRealTimeGameLiveScore_renew($betList, $member_idx);
            if (false == $retval) {
                $response['messages'] = $message;
                if(1 == $betType)
                    return $this->fail($response);
                else
                    return array(false, $response['messages']);
            }
        }
        
        $memberModel->db->transStart();
        list($retval, $message, $result, $af_money, $totalMoney, $array_league_tag_id, $keep_login_access_token) = $this->doAddBet($betList, $betType, $totalMoney
                , $itemId, $itemIdx, $itemValue, $isBettingSlip, $isClassic, $betGroup, $folderType, $keep_login_access_token, $memberModel);
        if (false == $retval) {
            $this->logger->error($message);
            $memberModel->db->transRollback();
            $response['messages'] = $message;
            if(1 == $betType)
                return $this->fail($response);
            else
                return array(false, $response['messages']);
        }
        $memberModel->db->transComplete();
        $response = [
            'code' => 200,
            'messages' => '배팅 성공',
            'data' => [
                'bet_id' => $result,
                'total_money' => $af_money,
                'total_bet_money' => $totalMoney,
                'arr_tag_ids' => $array_league_tag_id,
                'keep_login_access_token' => $keep_login_access_token
            ]
        ];
        
        if(1 == $betType)
            return $this->respond($response, 200);
        else
            return array(true, $response);
    }
    
    public function addBetting_old() {
        $betList = isset($_POST['betList']) ? $_POST['betList'] : NULL;
        $betType = isset($_POST['betType']) ? $_POST['betType'] : NULL;

        $totalMoney = isset($_POST['totalMoney']) ? $_POST['totalMoney'] : NULL;
        $itemId = isset($_POST['itemId']) ? $_POST['itemId'] : 0;
        $itemValue = isset($_POST['itemValue']) ? $_POST['itemValue'] : 0;
        $isBettingSlip = isset($_POST['isBettingSlip']) ? $_POST['isBettingSlip'] : 'OFF';
        $isClassic = isset($_POST['isClassic']) ? $_POST['isClassic'] : 'OFF';
        $betGroup = isset($_POST['betGroup']) ? $_POST['betGroup'] : NULL;
        $folderType = isset($_POST['folderType']) ? $_POST['folderType'] : 'S';
        $keep_login_access_token = isset($_POST['keep_login_access_token']) ? $_POST['keep_login_access_token'] : '';
        
        if (!CodeUtil::only_number($betType) ||
                (GT_SPORTS != $betType && GT_REALTIME != $betType ) ||
                !isset($betList) ||
                !CodeUtil::only_number($totalMoney) ||
                $totalMoney < 0 ||
                !CodeUtil::only_hangul($betGroup) ||
                !is_numeric($itemId) ||
                !is_numeric($itemValue) ||
                ('ON' != $isBettingSlip && 'OFF' != $isBettingSlip) ||
                ('ON' != $isClassic && 'OFF' != $isClassic) ||
                ('S' != $folderType && 'D' != $folderType)
        ) {
            $response['messages'] = '인자값이 잘못되었습니다.';

            $this->logger->error('$betType : ' . CodeUtil::only_number($betType) . ' $totalMoney : ' . CodeUtil::only_number($totalMoney) . ' $betGroup : ' . CodeUtil::only_hangul($betGroup));
            $this->logger->error('$isBettingSlip : ' . $isBettingSlip . ' $isClassic : ' . $isClassic . ' $folderTypev : ' . $folderType);
            return $this->fail($response);
        }

        if (false == session()->has('member_idx')) {
            $this->logger->error(__FUNCTION__ . ' not find member_idx : ' . session()->get('member_idx'));
            $response['messages'] = '유저를 찾을수 없습니다.';
            return $this->fail($response);
        }

        $member_idx = session()->get('member_idx');
        $memberModel = new MemberModel();

        // 실시간이면 스코어 체크
        if (2 == $betType) {
            list($retval, $message) = $this->checkRealTimeGameLiveScore_renew($betList, $member_idx);
            if (false == $retval) {
                $response['messages'] = $message;
                return $this->fail($response);
            }
        }
        
        $memberModel->db->transStart();
        list($retval, $message, $result, $af_money, $totalMoney, $array_league_tag_id, $keep_login_access_token) = $this->doAddBet($betList, $betType, $totalMoney
                , $itemId, $itemIdx, $itemValue, $isBettingSlip, $isClassic, $betGroup, $folderType, $keep_login_access_token, $memberModel);
        if (false == $retval) {
            $memberModel->db->transRollback();
            $response['messages'] = $message;
            return $this->fail($response);
        }
        $memberModel->db->transComplete();
        $response = [
            'code' => 200,
            'messages' => '배팅 성공',
            'data' => [
                'bet_id' => $result,
                'total_money' => $af_money,
                'total_bet_money' => $totalMoney,
                'arr_tag_ids' => $array_league_tag_id,
                'keep_login_access_token' => $keep_login_access_token
            ]
        ];
        
        return $this->respond($response, 200);
    }

    // 실시간 베팅전 스코어 저장
    public function setRealTimeScore($arr_fixture) {

        if ('' == $arr_fixture) {
            $this->logger->error(__FUNCTION__ . ' not find fixture_id : ' . json_encode($arr_fixture));
            return [false, 'not find fixture_id'];
        }

        $accessLogRedis = new accessLogRedis(config(App::class)->redis_ip, config(App::class)->redis_port, config(App::class)->redis_password, config(App::class)->redis_database, 180, $this->logger);
        if (!$accessLogRedis->connect()) {
            //$response['messages'] = '레디스 연결 실패.';
            //return $this->fail($response);
            return [false, 'fail connect to redis server'];
        }

        // get current score
        $lsSportsModel = new LSportsSportsModel();
        $sql = "SELECT fixture_id, livescore, fixture_sport_id, live_results_p1, live_results_p2 from lsports_fixtures where fixture_id in ($arr_fixture) and bet_type = ?";
        $result = $lsSportsModel->db->query($sql, [2])->getResultArray();
        if (0 == count($result)) {
            $this->logger->error(__FUNCTION__ . ' not find arr_fixture : ' . json_encode($arr_fixture));
            //$response['messages'] = '존재하지 않는 경기입니다.';
            //return $this->fail($response);
            return [false, '존재하지 않는 경기입니다.'];
        }

        foreach ($result as $key => $value) {
            // 배구면 세트점수로 넘어오기 때문에 피리어드에서 각 세트당 점수를 합산해줘야 함.
            $live_results_p1 = 0;
            $live_results_p2 = 0;
            $fixture_id = $value['fixture_id'];
            $fixture_sport_id = $value['fixture_sport_id'];
            $livescore = $value['livescore'];
            //$arrLivescore = json_decode($value['livescore'], true);
            $arrLivescore = json_decode($value['livescore'], true);
            
            $Scoreboard = $arrLivescore['Scoreboard'];
            if (VOLLEYBALL == $fixture_sport_id) {
                $arrLivescore = json_decode($value['livescore'], true);
                foreach ($arrLivescore['Periods'] as $key => $value) {
                    if ($value['Type'] < 6) {
                        if(1 == $value['Results'][0]['Position']){
                            $live_results_p1 += $value['Results'][0]['Value'];
                            $live_results_p2 += $value['Results'][1]['Value'];
                        } else{
                            $live_results_p1 += $value['Results'][1]['Value'];
                            $live_results_p2 += $value['Results'][0]['Value'];
                        }
                    }
                }
            } else if (ESPORTS == $fixture_sport_id && ($value['livescore'] == '' || $value['livescore'] == null || $value['livescore'] == '[object Object]')) {
                $live_results_p1 = $live_results_p2 = 0;
            } else {
                //$totalLiveScore = $arrLivescore['Scoreboard']['Results'][0]['Value'] + $arrLivescore['Scoreboard']['Results'][1]['Value'];
                if(1 == $Scoreboard['Results'][0]['Position']){
                    $live_results_p1 = $Scoreboard['Results'][0]['Value'];
                    $live_results_p2 = $Scoreboard['Results'][1]['Value'];
                } else{
                    $live_results_p1 = $Scoreboard['Results'][1]['Value'];
                    $live_results_p2 = $Scoreboard['Results'][0]['Value'];
                }
            }

            // save redis
            $checkBettingDate = date('Y-m-d H:i:s');
            $redis_key = 'realtime_bet_' . session()->get('member_idx') . '_' . $fixture_id;
            $arrRedisData = json_encode(array('live_results_p1' => $live_results_p1, 'live_results_p2' => $live_results_p2
                , 'fixture_sport_id' => $fixture_sport_id, 'livescore' => $livescore, 'checkBettingDate' => $checkBettingDate));
            $accessLogRedis->set($redis_key, json_encode($arrRedisData));
        }

        //return $this->respond($response, 200);
        return [true, 'success'];
    }

    // 실시간 경기 베팅가능 체크
    public function checkRealTimeGameLiveScore_renew($betList, $member_idx) {

        // create redis object
        $accessLogRedis = new accessLogRedis(config(App::class)->redis_ip, config(App::class)->redis_port, config(App::class)->redis_password, config(App::class)->redis_database, 120, $this->logger);
        if (!$accessLogRedis->connect()) {
            $this->logger->error(__FUNCTION__ . ' redis connect fail');
            //$response['messages'] = '레디스 연결 실패.';
            //return $this->fail($response);
            return [false, 'fail connect redis server'];
        }

        $live_score_1 = 0;
        $live_score_2 = 0;

        $checkBaseBallMarkets = array(Q2_1X2, Q2_OVER_UNDER, Q3_1X2, Q4_1X2, Q3_OVER_UNDER, Q4_OVER_UNDER, Q5_OVER_UNDER, Q5_1X2, Q6_1X2, Q6_OVER_UNDER, Q7_1X2, Q7_OVER_UNDER);
        $checkBasketBallMarkets = array(Q1_OVER_UNDER, Q2_OVER_UNDER, Q3_OVER_UNDER, Q4_OVER_UNDER);

        foreach ($betList as $key => $value) {
            $fixture_id = $value['fixtureId'];
            $betId = $value['betId'];

            // get before bet_data is redis...
            $redis_key = 'realtime_bet_' . $member_idx . '_' . $fixture_id;
            $redis_result = $accessLogRedis->get($redis_key);
            if (null == $redis_result) {
                //$response['messages'] = '스코어가 맞지않습니다.!!!';
                //return $this->fail($response);
                return [false, '스코어가 맞지않습니다.!!!'];
            }
            $redis_result = json_decode(json_decode($redis_result), true);


            $live_score_1 = $redis_result['live_results_p1'];
            $live_score_2 = $redis_result['live_results_p2'];
            $checkBettingDate = $redis_result['checkBettingDate'];

            if (0 == $fixture_id) {
                $response['messages'] = '잘못된 경기번호입니다.';
                //return $this->fail($response);
                return [false, $response['messages']];
            }
            $lsSportsModel = new LSportsSportsModel();

            // 축구(6046), 아이스하키(35232), 농구(48242), 미식축구(131506), 배구(154830), 야구(154914),이스포츠(687890),UFC(154919)
            $sql = "SELECT livescore, fixture_sport_id, live_results_p1, live_results_p2, fixture_sport_id from lsports_fixtures where fixture_id = ? and bet_type = 2";

            $result = $lsSportsModel->db->query($sql, [$fixture_id])->getResult();

            // 베팅정보를 가져온다. 
            $sql = "SELECT markets_id,bet_name  FROM lsports_bet where bet_type = 2 and fixture_id = ? and bet_id = ?";
            $betInfo = $lsSportsModel->db->query($sql, [$fixture_id, $betId])->getResult();

            $checkScore = 0;

            $currentDate = date('Y-m-d H:i:s');
            $delayTimeConfig = config(App::class)->betDelayTime;
            $betDelayTime = 1;  // 1초정도 허용
            // 축구
            if (SOCCER == $result[0]->fixture_sport_id) {
                $checkScore = 1;
                $betDelayTime += ($delayTimeConfig[SOCCER] / 1000);
            } else if (ICEHOCKEY == $result[0]->fixture_sport_id) {// 7초있다가 
                $checkScore = 1;
                $betDelayTime += ($delayTimeConfig[ICEHOCKEY] / 1000);
            } else if (BASKETBALL == $result[0]->fixture_sport_id) {
                $checkScore = 4;
                $betDelayTime += ($delayTimeConfig[BASKETBALL] / 1000);
            } else if (VOLLEYBALL == $result[0]->fixture_sport_id) {
                //$checkScore = 2;
                $checkScore = 9999;
                $betDelayTime += ($delayTimeConfig[VOLLEYBALL] / 1000);
            } else if (BASEBALL == $result[0]->fixture_sport_id) {
                $checkScore = 1;
                $betDelayTime += ($delayTimeConfig[BASEBALL] / 1000);
            } else {
                $checkScore = 9999;
                $betDelayTime = 9999;
            }

            $checkBetTime = date("Y-m-d H:i:s", strtotime($checkBettingDate . "+" . $betDelayTime . "seconds"));
            //$this->logger->error('current : '.$currentDate.' checkBet : '.$checkBetTime.' checkBettingDate : '.$checkBettingDate.' betDelayTime : '.$betDelayTime);
            if ($currentDate > $checkBetTime) {
                $response['messages'] = '베팅시간이 맞지않아서 베팅에 실패했습니다.';
                return [false, $response['messages']];
            }

            $livescore = json_decode($result[0]->livescore, true);
            // 스코어 차이체크
            $betTotalLiveScore = $live_score_1 + $live_score_2;
            // 배구면 세트점수로 넘어오기 때문에 피리어드에서 각 세트당 점수를 합산해줘야 함.
            $totalLiveScore = 0;
            if (VOLLEYBALL == $result[0]->fixture_sport_id) {
                foreach ($livescore['Periods'] as $key => $value) {
                    if ($value['Type'] < 6) {
                        $totalLiveScore += ($value['Results'][0]['Value'] + $value['Results'][1]['Value']);
                    }
                }
            } else {
                $totalLiveScore = $livescore['Scoreboard']['Results'][0]['Value'] + $livescore['Scoreboard']['Results'][1]['Value'];
            }

            if (BASKETBALL == $result[0]->fixture_sport_id && in_array($betInfo[0]->markets_id, $checkBasketBallMarkets) && 'Over' == $betInfo[0]->bet_name && 3 == abs($betTotalLiveScore - $totalLiveScore)) {
                $response['messages'] = '스코어가 맞지 않아서 배팅에 실패했습니다.';
                //return $this->fail($response);
                return [false, $response['messages']];
            }

            if ($checkScore <= abs($betTotalLiveScore - $totalLiveScore)) {
                $this->logger->debug('clientScore_1 : ' . $live_score_1);
                $this->logger->debug('clientScore_2 : ' . $live_score_2);
                $this->logger->debug('serverScore_1 : ' . $livescore['Scoreboard']['Results'][0]['Value']);
                $this->logger->debug('serverScore_2 : ' . $livescore['Scoreboard']['Results'][1]['Value']);

                $response['messages'] = '스코어가 맞지 않아서 배팅에 실패했습니다.';
                //return $this->fail($response);
                return [false, $response['messages']];
            }

            //  주자 상태가 바뀌었어도 체크한다.

            if (BASEBALL == $result[0]->fixture_sport_id) {
                $redis_livescore = json_decode($redis_result['livescore'], true);
                $redis_LivescoreExtraData = $redis_livescore['LivescoreExtraData'];

                $livescore = json_decode($result[0]->livescore, true);
                $LivescoreExtraData = $livescore['LivescoreExtraData'];

                $this->logger->info("!!!!!!!!!!!!!!!!!! ***************** checkRealTimeGameLiveScore redis_LivescoreExtraData: ==> " . json_encode($redis_LivescoreExtraData));
                $this->logger->info("!!!!!!!!!!!!!!!!!! ***************** checkRealTimeGameLiveScore LivescoreExtraData: ==> " . json_encode($LivescoreExtraData));

                $redis_turn = 1;
                $turn = 1;
                foreach ($redis_LivescoreExtraData as $value) {
                    if ('Turn' != $value['Name'])
                        continue;

                    $redis_turn = $value['Value'];
                    break;
                }

                foreach ($LivescoreExtraData as $value) {
                    if ('Turn' != $value['Name'])
                        continue;

                    $turn = $value['Value'];
                    break;
                }

                if ($redis_turn != $turn) {
                    foreach ($LivescoreExtraData as $value) {
                        if ('Bases' != $value['Name'])
                            continue;

                        $bases = $value['Value'];
                        $arr_bases = explode('/', $bases);

                        foreach ($arr_bases as $base) {
                            if (0 == $base)
                                continue;
                        }

                        $response['messages'] = '배팅중 주자가 진루하여 배팅이 취소됩니다.';
                        //return $this->fail($response);
                        return [false, $response['messages']];
                        //break;
                    }
                } else {
                    $arr_redis_bases = [];
                    $arr_bases = [];

                    foreach ($redis_LivescoreExtraData as $value) {
                        if ('Bases' != $value['Name'])
                            continue;

                        $bases = $value['Value'];
                        $arr_redis_bases = explode('/', $bases);

                        break;
                    }

                    foreach ($LivescoreExtraData as $value) {
                        if ('Bases' != $value['Name'])
                            continue;

                        $bases = $value['Value'];
                        $arr_bases = explode('/', $bases);

                        break;
                    }

                    if (0 == count($arr_redis_bases) && 0 < count($arr_bases)) {
                        $sumbase = $arr_bases[0] + $arr_bases[1] + $arr_bases[2];
                        if (0 < $sumbase) {
                            $response['messages'] = '배팅중 주자가 진루하여 배팅이 취소됩니다.';
                            //return $this->fail($response);
                            return [false, $response['messages']];
                        }
                    } else if (0 < count($arr_redis_bases) && 0 < count($arr_bases)) {
                        if ($arr_redis_bases[0] !== $arr_bases[0] || $arr_redis_bases[1] !== $arr_bases[1] || $arr_redis_bases[2] !== $arr_bases[2]) {

                            $response['messages'] = '배팅중 주자가 진루하여 배팅이 취소됩니다.';
                            //return $this->fail($response);
                            return [false, $response['messages']];
                        }
                    }
                }
            }
        }
        return [true, 'success'];
    }

}
