<?php

namespace App\Controllers;

use App\Models\AccountModel;
use App\Models\MemberModel;
use App\Models\TLogCashModel;
use App\Models\TGameConfigModel;
use App\Util\CodeUtil;
use App\Util\Casino;
use Cassandra\Date;
use CodeIgniter\API\ResponseTrait;
use App\RollingComps\UserPayBack;
use App\RollingComps\UserBettingLose;
use App\Util\Calculate;
use App\Entities\Member;

class HashController extends BaseController {

    use ResponseTrait;

    public function hash() {
        $member_idx = session()->get('member_idx');
        $chkMobile = CodeUtil::rtn_mobile_chk();
        $viewRoot = "PC" == $chkMobile ? 'web' : 'web';

        if (false == session()->has('member_idx')) {
            $url = base_url("/$viewRoot/index");
            echo "<script>
    		alert('로그인 후 이용해주세요.');
    		window.location.href='$url';
    		</script>";
            return;
        }

        $memberModel = new MemberModel();
        $member = $memberModel->getMemberWhereIdx($member_idx);
        if ($member == null) {
            return $this->fail('조회되는 회원 또는 메세지 idx 가 없습니다.');
        }

        $tgcModel = new TGameConfigModel();
        $str_sql_config = "SELECT set_type, set_type_val FROM t_game_config WHERE set_type = 'service_hash'";
        $arr_config_result = $tgcModel->db->query($str_sql_config)->getResultArray();

        $arr_config = array();
        foreach ($arr_config_result as $key => $value) {
            $arr_config[$value['set_type']] = $value['set_type_val'];
        }

        // 유저개인 점검
        $game_type_sql = "SELECT game_type, status FROM member_game_type where member_idx = ? and game_type in (13)";
        $arr_member_config = $tgcModel->db->query($game_type_sql, [$member_idx])->getResultArray();
        

        // 점검체크
        $findMember             = new Member();
        $user_level             = session()->get('level');

        if ('Y' == $arr_config['service_hash'] && 9 != $findMember->getLevel()) {
            echo "<script>
            alert('해쉬게임 점검중입니다.');
            window.history.back();
            </script>";
            return;
        }

        if ('OFF' == $arr_member_config[0]['status'] && 9 != $findMember->getLevel()) {
            $url = base_url("/$viewRoot/index");
            echo "<script>
            alert('해당 게임 배팅이 금지되어있습니다.');
            window.location.href='$url';
            </script>";
            return;
        }

        $profile = config(App::class)->profile;


        $mobileType = "PC" == $chkMobile ? 'web' : 'web';
        $viewRoot = $mobileType;

        return view("$viewRoot/hash", [
            'mobileType' => $mobileType
            , 'type' => $type
        ]);
    }

    private function selectHLHashBet($arrBet, $memberModel) {
        $sql = "SELECT  IFNULL(HS.HASH_BET_IDX,count(HS.HASH_BET_IDX)) as idx
		FROM
		OD_HASH_BET_HIST HS
		WHERE HS.MBR_IDX = ?
		AND HS.PRD_TYPE = ?
		AND HS.WAVE = ?
		AND HS.BT_TYPE = ?
		AND HS.BT_TYPE_LB = ? ";

        $result = $memberModel->db->query($sql, [$arrBet['code'], $arrBet['productType'], $arrBet['wave'], $arrBet['betting_type'], $arrBet['betting_type_label']])->getResultArray();

        if (!isset($result) || 0 == count($result)) {
            return 0;
        }

        return $result[0]['idx'];
    }

    private function selectRTHashBet($arrBet, $memberModel) {
        $sql = "SELECT  IFNULL(HS.HASH_BET_IDX,count(HS.HASH_BET_IDX)) as idx
		FROM
		OD_HASH_BET_HIST HS
		WHERE HS.MBR_IDX = ?
		AND HS.PRD_TYPE = ?
		AND HS.WAVE = ?";

        $result = $memberModel->db->query($sql, [$arrBet['code'], $arrBet['productType'], $arrBet['wave']])->getResultArray();

        $this->logger->error('selectRTHashBet lastquery : ' . $memberModel->getLastQuery());
        if (!isset($result) || 0 == count($result)) {
            return 0;
        }

        return $result[0]['idx'];
    }

    private function selectBRHashBet($arrBet, $memberModel) {
        $sql = "SELECT  IFNULL(HS.HASH_BET_IDX,count(HS.HASH_BET_IDX)) as idx
		FROM
		OD_HASH_BET_HIST HS
		WHERE HS.MBR_IDX = ?
		AND HS.PRD_TYPE = ?
		AND HS.WAVE = ?
		AND HS.CH_CODE = ?";

        $result = $memberModel->db->query($sql, [$arrBet['code'], $arrBet['productType'], $arrBet['wave'], $arrBet['channel_code']])->getResultArray();

        if (!isset($result) || 0 == count($result)) {
            return 0;
        }

        return $result[0]['idx'];
    }

    private function updateOdBetHashBet($arrBet, $memberModel) {
        $sql = "UPDATE OD_HASH_BET_HIST SET
		HLD_MNY = ?,
		BET_MNY = BET_MNY +  ?,
		RSLT_MNY = RSLT_MNY - ?,
		MOD_DTM = NOW()
		WHERE HASH_BET_IDX = ?";

        $memberModel->db->query($sql, [$arrBet['holdMoney'], $arrBet['betMoney'], $arrBet['betMoney'], $arrBet['hash_bet_idx']]);
    }

    private function insertOdHashBet($arrBet, $memberModel) {
        $sql = "INSERT INTO OD_HASH_BET_HIST (TYPE, MBR_IDX, HLD_MNY, BET_MNY, RSLT_MNY,
		PRD_TYPE, WAVE,CH_CODE,BT_TYPE,BT_TYPE_LB)
		VALUES ('B', ?, ?, ?, ?,
		?, ?, ?, ?, ?)";
        $memberModel->db->query($sql, [$arrBet['memberIdx'], $arrBet['holdMoney'], $arrBet['betMoney'], -$arrBet['betMoney'], $arrBet['productType']
            , $arrBet['wave'], $arrBet['channel_code'], $arrBet['betting_type'], $arrBet['betting_type_label']]);

        $row = $memberModel->db->query('SELECT LAST_INSERT_ID()')->getResultArray();

        $idx = $row[0]['LAST_INSERT_ID()'];
        return $idx;
    }

    private function updateLastDepositBet($arrBet, $memberModel) {
        $sql = " UPDATE member_money_charge_history
        SET   hash_bet_money = hash_bet_money + ? 
        WHERE member_idx = ? AND status = 3 ORDER BY update_dt DESC LIMIT 1 ";
        $memberModel->db->query($sql, [$arrBet['betMoney'], $arrBet['memberIdx']]);
    }

    private function updateBetPlus($arrBet, $memberModel) {
        $sql = " INSERT INTO tb_user_pay_back_info (member_idx) 
                VALUES (?) ON DUPLICATE KEY UPDATE tot_bet_money = IF(charge > 0 , tot_bet_money + ?, tot_bet_money);";
        $memberModel->db->query($sql, [$arrBet['memberIdx'], $arrBet['betMoney']]);
    }

    private function insertCashLog($arrBet, $memberModel) {

        $ip = CodeUtil::get_client_ip();
        $a_country = $memberModel->query('SELECT FN_GET_IP_COUNTRY(?)', [$ip])->getResultArray();

        $sql = "INSERT INTO t_log_cash (member_idx, ac_code, ac_idx, r_money, be_r_money, af_r_money, m_kind, coment,u_ip,a_country)
        VALUES (?, ?, ?, ABS(?), ?, ?, ?, ? ,?,?)";

        $ac_code = 3;
        $kind = 'M';
        $amount = $arrBet['betMoney'];
        $result = $arrBet['holdMoney'] - $arrBet['betMoney'];
        if ('B' != $arrBet['type']) {
            $ac_code = 7;
            $kind = 'P';
            $amount = $arrBet['addMoney'];
            $result = $arrBet['holdMoney'] + $arrBet['addMoney'];
        }

        $memberModel->db->query($sql, [$arrBet['memberIdx'], $ac_code, $arrBet['wave'], $amount, $arrBet['holdMoney'], $result
            , $kind, 'hash_game', $ip, $a_country[0]]);
    }

    private function doOdBetting($arrBet, $memberModel) {
        $idx = 0;
        switch ($arrBet['productType']) {
            case "H": {
                    //log.info("doOdBetting before selectHLHashBet: " + idx);
                    $idx = $this->selectHLHashBet($arrBet, $memberModel);
                    //log.info("doOdBetting after selectHLHashBet: " + idx);
                }
                break;
            case "R": {
                    $idx = $this->selectRTHashBet($arrBet, $memberModel);
                }
                break;
            case "B": {
                    $idx = $this->selectBRHashBet($arrBet, $memberModel);
                }
                break;
        }

        if (0 < $idx) {
            $arrBet['hash_bet_idx'] = $idx;
            $this->updateOdBetHashBet($arrBet, $memberModel);
        } else {
            $arrBet['hash_bet_idx'] = $this->insertOdHashBet($arrBet, $memberModel);
        }

        //$this->updateLastDepositBet($arrBet, $memberModel);

        //
        //$this->updateBetPlus($arrBet, $memberModel);
        Calculate::updateChargeBetMoney($arrBet['memberIdx'],'HASH',$arrBet['betMoney'], $memberModel, $this->logger);
        UserPayBack::AddBetting($arrBet['memberIdx'], $arrBet['betMoney'], $memberModel);
        UserBettingLose::RewardBettingComps($arrBet['memberIdx'],'hash', $arrBet['betMoney'], $this->logger, $memberModel);


        /* Cash Logs 저장 */
        $finalAcIdx = $idx;
        $arrBet['type'] = 'B';
        $this->insertCashLog($arrBet, $memberModel);

        /* 회원 머니 차감 */
        $memberModel->memberChangeMoney($arrBet['memberIdx'], -$arrBet['betMoney']);


        return $arrBet['holdMoney'] - $arrBet['betMoney'];
    }

    private function updateOdRewardHashBet($arrBet, $memberModel) {
        $sql = "UPDATE OD_HASH_BET_HIST SET
		TYPE = ?,
		RSLT_MNY = RSLT_MNY + ?,
		MOD_DTM = NOW()
		WHERE HASH_BET_IDX = ?";
        $memberModel->db->query($sql, [$arrBet['type'], $arrBet['addMoney'], $arrBet['hash_bet_idx']]);
    }

    private function modifyHashBetting($arrBet, $memberModel) {
        $idx = 0;
        switch ($arrBet['productType']) {
            case "H": {
                    $idx = $this->selectHLHashBet($arrBet, $memberModel);
                }
                break;
            case "R": {
                    $idx = $this->selectRTHashBet($arrBet, $memberModel);
                }
                break;
            case "B": {
                    $idx = $this->selectBRHashBet($arrBet, $memberModel);
                }
                break;
        }

        $this->logger->error('modifyHashBetting idx :' . $idx);
        $arrBet['hash_bet_idx'] = $idx;
        if (0 < $idx) {
            $this->updateOdRewardHashBet($arrBet, $memberModel);
        } else {
            throw new \Exception('INVALID_PRODUCT');
        }
        /* Cash Logs 저장 */
        $this->insertCashLog($arrBet, $memberModel);
    }

    private function doOdReward($arrBet, $memberModel) {

        //log.info("========================================================");
        //log.info("doOdReward");
        //log.info("params : " + params);
        //log.info("========================================================");
        $arrBet['type'] = 'W';
        $this->modifyHashBetting($arrBet, $memberModel);
        $memberModel->memberChangeMoney($arrBet['memberIdx'], $arrBet['addMoney']);

        return $arrBet['holdMoney'] + $arrBet['addMoney'];
    }

    private function selectDelayHashList($memberModel) {
        $sql = "select * from OD_HASH_BET_HIST
                WHERE TYPE = 'B'
                AND DATE_ADD(REG_DTM, INTERVAL 2 MINUTE) <= NOW()";
        $result = $memberModel->db->query($sql)->getResultArray();

        if (!isset($result) || 0 == count($result)) {
            return [];
        }

        return $result;
    }

    private function updateLoseHashBet($memberModel) {
        $sql = "  UPDATE OD_HASH_BET_HIST
        SET TYPE = 'L',
            MOD_DTM = NOW()
        WHERE TYPE = 'B'
        AND DATE_ADD(REG_DTM, INTERVAL 2 MINUTE) <= NOW()";
        $memberModel->db->query($sql);
    }

    public function oddsV1() {
        try {
            $memberModel = new MemberModel();
            $_REQUEST = $memberModel->filterSanitize($_REQUEST);
            $cmd = isset($_REQUEST['cmd']) ? $_REQUEST['cmd'] : '';
            $id = isset($_REQUEST['id']) ? $_REQUEST['id'] : '';
            $code = isset($_REQUEST['code']) ? $_REQUEST['code'] : 0;
            $money = isset($_REQUEST['money']) ? $_REQUEST['money'] : 0;
            $msg = isset($_REQUEST['msg']) ? $_REQUEST['msg'] : '';

            $this->logger->error('oddsV1 : ' . json_encode($_REQUEST));
            if (!CodeUtil::only_number($code)) {
                $response = [
                    'ret' => 0,
                    'msg' => "MISSING DATA"
                ];
                $this->logger->error('oddsV1 error : ' . json_encode($response));
                return $this->respond($response, 200);
            }


            $memberModel = new MemberModel();
            $member = $memberModel->getMemberWhereIdx($code);
            if ($member == null) {
                $response = [
                    'ret' => 0,
                    'msg' => "INVALID USER"
                ];
                $this->logger->error('oddsV1 error : ' . json_encode($response));
                return $this->respond($response, 200);
            }

            $afterMoney = 0;
            $holdMoney = $member->getMoney();
            $actionMoney = $money;

            switch ($cmd) {
                /* 회원 조회 */
                case "UserCheck":
                    $afterMoney = $holdMoney;
                    $response = [
                        'ret' => 1,
                        'msg' => "성공",
                        'balance' => $afterMoney,
                        'nick' => $member->getNickName(),
                        'isuse_highlow' => 1,
                        'isuse_roulette' => 1,
                        'isuse_baccarat' => 1
                    ];
                    $this->logger->error('oddsV1 error : ' . json_encode($response));
                    return $this->respond($response, 200);

                    break;
                /* 배팅 로그 */
                case "MoneyLog":
                    if (!CodeUtil::only_number($money) || true === empty($msg)) {
                        $response = [
                            'ret' => 0,
                            'msg' => "MISSING DATA"
                        ];
                        $this->logger->error('oddsV1 error : ' . json_encode($response));
                        return $this->respond($response, 200);
                    }

                    $arrData = explode('|', $msg);
                    $arrBet = array();
                    foreach ($arrData as $value) {
                        $arrSplit = explode('=', $value);
                        $arrBet[$arrSplit[0]] = $arrSplit[1];
                    }

                    switch ($arrBet['game']) {
                        case "highlow":
                            $prodType = "H";
                            break;
                        case "roulette":
                            $prodType = "R";
                            break;
                        case "baccarat":
                            $prodType = "B";
                            break;
                        default:
                            break;
                    }

                    $this->logger->error('oddsV1 arrBet : ' . json_encode($arrBet));
                    if (false == array_key_exists('reward', $arrBet)) {
                        /* 1.플레이어의 잔여 포인트가 없는 경우
                         * 2.베팅 금액보다 유저 머니가 적을 경우
                         */
                        $this->logger->error('doOdBetting');
                        if ($holdMoney == 0 || ($holdMoney < -$actionMoney && $actionMoney < 0)) {
                            $response = [
                                'ret' => 0,
                                'msg' => "not enough user point",
                                'balance' => 0
                            ];
                            $this->logger->error('oddsV1 error : ' . json_encode($response));
                            return $this->respond($response, 200);
                        }

                        $arrBet['memberIdx'] = $code;
                        $arrBet['betMoney'] = abs($money);
                        $arrBet['productType'] = $prodType;
                        $arrBet['holdMoney'] = $member->getMoney();

                        $afterMoney = $this->doOdBetting($arrBet, $memberModel);
                        
                        /* 당첨 */
                    } else {
                        $this->logger->error('doOdReward');
                        $arrBet['memberIdx'] = $code;
                        $arrBet['addMoney'] = $arrBet['reward'];
                        $arrBet['productType'] = $prodType;
                        $arrBet['userLevel'] = $member->getLevel();
                        $arrBet['holdMoney'] = $member->getMoney();
                        $afterMoney = $this->doOdReward($arrBet, $memberModel);
                        UserBettingLose::RewardWinComps($member->getIdx(), 'hash', abs($money), $this->logger, $memberModel); 
                        //UserBettingLose::RewardDistWinComps($member->getIdx(), 'hash', abs($money), $this->logger, $memberModel); 
                    }

                    break;
                default:
                    $response = [
                        'ret' => 0,
                        'msg' => "INVALID CMD"
                    ];
                    $this->logger->error('oddsV1 error : ' . json_encode($response));
                    return $this->respond($response, 200);
                    break;
            }

            $response = [
                'ret' => 1,
                'msg' => "성공",
                'balance' => $afterMoney
            ];
            $this->logger->error('oddsV1 success : ' . json_encode($response));
            return $this->respond($response, 200);
        } catch (\Exception $e) {
            $this->logger->critical('::::::::::::::: oddsV1 query : ' . $memberModel->getLastQuery());
            $response = [
                'ret' => 0,
                'msg' => $e->getMessage()
            ];
            $this->logger->error('oddsV1 error : ' . json_encode($response));
            return $this->respond($response, 200);
        } catch (\mysqli_sql_exception $e) {
            $this->logger->critical('oddsV1 [MYSQL EXCEPTION] message (code) : ' . $e->getMessage() . ' (' . $e->getCode() . ')');
            $this->logger->critical('::::::::::::::: oddsV1 query : ' . $memberModel->getLastQuery());

            $response = [
                'ret' => 0,
                'msg' => 'query error'
            ];
            $this->logger->error('oddsV1 error : ' . json_encode($response));
            return $this->respond($response, 200);
        }
    }

    public function modifyDelayOdBet() {
        $memberModel = new MemberModel();
        $betList = $this->selectDelayHashList($memberModel);

        $this->updateLoseHashBet($memberModel);
        
        foreach ($betList as $value) {
            UserBettingLose::RewardLoseComps($value['MBR_IDX'],'hash',$value['BET_MNY'],$this->logger,$memberModel);   
            //UserBettingLose::RewardDistLoseComps($value['MBR_IDX'],'hash',$value['BET_MNY'],$this->logger,$memberModel);   
        }
    }

}
