<?php

namespace App\Game;

use App\Models\TGameIdManager;
use App\Classes\TelegramClass;
use App\Models\TProductGroupManager;
use App\Models\TLogCashModel;
use App\Models\TBettingInfo;
use App\Models\TProductIdManager;
use App\Models\TBettingInfoCallback;
use App\Util\CodeUtil;
use App\Util\Calculate;
use App\RollingComps\UserPayBack;
use App\RollingComps\UserBettingLose;
use DateTime;

class Goodgaming extends GameAPI {
    private $opkey = null;
    private $api_url = null;
    private $provider_info = null;
    private $log_channel = null;
    private $log_file = null;
    private $request = null;
    private $member_info = null;
    private $product_group_info = null;
    private $game_information;

    public function __construct($logger) {
        $this->logger = $logger;
        $this->project = 'Noble';
        $this->operator_name = 'noble01';
        $this->DB = \Config\Database::connect();

        /* test creds */
        $this->opkey = '88a72bec795cef06286cad429d5191ab';
        $this->api_url = 'https://external.goodgame-api.link';

        /* live creds */
        if (ENVIRONMENT == "production") {
            $this->opkey = '88a72bec795cef06286cad429d5191ab';
            $this->api_url = 'https://external.goodgame-api.link';
        }
    }

    public function isLoginById($user_id, $function_name, $member_model) {

        if($function_name == "bet") {
            $function_name = "betting";
        }

        $this->logger->info(':: Goodgaming:: callback (' . $user_id . ', ' . $function_name . ')');

        // check of logged-in
        $member = $member_model->setMemberWhereId($user_id);

        if ($member == null) {
            return [null, '조회되는 회원 또는 메세지 idx 가 없습니다.'];
        }

        if ($member->getUBusiness() != 1) {
            return [null, '총판은 배팅 이용이 불가능합니다.'];
        }

        if ($member->getStatus() == 11) {
            $response['messages'] = '관리자 승인이 필요합니다.';
            return [null, '관리자 승인이 필요합니다.'];
        }

        if ('credit' != $function_name && 'cancel' != $function_name) {
            if ($member->getStatus() == 2 || $member->getStatus() == 3) {
                return [null, '사용이 불가능 한 계정으로 관리자에게 문의바랍니다.'];
            }
        }

        return [$member, ''];
    }

    function processRequest($member, $member_model, $function_name, $data) {
        if($function_name == "bet") {
            $function_name = "betting";
        }

        $this->function_name = $function_name;
        $this->opkey = $data["transactionId"];
        $this->userid = $data["userid"];
        $this->token = $data["token"];
        $this->third_party_code = $data["vendorCode"];
        $this->third_party_name = $data["vendorName"];
        $this->times = $data["times"];
        $this->member_info = $member;

        /* show logs / 로그 표시 */
        $this->logger->info(':: Goodgaming:: callback (' . $this->member_info->id . ', ' . $this->member_info->idx . ', ' . $this->member_info->money . ')');

        /* callback validation / 콜백 검증 */
        $validation_response = $this->callbackValidation($data, $function_name, $member, $member_model);

        if ($validation_response["status"] == 1) {
            $validation_response["member_info"];
            $this->setResult(1, 'Callback Successful');
            $this->$function_name($data, $member_model);
        }

        return [$this->result["return_code"], $this->result["balance"], $data, $this->result["message"]];
    }

    public function callbackValidation($data, $function_name, $member, $member_model) {
        $this->logger->info(':: Goodgaming: callbackValidation');
        $this->logger->info($data);

        /* check if user information exist / 사용자 정보가 존재하는지 확인 */
        if (!isset($this->member_info) || !$this->member_info) {
            $this->logger->error(':: GoodgamingController::' . $function_name . ' => Invalid User ID');
            $this->setResult(400, 'Invalid User ID');
            return ['status' => 0, 'member_info' => $this->member_info, 'message' => 'Invalid User ID'];
        }

        $client_ip = CodeUtil::get_client_ip();
        return ['status' => 1, 'member_info' => $this->member_info, 'message' => 'Missing Parameters'];
    }

    public function fetchAndSaveGameList() {
        $params = [];
        $api_response = $this->sendAPIRequest('/vendor', $params);
        $_game_companies = json_decode($api_response, true);

        $tb_static_product_group_manager = new TProductGroupManager();
        
        foreach($_game_companies as $key => $game_company) {
            $product_group_info = $tb_static_product_group_manager->where("good_gaming_id", $game_company["vendorCode"])->first();
            echo $game_company["vendorCode"];
            echo "<br>";
            if($product_group_info) {
                $_game_list = $this->__getGameList($game_company["vendorCode"]);

                foreach($_game_list as $game) {
                    echo "~ " . $game["gameTitle"] .  " - ". ($product_group_info["desc"]);
                    echo "<br>";
                }
                
                $this->__checkSaveGames($product_group_info, $_game_list);
                $_game_companies[$key]["game_list"] = $_game_list;
            }
        }

        return $_game_companies;
    }

    private function __checkSaveGames($product_group_info, $_game_list) {
        foreach($_game_list as $game) {
            $this->logger->info('Game Update: ' . $game["gameCode"]);
            $tb_static_game_id_manager = new TGameIdManager();

            $check_exist = $tb_static_game_id_manager->getGameInformation($game["gameCode"], CONSTANTS_GOODGAMING, $product_group_info["product_group_id"]);

            $date_time = new DateTime($game["date"]);
            $game_date = $date_time->format('Y-m-d H:i:s');

            $insert_game = [];
            $insert_game["game_short_name"] = $game["gameTitleKr"];
            $insert_game["gamet_name_en"] = $game["gameTitle"];
            $insert_game["game_name_kr"] = $game["gameTitleKr"];
            $insert_game["game_image"] = $game["thumbnail"];
            $insert_game["is_desktop"] = 1;
            $insert_game["is_mobile"] = 1;
            $insert_game["game_date"] = date('Y-m-d H:i:s');
            $insert_game["updated_at"] = date('Y-m-d H:i:s');

            if ($check_exist) {
                $tb_static_game_id_manager->set('game_short_name', $game["name_kor"]);
                $tb_static_game_id_manager->set('gamet_name_en', $game["name_eng"]);
                $tb_static_game_id_manager->set('game_name_kr', $game["name_kor"]);
                $tb_static_game_id_manager->set('game_image', $game["img_1"]);
                $tb_static_game_id_manager->set('is_desktop', $game["is_desktop"]);
                $tb_static_game_id_manager->set('is_mobile', $game["is_mobile"]);
                $tb_static_game_id_manager->set('game_date', $game_date);
                $tb_static_game_id_manager->set('updated_at', date('Y-m-d H:i:s'));
                $tb_static_game_id_manager->set('product_type_id', $product_group_info["product_type_id"]);
                $tb_static_game_id_manager->where('provider_id', CONSTANTS_GOODGAMING)->where('game_id', $game["code"])->where('product_group_id', $product_group_info["product_group_id"])->update();
                $this->logger->info("- Game ID (" . $game["gameCode"] . ") - (" . $product_group_info["desc"] . ") updated." . "(" . $product_group_info["product_type_id"] . ")");
            } else {
                $insert_game["created_at"] = date('Y-m-d H:i:s');
                $insert_game["provider_id"] = CONSTANTS_GOODGAMING;
                $insert_game["product_group_id"] = $product_group_info["product_group_id"];
                $insert_game["game_id"] = $game["gameCode"];
                $insert_game["product_type_id"] = $product_group_info["product_type_id"];

                $tb_static_game_id_manager->insert($insert_game);

                $this->logger->info("- Game ID (" . $game["gameCode"] . ") - (" . $product_group_info["gameTitleKr"] . ") created.");
            }
        }
    }

    public function getGameURL($member_info, $product_group_info, $game_id = null) {
        $this->logger->info('~ Enter Game Initialize ~');
        $this->logger->info('Enter Game Game: ' . json_encode($game_id));
        $this->logger->info('Enter Game Group: ' . json_encode($product_group_info));
        
        $this->member_info = $member_info;
        $this->product_group_info = $product_group_info;

        /* get lobby code if game id is null */
        if (!$game_id) {
            $game_id = $product_group_info["good_gaming_lobby_code"];
            $this->logger->info('Enter Game Lobby: ' . json_encode($game_id));
        }

        /* get game information from database */
        $tb_static_game_id_manager = new TGameIdManager();
        $tb_static_product_group_manager = new TProductGroupManager();
        $this->game_information = $tb_static_game_id_manager->getGameInformation($game_id, $product_group_info["provider_id"], $product_group_info["product_group_id"]);
        
        $product_group_information = $tb_static_product_group_manager->where("product_group_id", $this->game_information['product_group_id'])->first();
        $params_token["vendorCode"] = $product_group_information["good_gaming_id"];
        $_game_list = $this->__getGameList($params_token["vendorCode"]);
        $this->logger->info("game_id: ".$game_id);
        if (!$this->game_information) {
            $this->returnError("내부 오류: 데이터베이스에서 게임을 찾을 수 없습니다.");
        } else {

            foreach($_game_list as $gameCode){
                if($gameCode['gameCode'] == $game_id){
                    $game_exist = true;
                }
            }

            if($game_exist != true){
                $this->logger->info("game_exist -> ".$game_id ." is OFF");
                $response['code'] = 66;
                $response['message'] = "현재 해당 게임은 플레이 할 수 없습니다.";
                return $response;
            }

            $this->__createAccountOnGoodGaming();
            $params_token["vendorCode"] = $product_group_information["good_gaming_id"];
            $params_token["username"] = "goodgaming" . $this->member_info->idx;
            $raw_response_token = $this->sendAPIPatch("/user/token", $params_token, true);
            $response_token = json_decode($raw_response_token, true);

            $params["vendorCode"] = $params_token["vendorCode"];
            $params["gameCode"] = $this->game_information["game_id"];
            $params["token"] = $response_token["token"];
            $raw_response = $this->sendAPIRequest("/game/url", $params);
            $response = json_decode($raw_response, true);

            // 에이전트 돈 조회
            $agent_response = $this->sendAPIRequest("/agent", []);
            $agent_response = json_decode($agent_response, true);

            $user_params['username'] = "goodgaming" . $member_info->idx;
            // 현재 유저 보유금 조회(굿게이밍내)
            $user_response = $this->sendAPIRequest("/user", $user_params);
            $user_response = json_decode($user_response, true);
            
            if(!isset($agent_response)){
                $this->logger->info("Agent Infomation doesn't exist.");
                $response['code'] = 80;
                $response['message'] = "현재 게임을 이용할 수 없습니다.";
                return $response;
            }
            if(!isset($user_response)){
                $this->logger->info("User Infomation doesn't exist.");
                $response['code'] = 70;
                $response['message'] = "현재 게임을 이용할 수 없습니다.";
                return $response;
            }
            if($user_response['balance'] <= 0) {
                if($agent_response['balance'] < $member_info->money){
                    $this->logger->info("Can't Playing game now -> usermoney is bigger ");
                    $response['code'] = 99;
                    $response['message'] = "현재 게임을 이용할 수 없습니다.";
                    return $response;
                }
            }

            if (!isset($response["url"])) {
                echo "<pre>";
                print_r($response);
                echo "</pre>";
                die();
                // $telegram_class = new TelegramClass();
                // $this->__sendTelegramMessageErrorOpeningGame($response);
                // $this->returnError($response["msg"]);
            } else {
                $response['code'] = 200;
                $response['link'] = $response['url'];
                return $response;
            }
        }
    }

    //public function validateCallback($request) {
    //}

    public function member($data, $member_model) {
        $this->__setAPIResponse(1, "Member Successfully Fetched");
    }

    public function balance($data, $member_model) {
        $this->__setAPIResponse(1, "Balance Successfully Fetched");
    }
    public function betting($data, $member_model) {
        /* show logs / 로그 표시 */
        $this->logger->info(':: Goodgaming:: callbackBetting()');
        $betting_amount = doubleval($data["amount"]);

        /* get game information */
        $game_list_model = new TGameIdManager();
        $tb_static_product_group_manager = new TProductGroupManager();
        $betting_info_callback_model = new TBettingInfoCallback();
        
        $product_group_info = $tb_static_product_group_manager->where("good_gaming_id", $data["vendorCode"])->first();
        $game_information = $game_list_model->getGameInformation($data["gameCode"], 4, $product_group_info["product_group_id"]);

        //$check_betting_transaction = $betting_info_callback_model->where("trx_id", $data["transactionId"])->first();
        $betting_info_model = new TBettingInfo();
        $betting_check = $betting_info_model->getOrderByTransId($data["transactionId"]);
               
        $product_id_manager_model = new TProductIdManager();
        $product_id_manager_data = $product_id_manager_model->where('product_id', $data["vendorCode"])->first();

        /* insert game information if it doesn't exist */
        if (!$game_information) {
            $game_list_model->set('game_id', $data["gameCode"]);
            $game_list_model->set('game_short_name', $data["vendorName"]);
            $game_list_model->set('gamet_name_en', $data["vendorName"]);
            $game_list_model->set('game_name_kr', $data["vendorName"]);
            $game_list_model->set('is_use', "ON");
            $game_list_model->set('status', 1);
            $game_list_model->set('product_id', $product_id_manager_data["product_id"]);
            $game_list_model->set('provider_id', 4);
            $game_list_model->set('product_type_id', $data["gameType"] != 'SLOT' ? 1 : 2);
            $game_list_model->set('product_group_id', $product_group_info["product_group_id"]);
            $game_list_model->insert();
            $game_information = $game_list_model->getGameInformation($data["game_code"], 4, $product_group_info["product_group_id"]);
            $this->logger->info(' betting data : product_type_id ' . $game_information["product_type_id"]);
        }

        $game_information["product_type_id"] = $data["gameType"] != 'SLOT' ? PRODUCT_TYPE_CASINO : PRODUCT_TYPE_SLOT;

        if (!$data["gameCode"] || !$game_information) {
            $this->logger->error(':: Goodgaming:: Game ID Not Found => ' . $data["game_code"]);
            $this->setResult(500, 'Game Information Not Found');
        } else if ($betting_check) {
            $this->setResult(101, 'Trans ID already exist.');
        } else {
            /* record betting amount / 기록적인 베팅 금액 */
            if ($this->member_info->money < $betting_amount) {
                $this->setResult(102, 'Inefficient Balance');
            } else if ($betting_amount >= 0) {
                if ((0 < $game_information["min_bet"] && $betting_amount < $game_information["min_bet"]) ||
                        (0 < $game_information["max_bet"] && $game_information["max_bet"] < $betting_amount)
                ) {
                    $this->logger->error(':: Goodgaming:: Game ID Not Found => ' . $data["game_code"]);
                    $this->setResult(102, 'Inefficient Balance');
                } else {
                    /* check if bet exist for update */
                    $betting_info_model = new TBettingInfo();
                    $betting_check = $betting_info_model->getBettingInfoForUpdateGoodGaming($data, $this->member_info->idx);

                    if ($betting_check) {
                        $total_bet_money = floatval($betting_check["bet_money"]) + $betting_amount;
                        $betting_info_model = $betting_info_model->set('bet_money', $total_bet_money);
                        $betting_info_model = $betting_info_model->set('hold_money', $this->member_info->money - $betting_amount);
                        $betting_idx = $betting_check["idx"];
                        $betting_info_model->where("idx", $betting_idx)->update();
                    } else{
                        /* record betting information / 베팅 정보 기록 */
                        $betting_info_model = $betting_info_model->set('member_idx', $this->member_info->idx);
                        $betting_info_model = $betting_info_model->set('status', "B");
                        $betting_info_model = $betting_info_model->set('trx_id', $data["transactionId"]);
                        $betting_info_model = $betting_info_model->set('round_id', $data["gameRound"]);
                        $betting_info_model = $betting_info_model->set('bet_money', $betting_amount);
                        $betting_info_model = $betting_info_model->set('hold_money', $this->member_info->money - $betting_amount);
                        $betting_info_model = $betting_info_model->set('take_money', 0);
                        $betting_info_model = $betting_info_model->set('game_id', $data["gameCode"]);
                        $betting_info_model = $betting_info_model->set('product_id', $product_id_manager_data["product_id"]);
                        $betting_info_model = $betting_info_model->set('product_type_id', $game_information["product_type_id"]);
                        $betting_info_model = $betting_info_model->set('product_group_id', $product_group_info["product_group_id"]);
                        $betting_info_model = $betting_info_model->set('provider_id', CONSTANTS_GOODGAMING);
                        $betting_info_model = $betting_info_model->set('bonus_type', 0);
                        $betting_idx = $insert_betting_info = $betting_info_model->insert();
                    }

                    /* betting callback transaction */
                    $insert["betting_info_id"] = $betting_idx;
                    $insert["callback_type"] = "betting";
                    $insert["trx_id"] = $data["transactionId"];
                    $insert["amount"] = $data["amount"];

                    $insert["payload"] = json_encode($data);
                    $betting_info_callback_model->insert($insert);

                    /* deduct balance to user account / 사용자 계정으로 잔액 공제 */
                    $member_model = $member_model->set('money', 'money-' . $betting_amount, FALSE);
                    $member_model = $member_model->where('idx', $this->member_info->idx);
                    $member_model->update();

                    /* insert log - wallet received (intitialize) / 로그 삽입 - 지갑 수신(초기화) */
                    $t_log_cash_model = new TLogCashModel();
                    $ip = CodeUtil::get_client_ip();
                    $member_money = doubleval($this->member_info->money);
                    $ukey = md5($this->member_info->idx . strtotime('now'));
                    $amount_money = $betting_amount; //amount to convert
                    $before_money = $member_money; //balance before transaction
                    $after_money = $member_money - $betting_amount; //balance after transaction
                    $this->member_info->money = $after_money;
                    $description = "Goodgaming BETTING AMOUNT";

                    /* insert log - wallet received (create query) / 로그 삽입 - 지갑 수신(쿼리 생성) */
                    $ac_code = ADD_BET;
                    $t_log_cash_model = $t_log_cash_model->set('u_key', $ukey);
                    $t_log_cash_model = $t_log_cash_model->set('member_idx', $this->member_info->idx);
                    $t_log_cash_model = $t_log_cash_model->set('ac_code', $ac_code);
                    $t_log_cash_model = $t_log_cash_model->set('ac_idx', 0);
                    $t_log_cash_model = $t_log_cash_model->set('r_money', $amount_money);
                    $t_log_cash_model = $t_log_cash_model->set('be_r_money', $before_money);
                    $t_log_cash_model = $t_log_cash_model->set('af_r_money', $after_money);
                    $t_log_cash_model = $t_log_cash_model->set('m_kind', 'M');
                    $t_log_cash_model = $t_log_cash_model->set('u_ip', $ip);
                    $commend = $data["gameType"] != "SLOT" ? 'casino' : 'slot';
                    $t_log_cash_model = $t_log_cash_model->set('coment', CodeUtil::TLogACCodeToStr($ac_code) . ' ' . $commend);
                    $insert_cash = $t_log_cash_model->insert();
                    $type = $data["gameType"] != "SLOT" ? 'casino' : 'slot';
                    $this->logger->info("~~~ OK ~~~" . $type);

                }
            }
        }
    }
    
    public function result($data, $member_model) {
        /* show logs/ 로그 표시 */
        $this->logger->info(':: Goodgaming:: callbackBetting()');

        /* start transaction / geolae sijag */
        //$this->DB->transBegin();

        $winning_amount = doubleval($data["amount"]);
        $bet_status = "B";

        /* check if bet exist for update */
        $betting_info_model = new TBettingInfo();
        $betting_info_callback_model = new TBettingInfoCallback();

        $betting_check = $betting_info_model->getBettingInfoForUpdateGoodGaming($data, $this->member_info->idx);
        
        $check_betting_transaction = $betting_info_callback_model->where("betting_info_id", $betting_check["idx"])->findAll();
        $b_is_exist = false;
        $b_is_exist_result = false;
        $b_is_win_result = false;

        if ($check_betting_transaction && 0 < count($check_betting_transaction)) {
            foreach ($check_betting_transaction as $tran) {
                if ('result' == $tran['callback_type']) {
                    $b_is_exist_result = true;
                    if($tran["amount"] + $winning_amount > 0){
                        $b_is_win_result = true;
                    }
                }
                if ($data["transactionId"] == $tran['trx_id']) {
                    
                    $b_is_exist = true;
                }

                if (true == $b_is_exist_result && true == $b_is_exist) {
                    break;
                }
            }
        }

        if($data["transactionType"] == 'WIN'){
            $this->logger->info("WIN ~  " . $data["transactionType"]);

            if (!$betting_check) {
                /* get game information */
                return $this->setResult(500, "Betting doesn't exist.");
            }
            if (true == $b_is_exist) {
                return $this->setResult(101, "Trans ID already exist.");
            }
            $betting_transaction = null;
            foreach ($check_betting_transaction as $tran) {
                if($tran["trx_id"] == $data["refererId"]){
                    $betting_transaction = $tran;
                    break;
                }
            }
            if ($betting_transaction == null) {
                /* get game information */
                return $this->setResult(500, "betting_transaction doesn't exist.");
            }
    
            $bet_status = $betting_check["status"];
            $ac_code = BET_RESULT;
            $betting_check["product_type_id"] = $data["game_type"] != 'SLOT' ? PRODUCT_TYPE_CASINO : PRODUCT_TYPE_SLOT;
            $commend = $data["gameType"] != "SLOT" ? 'casino' : 'slot';
            $total_win = $betting_check["take_money"] + $winning_amount;
    
            $transaction_amount = doubleval($betting_transaction["amount"]);

            $this->logger->info("GGG ~  " . $total_win);
            $this->logger->info(':: Goodgaming:: betting_transaction => '.json_encode($betting_transaction));
            if ($total_win > 0) {
                /* show logs/ 로그 표시 */
                $this->logger->info(':: Goodgaming:: amount => ' . $data["amount"]);
                $this->logger->info(':: Goodgaming:: round_id => ' . $data["gameRound"]);
                $this->logger->info(':: Goodgaming:: trans_id => ' . $data["transactionId"]);
                $this->logger->info(':: Goodgaming:: table_code => ' . $data["gameTitle"]);
                $this->logger->info(':: Goodgaming:: date => ' . $data["createdAt"]);
                $this->logger->info(':: Goodgaming:: game_type => ' . $data["game_type"]);
    
                /* reward balance to user account / 사용자 계정으로 잔액 공제 */
                $member_model = $member_model->set('money', 'money+' . $winning_amount, FALSE);
                $member_model = $member_model->where('idx', $this->member_info->idx);
                $member_model->update();
             
                /* only compute rolling comps on first result */
                if (false == $b_is_exist_result && $betting_check["status"] == "B") {
                    /* betting money and winning money same = rollback */
                    if ($winning_amount == $transaction_amount && $commend == "casino") {
                        /* rollback rolling comps / user pay back / charge bet money */
                        $ac_code = BET_REFUND;
                        $bet_status = "C"; // draw
                        Calculate::updateChargeBetMoney($this->member_info->idx, strtoupper($commend), $transaction_amount, $member_model, $this->logger);
                    } else { // win
                        $bet_status = "W"; //
                        UserBettingLose::RewardWinComps($this->member_info->idx, strtoupper($commend), $transaction_amount, $this->logger, $member_model);
                    }
                }
            } else if ($betting_check["status"] == "B" && false == $b_is_exist_result && 0 == $winning_amount) {
                $bet_status = "L"; // lose
                /* betting lose */
                UserBettingLose::RewardLoseComps($this->member_info->idx, $commend, $transaction_amount, $this->logger, $member_model);
            }
    
            if ('B' == $bet_status) {
                $this->logger->info(':: Goodgaming:: status error winning_amount => ' . $winning_amount);
                $this->logger->info(':: Goodgaming:: status error b_is_exist_result => ' . $b_is_exist_result);
                $this->logger->info(':: Goodgaming:: status error status => ' . $betting_check["status"]);
                $this->logger->info(':: Goodgaming:: status error product_type_id => ' . $betting_check["product_type_id"]);
                $this->logger->info(':: Goodgaming:: status error commend => ' . $commend);
            }
            $this->logger->info(':: Goodgaming:: betting_check => '.json_encode($betting_check));
            if ("C" != $bet_status && $betting_check["status"] == "B" && false == $b_is_exist_result) {
                $this->logger->info(':: Goodgaming:: RewardBettingComps => ');
                Calculate::updateChargeBetMoney($this->member_info->idx, strtoupper($commend), $transaction_amount, $member_model, $this->logger);
                UserPayBack::AddBetting($this->member_info->idx, $transaction_amount, $member_model);
                UserBettingLose::RewardBettingComps($this->member_info->idx, $commend, $transaction_amount, $this->logger, $member_model);
            }

            if(($total_win > 0 || $b_is_win_result == true) && $winning_amount != $transaction_amount){
                $bet_status = 'W';
            }
    
            $this->logger->info(':: Goodgaming:: setStatus(' . $bet_status . ')');
            $this->logger->info(':: Goodgaming:: setWhereRoundId(' . $data["gameRound"] . ')');
            $this->logger->info(':: Goodgaming:: setWhereGameId(' . $data["gameCode"] . ')');
    
            if($betting_check){
                $betting_idx = $betting_check["idx"];
                $betting_info_models = $betting_info_model->updateBettingInfo($winning_amount, $bet_status, $data['transactionId'], $betting_idx);
            } else {
                /* record betting information / 베팅 정보 기록 */
                $betting_info_model = $betting_info_model->set('status', $bet_status); // 0 Pending - 1 Win - 2 Lose
                $betting_info_model = $betting_info_model->set('take_money', $total_win);

                if( $winning_amount > 0) {
                    $betting_info_model = $betting_info_model->set('trx_id', $data["transactionId"]);
                }

                $betting_info_model = $betting_info_model->where('round_id', $data["gameRound"]);

                $betting_info_model = $betting_info_model->where('game_id', $data["gameCode"]);
                $betting_info_model = $betting_info_model->where('member_idx', $this->member_info->idx);
                $update_betting_info = $betting_info_model->update();
            }
    
            /* betting callback transaction */
            $insert["betting_info_id"] = $betting_check["idx"];
            $insert["callback_type"] = "result";
            $insert["trx_id"] = $data["transactionId"];
            $insert["amount"] = $data["amount"];
            $insert["payload"] = json_encode($data);
            $betting_info_callback_model->insert($insert);
    
            /* insert log - wallet received (intitialize) / 로그 삽입 - 지갑 수신(초기화) */
            $t_log_cash_model = new TLogCashModel();
            $ip = CodeUtil::get_client_ip();
            $member_money = doubleval($this->member_info->money);
            $ukey = md5($this->member_info->idx . strtotime('now'));
            $amount_money = $winning_amount; //amount to convert
            $before_money = $member_money; //balance before transaction
            $after_money = $member_money + $winning_amount; //balance after transaction
            $this->member_info->money = $after_money;
            $description = "Goodgaming BETTING AMOUNT";
    
    
            $t_log_cash_model = $t_log_cash_model->set('u_key', $ukey);
            $t_log_cash_model = $t_log_cash_model->set('member_idx', $this->member_info->idx);
            $t_log_cash_model = $t_log_cash_model->set('ac_code', $ac_code);
            $t_log_cash_model = $t_log_cash_model->set('ac_idx', 0);
            $t_log_cash_model = $t_log_cash_model->set('r_money', $amount_money);
            $t_log_cash_model = $t_log_cash_model->set('be_r_money', $before_money);
            $t_log_cash_model = $t_log_cash_model->set('af_r_money', $after_money);
            $t_log_cash_model = $t_log_cash_model->set('m_kind', 'P');
            $t_log_cash_model = $t_log_cash_model->set('u_ip', $ip);
    
            $t_log_cash_model = $t_log_cash_model->set('coment', CodeUtil::TLogACCodeToStr($ac_code) . ' ' . $commend);
            $insert_cash = $t_log_cash_model->insert();

        } else if($data["transactionType"] == 'CANCEL'){
            $this->logger->info("CANCEL ~  " . $data["transactionType"]);

            if (true == $b_is_exist) {
                return $this->setResult(101, "Trans ID already exist.");
            }

            $bet_status = $betting_check["status"];
            $ac_code = BET_RESULT;
            $betting_check["product_type_id"] = $data["game_type"] != 'SLOT' ? PRODUCT_TYPE_CASINO : PRODUCT_TYPE_SLOT;
            $commend = $data["gameType"] != "SLOT" ? 'casino' : 'slot';

            if ($winning_amount > 0) {
                $member_model = $member_model->set('money', 'money+' . $winning_amount, FALSE);
                $member_model = $member_model->where('idx', $this->member_info->idx);
                $member_model->update();

                $bet_status = "R";
            }

            $this->logger->info(':: Goodgaming:: setStatus(' . $bet_status . ')');
            /* record betting information / 베팅 정보 기록 */
            $betting_info_model = $betting_info_model->set('status', $bet_status); // 0 Pending - 1 Win - 2 Lose
            $betting_info_model = $betting_info_model->set('take_money', $winning_amount);
            
            if( $winning_amount > 0) {
                $betting_info_model = $betting_info_model->set('trx_id', $data["transactionId"]);
            }
            
            $betting_info_model = $betting_info_model->where('round_id', $data["gameRound"]);
      
            $betting_info_model = $betting_info_model->where('game_id', $data["gameCode"]);
            $betting_info_model = $betting_info_model->where('member_idx', $this->member_info->idx);
            $update_betting_info = $betting_info_model->update();
    
            /* betting callback transaction */
            $insert["betting_info_id"] = $betting_check["idx"];
            $insert["callback_type"] = "result";
            $insert["trx_id"] = $data["transactionId"];
            $insert["amount"] = $data["amount"];
            $insert["payload"] = json_encode($data);
            $betting_info_callback_model->insert($insert);

            $member_money = doubleval($this->member_info->money);
            $ukey = md5($this->member_info->idx . strtotime('now'));
            $amount_money = $winning_amount; //amount to convert
            $before_money = $member_money; //balance before transaction
            $after_money = $member_money - $winning_amount; //balance after transaction
            $this->member_info->money = $after_money;

        } else if($data["transactionType"] == 'PROMO_WIN'){
            $this->logger->info("PROMO_WIN ~  " . $data["transactionType"]);

            if (true == $b_is_exist) {
                return $this->setResult(101, "Trans ID already exist.");
            }

            $bet_status = $betting_check["status"];
            $ac_code = BET_RESULT;
            $betting_check["product_type_id"] = $data["game_type"] != 'SLOT' ? PRODUCT_TYPE_CASINO : PRODUCT_TYPE_SLOT;
            $commend = $data["gameType"] != "SLOT" ? 'casino' : 'slot';
    
            if ($winning_amount > 0) {
                /* show logs/ 로그 표시 */
                $this->logger->info(':: Goodgaming:: amount => ' . $data["amount"]);
                $this->logger->info(':: Goodgaming:: round_id => ' . $data["gameRound"]);
                $this->logger->info(':: Goodgaming:: trans_id => ' . $data["transactionId"]);
                $this->logger->info(':: Goodgaming:: table_code => ' . $data["gameTitle"]);
                $this->logger->info(':: Goodgaming:: date => ' . $data["createdAt"]);
                $this->logger->info(':: Goodgaming:: game_type => ' . $data["game_type"]);
    
                /* reward balance to user account / 사용자 계정으로 잔액 공제 */
                $member_model = $member_model->set('money', 'money+' . $winning_amount, FALSE);
                $member_model = $member_model->where('idx', $this->member_info->idx);
                $member_model->update();
             
                $bet_status = "P";
            }
    
            $this->logger->info(':: Goodgaming:: setStatus(' . $bet_status . ')');
            $this->logger->info(':: Goodgaming:: setWhereRoundId(' . $data['details']['campaignId'] . ')');
            $this->logger->info(':: Goodgaming:: setWhereGameId(' . $data['details']['campaignType'] . ')');

            /* record betting information / 베팅 정보 기록 */
            $betting_info_model = $betting_info_model->set('status', $bet_status); // 0 Pending - 1 Win - 2 Lose
            $betting_info_model = $betting_info_model->set('take_money', $winning_amount);
            
            if( $winning_amount > 0) {
                $betting_info_model = $betting_info_model->set('trx_id', $data["transactionId"]);
            }
            $betting_info_model = $betting_info_model->where('member_idx', $this->member_info->idx);
            $betting_info_model = $betting_info_model->where('round_id', $data["gameRound"]); //고쳐야함
            $betting_info_model = $betting_info_model->where('game_id', $data["gameCode"]); //고쳐야함
            $update_betting_info = $betting_info_model->update();
    
            /* betting callback transaction */
            $insert["betting_info_id"] = $betting_check["idx"];
            $insert["callback_type"] = "result";
            $insert["trx_id"] = $data["transactionId"];
            $insert["amount"] = $data["amount"];
            $insert["payload"] = json_encode($data);
            $betting_info_callback_model->insert($insert);
    
            /* insert log - wallet received (intitialize) / 로그 삽입 - 지갑 수신(초기화) */
            $t_log_cash_model = new TLogCashModel();
            $ip = CodeUtil::get_client_ip();
            $member_money = doubleval($this->member_info->money);
            $ukey = md5($this->member_info->idx . strtotime('now'));
            $amount_money = $winning_amount; //amount to convert
            $before_money = $member_money; //balance before transaction
            $after_money = $member_money + $winning_amount; //balance after transaction
            $this->member_info->money = $after_money;

        } else if($data["transactionType"] == 'EXCEED_WIN_LIMIT'){
            $this->logger->info("EXCEED_WIN_LIMIT ~  " . $data["transactionType"]);
            $this->logger->info("EXCEED_WIN_LIMIT trx_id ~ : " . $betting_check['trx_id']);

            if (true == $b_is_exist) {
                return $this->setResult(101, "Trans ID already exist.");
            }

            $bet_status = $betting_check["status"];
            $ac_code = BET_RESULT;
            $betting_check["product_type_id"] = $data["game_type"] != 'SLOT' ? PRODUCT_TYPE_CASINO : PRODUCT_TYPE_SLOT;
            $commend = $data["gameType"] != "SLOT" ? 'casino' : 'slot';

            if ($winning_amount > 0) {
                /* show logs/ 로그 표시 */
                $this->logger->info(':: Goodgaming:: amount => ' . $data["amount"]);
                $this->logger->info(':: Goodgaming:: round_id => ' . $data["gameRound"]);
                $this->logger->info(':: Goodgaming:: trans_id => ' . $data["transactionId"]);
                $this->logger->info(':: Goodgaming:: table_code => ' . $data["gameTitle"]);
                $this->logger->info(':: Goodgaming:: date => ' . $data["createdAt"]);
                $this->logger->info(':: Goodgaming:: game_type => ' . $data["game_type"]);
    
                /* reward balance to user account / 사용자 계정으로 잔액 공제 */
                $member_model = $member_model->set('money', 'money-' . $winning_amount, FALSE);
                $member_model = $member_model->where('idx', $this->member_info->idx);
                $member_model->update();

                /* record betting information / 베팅 정보 기록 */
                $betting_info_model = $betting_info_model->set('take_money', 'take_money-'. $winning_amount, FALSE);
                $betting_info_model = $betting_info_model->where('member_idx', $this->member_info->idx);
                $betting_info_model = $betting_info_model->where('round_id', $data["gameRound"]);
                $betting_info_model = $betting_info_model->where('game_id', $data["gameCode"]);
                $update_betting_info = $betting_info_model->update();

                $betting_info_model = $betting_info_model->set('trx_id', $data["transactionId"]);
                $betting_info_model = $betting_info_model->where('member_idx', $this->member_info->idx);
                $betting_info_model = $betting_info_model->where('round_id', $data["gameRound"]);
                $betting_info_model = $betting_info_model->where('game_id', $data["gameCode"]);
                $update_betting_info = $betting_info_model->update();

                /* betting callback transaction */
                $insert["betting_info_id"] = $betting_check["idx"];
                $insert["callback_type"] = "result";
                $insert["trx_id"] = $data["transactionId"];
                $insert["amount"] = $data["amount"];
                $insert["payload"] = json_encode($data);
                $betting_info_callback_model->insert($insert);

                $member_money = doubleval($this->member_info->money);
                $ukey = md5($this->member_info->idx . strtotime('now'));
                $amount_money = $winning_amount; //amount to convert
                $before_money = $member_money; //balance before transaction
                $after_money = $member_money - $winning_amount; //balance after transaction
                $this->member_info->money = $after_money;

            }

        } else {
            return $this->setResult(500, "TransactionType is not defined.");
        }

        /* commit or rollback if there is no error / 오류가 없으면 커밋 또는 롤백 */
        /*if ($this->DB->transStatus() === false) {
            $this->DB->transRollback();
            $this->setResult(400, 'Database Error');
        } else {
            $this->DB->transCommit();
        }*/
    }
    private function __sendTelegramMessageErrorOpeningGame($error) {
        $telegram_class = new TelegramClass();
        $send_message = "An error has occurred while opening a game in Goodgaming. Please check the details below. \n\n";
        $send_message .= "Project: <code>" . $this->project . "</code> \n";
        $send_message .= "Operator Name: <code>" . $this->operator_name . "</code> \n";
        $send_message .= "Date/Time: <code>" . date('m/d/Y h:i:s A', time()) . "</code> \n";
        $send_message .= "Game ID: <code>" . $this->game_information["game_id"] . "</code> \n";
        $send_message .= "Game Name (EN): <code>" . $this->game_information["gamet_name_en"] . "</code> \n";
        $send_message .= "Game Name (KR): <code>" . $this->game_information["game_name_kr"] . "</code> \n";
        $send_message .= "Provider Name (EN): <code>" . $this->product_group_info["desc"] . "</code> \n";
        $send_message .= "Provider Name (KR): <code>" . $this->product_group_info["product_group_name"] . "</code> \n";
        $send_message .= "Error Code: <code>" . $error["result"] . "</code> \n";
        $send_message .= "Error Message: <code>" . $error["msg"] . "</code> \n\n";
        $send_message .= "Full Error Details: \n<code>" . serialize($error) . "</code> \n\n";
        $telegram_class->send('Goodgaming_monitoring', $send_message);
    }
    private function sendAPIPatch($api_path, $params) {
        /* create a URL for the API request */
        $url = $this->api_url . $api_path ;

        $data = array(
            'vendorCode' => $params["vendorCode"],
            // 'vendorCode' => "Evolution",
            'username' => $params["username"]
        );

        $jsondata = json_encode($data);

        /* send api request via CURL */
        $authorizationHeader = $this->opkey;

        $auth = 'Authorization: Bearer '.$authorizationHeader;
        $headers = [$auth,
                    'Content-Type: application/json',
                    'Accept: application/json',
                    'Access-Control-Allow-Origin:*',
                    'Access-Control-Allow-Methods: GET,POST,OPTIONS'];
        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'PATCH');
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $jsondata);
        $res = curl_exec($curl);

        if (curl_errno($curl)) {
            echo "cURL error: " . curl_error($curl);
        }

        curl_close($curl);

        return $res;
    }
    private function sendAPICreatePost($api_path, $params) {
        /* create a URL for the API post */
        $url = $this->api_url . $api_path;

        $data = array(
            'username' => $params["username"],
            'nickname' => $params["nickname"],
        );

        $body = json_encode($data);

        /* send api request via CURL */
        $curl = curl_init($url);
        $authorizationHeader = $this->opkey;
        $headers = array('Authorization: Bearer '.$authorizationHeader,
        'Content-Type: application/json');
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $body);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($curl);

        if (curl_errno($curl)) {
            echo "cURL error: " . curl_error($curl);
        }

        curl_close($curl);

        return $res;
    }
    private function sendAPIRequest($api_path, $params) {
        /* create a URL for the API request */
        $params["opkey"] = $this->opkey;
        $params["hash"] = md5($this->opkey . "|" . http_build_query($params));
              
        $url = $this->api_url . $api_path . "?" . http_build_query($params);

        /* send api request via CURL */
        $headers = [];
        $curl = curl_init();
        
        curl_setopt($curl, CURLOPT_URL, $url);
        $authorizationHeader = $this->opkey;
        $headers = ['Authorization: Bearer '.$authorizationHeader,
                    'Content-Type: application/json',];

        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($curl);

        if (curl_errno($curl)) {
            echo "cURL error: " . curl_error($curl);
        }

        curl_close($curl);
        return $res;
    }

    private function __createAccountOnGoodGaming() {
        $params["username"] = "goodgaming".$this->member_info->idx;
        $params["nickname"] = "goodgaming".$this->member_info->idx;
        $response = json_decode($this->sendAPICreatePost("/user", $params), true);

        if(array_key_exists('resultCode', $response)){
            if($response["resultCode"] == "ERROR"){
                $this->logger->info("__createAccountOnGoodGaming_____resultCode : ".$response);
            }
        }
    }
    private function __getGameList($game_company_code) {
        $params["vendorCode"] = $game_company_code;
        $params["gameType"] = '';
        $api_response = $this->sendAPIRequest('/game', $params);
        $_game_list = json_decode($api_response, true);
        return $_game_list;
    }

    // private function __checkSaveGames($product_group_info, $_game_list) {
    //     foreach ($_game_list as $game) {
    //         $tb_static_game_id_manager = new TGameIdManager();
    //         $check_exist = $tb_static_game_id_manager->getGameInformation($game["code"], 4, $product_group_info["product_group_id"]);

    //         $date_time = new DateTime($game["date"]);
    //         $game_date = $date_time->format('Y-m-d H:i:s');

    //         $insert_game = [];
    //         $insert_game["game_short_name"] = $game["name_kor"];
    //         $insert_game["gamet_name_en"] = $game["name_eng"];
    //         $insert_game["game_name_kr"] = $game["name_kor"];
    //         $insert_game["game_image"] = $game["img_1"];
    //         $insert_game["is_desktop"] = $game["is_desktop"];
    //         $insert_game["is_mobile"] = $game["is_mobile"];
    //         $insert_game["game_date"] = $game_date;
    //         $insert_game["updated_at"] = date('Y-m-d H:i:s');

    //         if ($check_exist) {
    //             $tb_static_game_id_manager->set('game_short_name', $game["name_kor"]);
    //             $tb_static_game_id_manager->set('gamet_name_en', $game["name_eng"]);
    //             $tb_static_game_id_manager->set('game_name_kr', $game["name_kor"]);
    //             $tb_static_game_id_manager->set('game_image', $game["img_1"]);
    //             $tb_static_game_id_manager->set('is_desktop', $game["is_desktop"]);
    //             $tb_static_game_id_manager->set('is_mobile', $game["is_mobile"]);
    //             $tb_static_game_id_manager->set('game_date', $game_date);
    //             $tb_static_game_id_manager->set('updated_at', date('Y-m-d H:i:s'));
    //             $tb_static_game_id_manager->set('product_type_id', $product_group_info["product_type_id"]);
    //             $tb_static_game_id_manager->where('provider_id', 3)->where('game_id', $game["code"])->where('product_group_id', $product_group_info["product_group_id"])->update();
    //             $this->logger->info("- Game ID (" . $game["code"] . ") - (" . $product_group_info["desc"] . ") updated." . "(" . $product_group_info["product_type_id"] . ")");
    //         } else {
    //             $insert_game["created_at"] = date('Y-m-d H:i:s');
    //             $insert_game["provider_id"] = 3;
    //             $insert_game["product_group_id"] = $product_group_info["product_group_id"];
    //             $insert_game["game_id"] = $game["code"];
    //             $insert_game["product_type_id"] = $product_group_info["product_type_id"];
    //             $tb_static_game_id_manager->insert($insert_game);
    //             $this->logger->info("- Game ID (" . $game["code"] . ") - (" . $product_group_info["desc"] . ") created.");
    //         }
    //     }
    // }

    private function __checkSaveGameCompany($reference, $data) {
        
    }

    public function __setAPIResponse($response_code, $message, $additional_response = null) {
        $response["result"] = $response_code;

        if ($this->player_information && $response_code == 1) {
            $response["balance"] = $this->player_information->account_money;
            $response["player_id"] = $this->player_information->player_id;
            $response["username"] = $this->player_information->username;
            $response["nickname"] = $this->player_information->nickname;
        }

        if ($additional_response) {
            foreach ($additional_response as $key => $additional) {
                $response[$key] = $additional;
            }
        }

        $response["message"] = $message;

        if ($response_code !== 1) {
            $this->is_error = true;
        }

        $this->api_response = $response;
    }

    public function setResult($code, $message = null, $balance = 0) {
        $this->result["return_code"] = $code;
        $this->result["message"] = $message;

        if ($code == 1) {
            $this->result["balance"] = $this->member_info->money;
        }
    }
    private function getAPIUrl($api_path, $params) {
        /* create a URL for the API request */
        $url = $this->api_url . $api_path . "/" . $params . "/result";

        /* send api request via CURL */
        $headers = [];
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        $authorizationHeader = $this->opkey;
        $headers = ['Authorization: Bearer '.$authorizationHeader,
                    'Content-Type: application/json',];
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($curl);

        if (curl_errno($curl)) {
            echo "cURL error: " . curl_error($curl);
        }

        curl_close($curl);

        return $res;
    }
    public function getRoundResult($trans_id) {
        $response = json_decode($this->getAPIUrl("/bet", $trans_id), true);

        $loggableData = json_encode($response, JSON_PRETTY_PRINT);
        $this->logger->info($trans_id);
        $this->logger->info('Response Array:', ['response' => $loggableData]);

        if(array_key_exists('resultType', $response)){
            if($response["resultType"] == "URL"){
                return $response["value"];
            } else if($response["resultType"] == "JSON"){
                echo "<pre>";
                print_r($response);
                echo "</pre>";
                die();
            }
        } else if (isset($response['resultCode']) && $response['resultCode'] == "GAME_DETAIL_NOT_READY") {
            $this->returnError($response["response"]);
        } else {
            $this->returnError($response["message"]);
        }
    }
}
