<?php

namespace App\Game;

use App\Models\TGameIdManager;
use App\Classes\TelegramClass;
use App\Models\TProductGroupManager;
use App\Models\TLogCashModel;
use App\Models\TBettingInfo;
use App\Models\TProductIdManager;
use App\Models\TBettingInfoCallback;
use App\Util\CodeUtil;
use App\Util\Calculate;
use App\RollingComps\UserPayBack;
use App\RollingComps\UserBettingLose;
use DateTime;

class Slotbank extends GameAPI {

    private $opkey = null;
    private $api_url = null;
    private $provider_info = null;
    private $log_channel = null;
    private $log_file = null;
    private $request = null;
    private $member_info = null;
    private $product_group_info = null;
    private $game_information;

    public function __construct($logger) {
        $this->logger = $logger;
        $this->project = 'Noble';
        $this->operator_name = 'noble01';
        $this->DB = \Config\Database::connect();

        /* test creds */
        $this->opkey = '6972dcac8d4e0052834981a2157df3be';
        $this->api_url = 'https://api2.popsapi.com';

        /* live creds */
        if (ENVIRONMENT == "production") {
            $this->opkey = '6972dcac8d4e0052834981a2157df3be';
            $this->api_url = 'https://api2.popsapi.com';
        }
    }

    public function isLoginById($user_id, $function_name, $member_model) {
        $this->logger->info(':: SlotBank:: callback (' . $user_id . ', ' . $function_name . ')');

        // check of logged-in
        $member = $member_model->setMemberWhereId($user_id);

        if ($member == null) {
            return [null, '조회되는 회원 또는 메세지 idx 가 없습니다.'];
        }

        if ($member->getUBusiness() != 1) {
            return [null, '총판은 배팅 이용이 불가능합니다.'];
        }

        if ($member->getStatus() == 11) {
            $response['messages'] = '관리자 승인이 필요합니다.';
            return [null, '관리자 승인이 필요합니다.'];
        }

        if ('credit' != $function_name && 'cancel' != $function_name) {
            if ($member->getStatus() == 2 || $member->getStatus() == 3) {
                return [null, '사용이 불가능 한 계정으로 관리자에게 문의바랍니다.'];
            }
        }

        return [$member, ''];
    }

    function processRequest($member, $member_model, $function_name, $data) {
        $this->function_name = $function_name;
        $this->opkey = $data["opkey"];
        $this->userid = $data["userid"];
        $this->token = $data["token"];
        $this->third_party_code = $data["third_party_code"];
        $this->third_party_name = $data["third_party_name"];
        $this->times = $data["times"];
        $this->member_info = $member;

        /* show logs / 로그 표시 */
        $this->logger->info(':: SlotBank:: callback (' . $this->third_party_code . ', ' . $this->third_party_name . ', ' . $this->member_info->id . ', ' . $this->member_info->idx . ', ' . $this->member_info->money . ')');

        /* callback validation / 콜백 검증 */
        $validation_response = $this->callbackValidation($data, $function_name, $member, $member_model);

        if ($validation_response["status"] == 1) {
            $validation_response["member_info"];
            $this->setResult(1, 'Callback Successful');
            $this->$function_name($data, $member_model);
        }

        /* return information as callback response */
        if ($this->result["return_code"] != 1) {
            $this->logger->info(':: Slotbank:: error (' . $this->result["message"] . ') ');
            return [$this->result["return_code"], 0, $data, $this->result["message"]];
        } else {
            $this->setResult(1);
            return [$this->result["return_code"], $this->result["balance"], $data, $this->result["message"]];
        }
    }

    public function callbackValidation($data, $function_name, $member, $member_model) {
        $this->logger->info(':: SlotBank: callbackValidation');
        $this->logger->info($data);
        /* check if required parameters exist / 필수 매개변수가 있는지 확인 */
        if (!isset($this->userid) || !isset($this->token) || !isset($this->opkey)) {
            $this->logger->error(':: SlotBank::' . $function_name . ' => Missing Parameters');
            $this->setResult(99, 'Missing Parameters');
            return ['status' => 0, 'member_info' => $this->member_info, 'message' => 'Missing Parameters'];
        }

        /* check if user information exist / 사용자 정보가 존재하는지 확인 */
        if (!isset($this->member_info) || !$this->member_info) {
            $this->logger->error(':: SlotBankController::' . $function_name . ' => Invalid User ID');
            $this->setResult(20, 'Invalid User ID');
            return ['status' => 0, 'member_info' => $this->member_info, 'message' => 'Invalid User ID'];
        }

        $client_ip = CodeUtil::get_client_ip();
        $sql = "select * from tb_static_whitelist_slotbank where ip = ?";
        $result = $member_model->db->query($sql, [$client_ip])->getResultArray();

        if (!$result || 0 == count($result)) {
            $this->logger->error(':: SlotBankController::' . $function_name . ' => Invalid IP address' . ' client ip :' . $client_ip);
            $this->setResult(99, 'Invalid IP address');
            return ['status' => 0, 'member_info' => $this->member_info, 'message' => 'Invalid IP address'];
        }

        return ['status' => 1, 'member_info' => $this->member_info, 'message' => 'Missing Parameters'];
    }

    public function fetchAndSaveGameList() {
        $params = [];
        $api_response = $this->sendAPIRequest('/provlist', $params);
        $_game_companies = json_decode($api_response, true)["data"]["list"];
        $tb_static_product_group_manager = new TProductGroupManager();

        $this->logger->info('');
        $this->logger->info('');
        $this->logger->info('');
        $this->logger->info('[[ SLOTBANK LIST OF GAMES UPDATE ]]');
        $this->logger->info('[[ ' . date('m/d/Y h:i:s A', time()) . ' ]]');

        foreach ($_game_companies as $key => $game_company) {
            /* check if game company is supported */
            $product_group_info = $tb_static_product_group_manager->where("slotbank_id", $game_company["code"])->first();

            if ($product_group_info) {
                $this->logger->info('');
                $this->logger->info("@@@ Provider (" . $game_company["name"] . ") checking list of games.");
                $_game_list = $this->__getGameList($game_company["code"]);
                $this->__checkSaveGames($product_group_info, $_game_list);
            } else {
                $this->logger->info('');
                $this->logger->info("@@@ Provider (" . $game_company["name"] . ") is not supported.");
            }
        }

        return "Game successfully updated.";
    }

    public function getGameURL($member_info, $product_group_info, $game_id = null) {
        $this->member_info = $member_info;
        $this->product_group_info = $product_group_info;

        /* get lobby code if game id is null */
        if (!$game_id) {
            $game_id = $product_group_info["slotbank_lobby_code"];
        }

        /* get game information from database */
        $tb_static_game_id_manager = new TGameIdManager();
        $this->game_information = $tb_static_game_id_manager->getGameInformation($game_id, $product_group_info["provider_id"], $product_group_info["product_group_id"]);

        if (!$this->game_information) {
            $this->returnError("내부 오류: 데이터베이스에서 게임을 찾을 수 없습니다.");
        } else {

            $params["thirdpartycode"] = $this->product_group_info["slotbank_id"];
            $params["userid"] = $this->member_info->idx;
            $params["gamecode"] = $this->game_information["game_id"];
            $params["platform"] = CodeUtil::rtn_mobile_chk() == "PC" ? "DESKTOP" : "MOBILE";
            $params["bet_limit"] = "A";

            $this->__createAccountOnSlotBank();

            $response = json_decode($this->sendAPIRequest("/play", $params), true);

            if ($response["result"] !== 1) {
                if($response['result'] == 4104){
                    $response['code'] = 66;
                    $response['message'] = "현재 해당 게임은 플레이 할 수 없습니다.";
                } else {
                    $telegram_class = new TelegramClass();
                    $this->__sendTelegramMessageErrorOpeningGame($response);
                    // $this->returnError($response["msg"]);
                    $response['message'] = $response['msg'];
                }
            }
            $response['link'] = $response['data']['link'];
            return $response;
        }
    }

    //public function validateCallback($request) {
    //}

    public function member($data, $member_model) {
        $this->__setAPIResponse(1, "Member Successfully Fetched");
    }

    public function balance($data, $member_model) {
        $this->__setAPIResponse(1, "Balance Successfully Fetched");
    }

    public function betting($data, $member_model) {
        /* show logs / 로그 표시 */
        $this->logger->info(':: SlotBank:: callbackBetting()');
        try {
            /* start transaction / 거래 시작 */
            //$member_model->db->transStart();

            $betting_amount = doubleval($data["amount"]);


            $this->logger->info(':: SlotBank:: amount => ' . json_encode($data));

            /* get game information */
            $game_list_model = new TGameIdManager();
            $tb_static_product_group_manager = new TProductGroupManager();
            $betting_info_callback_model = new TBettingInfoCallback();
            $product_group_info = $tb_static_product_group_manager->where("slotbank_id", $data["third_party_code"])->first();
            $game_information = $game_list_model->getGameInformation($data["game_code"], 3, $product_group_info["product_group_id"]);
            $check_betting_transaction = $betting_info_callback_model->where("trx_id", $data["trans_id"])->first();

            $product_id_manager_model = new TProductIdManager();
            $product_id_manager_data = $product_id_manager_model->where('product_id', $data["third_party_code"])->first();


            /* insert game information if it doesn't exist */
            if (!$game_information) {
                // $product_id_manager_model = new TProductIdManager();
                // $product_id_manager_data = $product_id_manager_model->where('product_id', $data["third_party_code"])->first();

                $game_list_model->set('game_id', $data["game_code"]);
                $game_list_model->set('game_short_name', $data["table_code"]);
                $game_list_model->set('gamet_name_en', $data["table_code"]);
                $game_list_model->set('game_name_kr', $data["table_code"]);
                $game_list_model->set('is_use', "ON");
                $game_list_model->set('status', 1);
                $game_list_model->set('product_id', $product_id_manager_data["product_id"]);
                $game_list_model->set('provider_id', 3);
                $game_list_model->set('product_type_id', $product_id_manager_data["product_type_id"]);
                $game_list_model->set('product_group_id', $product_group_info["product_group_id"]);
                $game_list_model->insert();

                // $game_information = $game_list_model->getGameInformation($data["game_code"], 3 ,$data["third_party_code"]);

                $game_information = $game_list_model->getGameInformation($data["game_code"], 3, $product_group_info["product_group_id"]);
            }

            $game_information["product_type_id"] = $data["game_type"] == 'live' ? PRODUCT_TYPE_CASINO : PRODUCT_TYPE_SLOT;

            if (!$data["game_code"] || !$game_information) {
                $this->logger->info(':: SlotBank:: Game ID Not Found => ' . $data["game_code"]);
                $this->setResult(99, 'Game Information Not Found');
            } else if ($check_betting_transaction) {
                $this->setResult(1, 'Trans ID already exist.');
            }  else {
                /* record betting amount / 기록적인 베팅 금액 */
                if ($this->member_info->money < $betting_amount) {
                    $this->setResult(20, 'Inefficient Balance');
                } else if ($betting_amount >= 0) {
                    if ((0 < $game_information["min_bet"] && $betting_amount < $game_information["min_bet"]) ||
                            (0 < $game_information["max_bet"] && $game_information["max_bet"] < $betting_amount)
                    ) {
                        $this->logger->info(':: SlotBank:: Game ID Not Found => ' . $data["game_code"]);
                        $this->setResult(20, 'Inefficient Balance');
                    } else {

                        /* check if bet exist for update */
                        $betting_info_model = new TBettingInfo();
                        $betting_check = $betting_info_model->getBettingInfoForUpdate($data);

                        if ($betting_check) {
                            $total_bet_money = floatval($betting_check["bet_money"]) + $betting_amount;
                            $betting_info_model = $betting_info_model->set('bet_money', $total_bet_money);
                            $betting_info_model = $betting_info_model->set('hold_money', $this->member_info->money - $betting_amount);
                            $betting_idx = $betting_check["idx"];
                            $betting_info_model->where("idx", $betting_idx)->update();
                        } else {

                            // 해당 배팅이 있는지 먼저 체크한다. for update
                            /* record betting information / 베팅 정보 기록 */
                            $betting_info_model = $betting_info_model->set('member_idx', $this->member_info->idx);
                            $betting_info_model = $betting_info_model->set('status', "B");
                            $betting_info_model = $betting_info_model->set('trx_id', $data["trans_id"]);
                            $betting_info_model = $betting_info_model->set('round_id', $data["round_id"]);
                            $betting_info_model = $betting_info_model->set('bet_money', $betting_amount);
                            $betting_info_model = $betting_info_model->set('hold_money', $this->member_info->money - $betting_amount);
                            $betting_info_model = $betting_info_model->set('take_money', 0);
                            $betting_info_model = $betting_info_model->set('game_id', $data["game_code"]);
                            $betting_info_model = $betting_info_model->set('product_id', $product_id_manager_data["product_id"]);
                            $betting_info_model = $betting_info_model->set('product_type_id', $game_information["product_type_id"]);
                            $betting_info_model = $betting_info_model->set('product_group_id', $game_information["product_group_id"]);
                            $betting_info_model = $betting_info_model->set('provider_id', CONSTANTS_SLOTBANK);
                            $betting_info_model = $betting_info_model->set('bonus_type', 0);
                            $betting_idx = $insert_betting_info = $betting_info_model->insert();
                        }
                        

                        /* betting callback transaction */
                        $insert["betting_info_id"] = $betting_idx;
                        $insert["callback_type"] = "betting";
                        $insert["trx_id"] = $data["trans_id"];
                        $insert["amount"] = $data["amount"];

                        $insert["payload"] = json_encode($data);
                        $betting_info_callback_model->insert($insert);

                        /* show logs / 로그 표시 */
                        $this->logger->info(':: SlotBank:: amount => ' . $data["amount"]);
                        $this->logger->info(':: SlotBank:: round_id => ' . $data["round_id"]);
                        $this->logger->info(':: SlotBank:: trans_id => ' . $data["trans_id"]);

                        /* deduct balance to user account / 사용자 계정으로 잔액 공제 */
                        $member_model = $member_model->set('money', 'money-' . $betting_amount, FALSE);
                        $member_model = $member_model->where('idx', $this->member_info->idx);
                        $member_model->update();

                        /* insert log - wallet received (intitialize) / 로그 삽입 - 지갑 수신(초기화) */
                        $t_log_cash_model = new TLogCashModel();
                        $ip = CodeUtil::get_client_ip();
                        $member_money = doubleval($this->member_info->money);
                        $ukey = md5($this->member_info->idx . strtotime('now'));
                        $amount_money = $betting_amount; //amount to convert
                        $before_money = $member_money; //balance before transaction
                        $after_money = $member_money - $betting_amount; //balance after transaction
                        $this->member_info->money = $after_money;
                        $description = "SLOTBANK BETTING AMOUNT";

                        /* insert log - wallet received (create query) / 로그 삽입 - 지갑 수신(쿼리 생성) */
                        $ac_code = ADD_BET;
                        $t_log_cash_model = $t_log_cash_model->set('u_key', $ukey);
                        $t_log_cash_model = $t_log_cash_model->set('member_idx', $this->member_info->idx);
                        $t_log_cash_model = $t_log_cash_model->set('ac_code', $ac_code);
                        $t_log_cash_model = $t_log_cash_model->set('ac_idx', 0);
                        $t_log_cash_model = $t_log_cash_model->set('r_money', $amount_money);
                        $t_log_cash_model = $t_log_cash_model->set('be_r_money', $before_money);
                        $t_log_cash_model = $t_log_cash_model->set('af_r_money', $after_money);
                        $t_log_cash_model = $t_log_cash_model->set('m_kind', 'M');
                        $t_log_cash_model = $t_log_cash_model->set('u_ip', $ip);
                        $commend = $product_id_manager_data["product_type_id"] == 1 ? 'casino' : 'slot';
                        $t_log_cash_model = $t_log_cash_model->set('coment', CodeUtil::TLogACCodeToStr($ac_code) . ' ' . $commend);
                        $insert_cash = $t_log_cash_model->insert();
                        $type = $product_id_manager_data["product_type_id"] == 1 ? 'casino' : 'slot';

                    }
                }
            }
        } catch (\mysqli_sql_exception $e) {
            $this->logger->error(':::::::::::::::  betting error : ' . $e->getMessage());
            $this->logger->error(':::::::::::::::  betting query : ' . $member_model->getLastQuery());
            //$member_model->db->transRollback();
            return $this->setResult(400, 'Database Error');
        }
        //}
    }
    
    public function result($data, $member_model) {
        /* show logs/ 로그 표시 */
        $this->logger->info(':: SlotBank:: callbackResult()');

        /* start transaction / geolae sijag */
        //$this->DB->transBegin();

        $winning_amount = doubleval($data["amount"]);
        $bet_status = "B";

        /* check if bet exist for update */
        $betting_info_model = new TBettingInfo();
        $betting_info_callback_model = new TBettingInfoCallback();

        $betting_check = $betting_info_model->getBettingInfoForUpdate($data);
        $check_betting_transaction = $betting_info_callback_model->where("betting_info_id", $betting_check["idx"])->findAll();
        $b_is_exist = false;
        $b_is_exist_result = false;
        if ($check_betting_transaction && 0 < count($check_betting_transaction)) {
            foreach ($check_betting_transaction as $tran) {
                if ('result' == $tran['callback_type']) {
                    $b_is_exist_result = true;
                }
                if ($data["trans_id"] == $tran['trx_id']) {
                    $b_is_exist = true;
                }

                if (true == $b_is_exist_result && true == $b_is_exist) {
                    break;
                }
            }
        }

        if (!$betting_check) {
            /* get game information */
            return $this->setResult(90, "Betting doesn't exist.");
        }
        if (true == $b_is_exist) {
            return $this->setResult(1, "Trans ID already exist.");
        }

        $bet_status = $betting_check["status"];
        $ac_code = BET_RESULT;
        $betting_check["product_type_id"] = $data["game_type"] == 'live' ? PRODUCT_TYPE_CASINO : PRODUCT_TYPE_SLOT;
        $commend = $betting_check["product_type_id"] == 1 ? 'casino' : 'slot';
        $total_win = $betting_check["take_money"] + $winning_amount;
        if ($winning_amount > 0) {
            /* show logs/ 로그 표시 */
            $this->logger->info(':: SlotBank:: amount => ' . $data["amount"]);
            $this->logger->info(':: SlotBank:: round_id => ' . $data["round_id"]);
            $this->logger->info(':: SlotBank:: trans_id => ' . $data["trans_id"]);
            $this->logger->info(':: SlotBank:: table_code => ' . $data["table_code"]);
            $this->logger->info(':: SlotBank:: date => ' . $data["date"]);
            $this->logger->info(':: SlotBank:: game_type => ' . $data["game_type"]);

            /* reward balance to user account / 사용자 계정으로 잔액 공제 */
            $member_model = $member_model->set('money', 'money+' . $winning_amount, FALSE);
            $member_model = $member_model->where('idx', $this->member_info->idx);
            $member_model->update();
         
            /* only compute rolling comps on first result */
            if (false == $b_is_exist_result && $betting_check["status"] == "B") {
                /* betting money and winning money same = rollback */
                if ($winning_amount == $betting_check["bet_money"] && $commend == "casino") {
                    /* rollback rolling comps / user pay back / charge bet money */
                    $ac_code = BET_REFUND;
                    $bet_status = "C"; // draw
                    Calculate::updateChargeBetMoney($this->member_info->idx, strtoupper($commend), $betting_check["bet_money"], $member_model, $this->logger);
                    //UserPayBack::AddBetting($this->member_info->idx, $betting_check["bet_money"] * - 1, $member_model);
                    //UserBettingLose::RewardBettingComps($this->member_info->idx, $commend, $betting_check["bet_money"] * - 1, $this->logger, $member_model);
                } else { // win
                    $bet_status = "W"; //
                    UserBettingLose::RewardWinComps($this->member_info->idx, strtoupper($commend), $betting_check["bet_money"], $this->logger, $member_model);
                    //UserBettingLose::RewardDistWinComps($this->member_info->idx, $commend, $betting_check["bet_money"], $this->logger, $member_model);
                    //} else if($winning_amount != $betting_check["bet_money"] && $commend == 'slot'){
                }
            }
        } else if ($betting_check["status"] == "B" && false == $b_is_exist_result && 0 == $winning_amount) {
            $bet_status = "L"; // lose
            /* betting lose */
            //$type = $betting_check["product_type_id"] == 1 ? 'casino' : 'slot';
            UserBettingLose::RewardLoseComps($this->member_info->idx, $commend, $betting_check["bet_money"], $this->logger, $member_model);
            //UserBettingLose::RewardDistLoseComps($this->member_info->idx, $commend, $betting_check["bet_money"], $this->logger, $member_model);
        }

        if ('B' == $bet_status) {
            $this->logger->info(':: SlotBank:: status error winning_amount => ' . $winning_amount);
            $this->logger->info(':: SlotBank:: status error b_is_exist_result => ' . $b_is_exist_result);
            $this->logger->info(':: SlotBank:: status error status => ' . $betting_check["status"]);
            $this->logger->info(':: SlotBank:: status error product_type_id => ' . $betting_check["product_type_id"]);
            $this->logger->info(':: SlotBank:: status error commend => ' . $commend);
        }

        if ("C" != $bet_status && $betting_check["status"] == "B" && false == $b_is_exist_result) {
            Calculate::updateChargeBetMoney($this->member_info->idx, strtoupper($commend), $betting_check["bet_money"], $member_model, $this->logger);
            UserPayBack::AddBetting($this->member_info->idx, $betting_check["bet_money"], $member_model);
            UserBettingLose::RewardBettingComps($this->member_info->idx, $commend, $betting_check["bet_money"], $this->logger, $member_model);
        }

        /* record betting information / 베팅 정보 기록 */
        $total_take_money = floatval($betting_check["take_money"]) + $winning_amount;
        /* update win or lose */
        if($total_take_money == $betting_check["bet_money"]) {
            $bet_status = "C";
        }
        else if($total_take_money > 0) {
            $bet_status = "W";
        }
        $this->logger->info(':: SlotBank:: total_take_money => ' . $total_take_money);
        $this->logger->info(':: SlotBank:: winning_amount => ' . $winning_amount);
        $this->logger->info(':: SlotBank:: money => ' . $this->member_info->money);
        $this->logger->info(':: SlotBank:: setStatus(' . $bet_status . ')');
        $this->logger->info(':: SlotBank:: setWhereRoundId(' . $data["round_id"] . ')');
        $this->logger->info(':: SlotBank:: setWhereGameId(' . $data["game_code"] . ')');
        $this->logger->info(':: SlotBank:: betting_check => ' . json_encode($betting_check));

        if($betting_check){
            $betting_idx = $betting_check["idx"];
            $betting_info_models = $betting_info_model->updateBettingInfo($winning_amount, $bet_status, $data['trans_id'], $betting_idx);
        } else {
            $game_list_model = new TGameIdManager();
            $tb_static_product_group_manager = new TProductGroupManager();
            $betting_info_callback_model = new TBettingInfoCallback();
            $product_group_info = $tb_static_product_group_manager->where("slotbank_id", $data["third_party_code"])->first();
            $game_information = $game_list_model->getGameInformation($data["game_code"], 3, $product_group_info["product_group_id"]);

            $product_id_manager_model = new TProductIdManager();
            $product_id_manager_data = $product_id_manager_model->where('product_id', $data["third_party_code"])->first();

            /* insert game information if it doesn't exist */
            if (!$game_information) {
                $game_list_model->set('game_id', $data["game_code"]);
                $game_list_model->set('game_short_name', $data["table_code"]);
                $game_list_model->set('gamet_name_en', $data["table_code"]);
                $game_list_model->set('game_name_kr', $data["table_code"]);
                $game_list_model->set('is_use', "ON");
                $game_list_model->set('status', 1);
                $game_list_model->set('product_id', $product_id_manager_data["product_id"]);
                $game_list_model->set('provider_id', 3);
                $game_list_model->set('product_type_id', $product_id_manager_data["product_type_id"]);
                $game_list_model->set('product_group_id', $product_group_info["product_group_id"]);
                $game_list_model->insert();

                $game_information = $game_list_model->getGameInformation($data["game_code"], 3, $product_group_info["product_group_id"]);
            }

            $game_information["product_type_id"] = $data["game_type"] == 'live' ? PRODUCT_TYPE_CASINO : PRODUCT_TYPE_SLOT;

            $member_money = doubleval($this->member_info->money);
            $betting_info_model = $betting_info_model->set('take_money', $total_take_money);
            $betting_info_model = $betting_info_model->set('status', $bet_status); // 0 Pending - 1 Win - 2 Lose
            $betting_info_model = $betting_info_model->set('member_idx', $this->member_info->idx);
            $betting_info_model = $betting_info_model->set('status', "P");
            $betting_info_model = $betting_info_model->set('trx_id', $data["trans_id"]);
            $betting_info_model = $betting_info_model->set('round_id', $data["round_id"]);
            $betting_info_model = $betting_info_model->set('bet_money', 0);
            $betting_info_model = $betting_info_model->set('hold_money', $member_money + $winning_amount);
            $betting_info_model = $betting_info_model->set('game_id', $data["game_code"]);
            $betting_info_model = $betting_info_model->set('product_id', $product_id_manager_data["product_id"]);
            $betting_info_model = $betting_info_model->set('product_type_id', $game_information["product_type_id"]);
            $betting_info_model = $betting_info_model->set('product_group_id', $game_information["product_group_id"]);
            $betting_info_model = $betting_info_model->set('provider_id', CONSTANTS_SLOTBANK);
            $betting_info_model = $betting_info_model->set('bonus_type', 0);
            $betting_info_model = $betting_info_model->insert();
        }

        $betting_check_last = $betting_info_model->getBettingInfoForUpdate($data);
        $this->logger->info(':: SlotBank:: betting_check_last => ' . json_encode($betting_check_last));

        if(($betting_check_last["status"] == "C" || $betting_check_last["status"] == "L") && $betting_check_last['trx_id'] != $data['trans_id'] && $betting_check_last["take_money"] > $betting_check_last["bet_money"]){
            $bet_status = "W";
            $betting_info_model = $betting_info_model->set('status', $bet_status);
            $betting_idx = $betting_check_last["idx"];
            $betting_info_model->where("idx", $betting_idx)->update();
        }

        /* betting callback transaction */
        $insert["betting_info_id"] = $betting_check["idx"];
        $insert["callback_type"] = "result";
        $insert["trx_id"] = $data["trans_id"];
        $insert["amount"] = $data["amount"];
        $insert["payload"] = json_encode($data);
        $betting_info_callback_model->insert($insert);

        /* insert log - wallet received (intitialize) / 로그 삽입 - 지갑 수신(초기화) */
        $t_log_cash_model = new TLogCashModel();
        $ip = CodeUtil::get_client_ip();
        $member_money = doubleval($this->member_info->money);
        $ukey = md5($this->member_info->idx . strtotime('now'));
        $amount_money = $winning_amount; //amount to convert
        $before_money = $member_money; //balance before transaction
        $after_money = $member_money + $winning_amount; //balance after transaction
        $this->member_info->money = $after_money;
        $description = "SLOTBANK BETTING AMOUNT";


        $t_log_cash_model = $t_log_cash_model->set('u_key', $ukey);
        $t_log_cash_model = $t_log_cash_model->set('member_idx', $this->member_info->idx);
        $t_log_cash_model = $t_log_cash_model->set('ac_code', $ac_code);
        $t_log_cash_model = $t_log_cash_model->set('ac_idx', 0);
        $t_log_cash_model = $t_log_cash_model->set('r_money', $amount_money);
        $t_log_cash_model = $t_log_cash_model->set('be_r_money', $before_money);
        $t_log_cash_model = $t_log_cash_model->set('af_r_money', $after_money);
        $t_log_cash_model = $t_log_cash_model->set('m_kind', 'P');
        $t_log_cash_model = $t_log_cash_model->set('u_ip', $ip);

        $t_log_cash_model = $t_log_cash_model->set('coment', CodeUtil::TLogACCodeToStr($ac_code) . ' ' . $commend);
        $insert_cash = $t_log_cash_model->insert();

        /* commit or rollback if there is no error / 오류가 없으면 커밋 또는 롤백 */
        /*if ($this->DB->transStatus() === false) {
            $this->DB->transRollback();
            $this->setResult(400, 'Database Error');
        } else {
            $this->DB->transCommit();
        }*/
    }

    public function refund($data, $member_model) {
        /* check if bet exist for update */
        $this->logger->info(':: SlotBank:: refund data : ' . json_encode($data));
        $bet_status = "B";
        $betting_info_model = new TBettingInfo();
        $betting_info_callback_model = new TBettingInfoCallback();
        $betting_check = $betting_info_model->getBettingInfoForUpdate($data);

        if (!$betting_check) {
            $this->setResult(90, "Betting doesn't exist.");
        } else if ($bet_status != $betting_check['status']) {
            $this->setResult(99, "result function duplicate invocation.");
        } /* else if ($betting_check["bet_money"] != $data["amount"]) {
          $this->setResult(99, "amount data error.");
          } */ else if ($betting_check["bet_money"] > 0) {
            //$betting_amount = $betting_check["bet_money"];
            $betting_amount = $data["amount"];

            $this->logger->info(':: SlotBank refund:: amount => ' . $data["amount"]);

            if ($betting_check["take_money"] + $data["amount"] == $betting_check["bet_money"]) {
                $bet_status = "R"; //canceled
            }
            /* add balance to user account / 사용자 계정에 잔액 추가 */
            $member_model = $member_model->set('money', 'money+' . $betting_amount, FALSE);
            $member_model = $member_model->where('idx', $this->member_info->idx);
            $member_model->update();

            /* insert log - wallet received (intitialize) / 로그 삽입 - 지갑 수신(초기화) */
            $t_log_cash_model = new TLogCashModel();
            $ip = CodeUtil::get_client_ip();
            $member_money = doubleval($this->member_info->money);
            $ukey = md5($this->member_info->idx . strtotime('now'));
            $amount_money = $betting_amount; //amount to convert
            $before_money = $member_money; //balance before transaction
            $after_money = $member_money + $betting_amount; //balance after transaction
            $this->member_info->money = $after_money;
            $description = "SLOTBANK BETTING AMOUNT REFUNDED";

            /* insert log - wallet received (create query) / 로그 삽입 - 지갑 수신(쿼리 생성) */
            $ac_code = BET_REFUND;
            $t_log_cash_model = $t_log_cash_model->set('u_key', $ukey);
            $t_log_cash_model = $t_log_cash_model->set('member_idx', $this->member_info->idx);
            $t_log_cash_model = $t_log_cash_model->set('ac_code', $ac_code);
            $t_log_cash_model = $t_log_cash_model->set('ac_idx', 0);
            $t_log_cash_model = $t_log_cash_model->set('r_money', $amount_money);
            $t_log_cash_model = $t_log_cash_model->set('be_r_money', $before_money);
            $t_log_cash_model = $t_log_cash_model->set('af_r_money', $after_money);
            $t_log_cash_model = $t_log_cash_model->set('m_kind', 'P');
            $t_log_cash_model = $t_log_cash_model->set('u_ip', $ip);
            $commend = $betting_check["product_type_id"] == 1 ? 'casino' : 'slot';
            $t_log_cash_model = $t_log_cash_model->set('coment', CodeUtil::TLogACCodeToStr($ac_code) . ' ' . $commend);
            $insert_cash = $t_log_cash_model->insert();

            /* record betting information / 베팅 정보 기록 */

            $take_money = $betting_check["take_money"] + $betting_amount;
            $betting_info_model = $betting_info_model->set('status', $bet_status); // 0 Pending - 1 Win - 2 Lose
            $betting_info_model = $betting_info_model->set('take_money', $take_money);
            //$betting_info_model = $betting_info_model->set('take_money', 'take_money+' . $betting_amount, FALSE);
            $betting_info_model = $betting_info_model->where('round_id', $data["round_id"]);
            $betting_info_model = $betting_info_model->where('game_id', $data["game_code"]);
            $betting_info_model = $betting_info_model->where('trx_id', $data["trans_id"]);
            $update_betting_info = $betting_info_model->update();

            /* betting callback transaction */
            $insert["betting_info_id"] = $betting_check["idx"];
            $insert["callback_type"] = "result";
            $insert["trx_id"] = $data["trans_id"];
            $insert["amount"] = $data["amount"];
            $insert["payload"] = json_encode($data);
            $betting_info_callback_model->insert($insert);

            if ("R" == $bet_status) {
                Calculate::updateChargeBetMoney($this->member_info->idx, strtoupper($commend), $betting_check["bet_money"] * - 1, $member_model, $this->logger);
                UserPayBack::AddBetting($this->member_info->idx, $betting_check["bet_money"] * - 1, $member_model);
                UserBettingLose::RewardBettingComps($this->member_info->idx, $commend, $betting_check["bet_money"] * - 1, $this->logger, $member_model);
            }
        }

        $this->logger->info(':: SlotBank:: refund()');
    }

    private function __sendTelegramMessageErrorOpeningGame($error) {
        $telegram_class = new TelegramClass();
        $send_message = "An error has occurred while opening a game in Slotbank. Please check the details below. \n\n";
        $send_message .= "Project: <code>" . $this->project . "</code> \n";
        $send_message .= "Operator Name: <code>" . $this->operator_name . "</code> \n";
        $send_message .= "Date/Time: <code>" . date('m/d/Y h:i:s A', time()) . "</code> \n";
        $send_message .= "Game ID: <code>" . $this->game_information["game_id"] . "</code> \n";
        $send_message .= "Game Name (EN): <code>" . $this->game_information["gamet_name_en"] . "</code> \n";
        $send_message .= "Game Name (KR): <code>" . $this->game_information["game_name_kr"] . "</code> \n";
        $send_message .= "Provider Name (EN): <code>" . $this->product_group_info["desc"] . "</code> \n";
        $send_message .= "Provider Name (KR): <code>" . $this->product_group_info["product_group_name"] . "</code> \n";
        $send_message .= "Error Code: <code>" . $error["result"] . "</code> \n";
        $send_message .= "Error Message: <code>" . $error["msg"] . "</code> \n\n";
        $send_message .= "Full Error Details: \n<code>" . serialize($error) . "</code> \n\n";
        $telegram_class->send('slotbank_monitoring', $send_message);
    }

    private function sendAPIRequest($api_path, $params) {
        /* create a URL for the API request */
        $params["opkey"] = $this->opkey;
        $params["hash"] = md5($this->opkey . "|" . http_build_query($params));
        $url = $this->api_url . $api_path . "?" . http_build_query($params);

        /* send api request via CURL */
        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($curl);
        curl_close($curl);
        return $res;
    }

    private function __createAccountOnSlotBank() {
        $params["userid"] = $this->member_info->idx;
        $params["nick"] = $this->member_info->id;
        $response = $this->sendAPIRequest("/create_account", $params);
    }

    private function __getGameList($game_company_code) {
        $params["thirdpartycode"] = $game_company_code;
        $api_response = $this->sendAPIRequest('/gamelist', $params);
        $_game_list = json_decode($api_response, true)["data"]["list"];
        $this->logger->info("@@@ " . count($_game_list) . " games has been found.");



        return $_game_list;
    }

    private function __checkSaveGames($product_group_info, $_game_list) {
        foreach ($_game_list as $game) {
            $tb_static_game_id_manager = new TGameIdManager();
            $check_exist = $tb_static_game_id_manager->getGameInformation($game["code"], 3, $product_group_info["product_group_id"]);

            $date_time = new DateTime($game["date"]);
            $game_date = $date_time->format('Y-m-d H:i:s');

            $insert_game = [];
            $insert_game["game_short_name"] = $game["name_kor"];
            $insert_game["gamet_name_en"] = $game["name_eng"];
            $insert_game["game_name_kr"] = $game["name_kor"];
            $insert_game["game_image"] = $game["img_1"];
            $insert_game["is_desktop"] = $game["is_desktop"];
            $insert_game["is_mobile"] = $game["is_mobile"];
            $insert_game["game_date"] = $game_date;
            $insert_game["updated_at"] = date('Y-m-d H:i:s');

            if ($check_exist) {
                $tb_static_game_id_manager->set('game_short_name', $game["name_kor"]);
                $tb_static_game_id_manager->set('gamet_name_en', $game["name_eng"]);
                $tb_static_game_id_manager->set('game_name_kr', $game["name_kor"]);
                $tb_static_game_id_manager->set('game_image', $game["img_1"]);
                $tb_static_game_id_manager->set('is_desktop', $game["is_desktop"]);
                $tb_static_game_id_manager->set('is_mobile', $game["is_mobile"]);
                $tb_static_game_id_manager->set('game_date', $game_date);
                $tb_static_game_id_manager->set('updated_at', date('Y-m-d H:i:s'));
                $tb_static_game_id_manager->set('product_type_id', $product_group_info["product_type_id"]);
                $tb_static_game_id_manager->where('provider_id', 3)->where('game_id', $game["code"])->where('product_group_id', $product_group_info["product_group_id"])->update();
                $this->logger->info("- Game ID (" . $game["code"] . ") - (" . $product_group_info["desc"] . ") updated." . "(" . $product_group_info["product_type_id"] . ")");
            } else {
                $insert_game["created_at"] = date('Y-m-d H:i:s');
                $insert_game["provider_id"] = 3;
                $insert_game["product_group_id"] = $product_group_info["product_group_id"];
                $insert_game["game_id"] = $game["code"];
                $insert_game["product_type_id"] = $product_group_info["product_type_id"];
                $tb_static_game_id_manager->insert($insert_game);
                $this->logger->info("- Game ID (" . $game["code"] . ") - (" . $product_group_info["desc"] . ") created.");
            }
        }
    }

    private function __checkSaveGameCompany($reference, $data) {
        
    }

    public function __setAPIResponse($response_code, $message, $additional_response = null) {
        $response["result"] = $response_code;

        if ($this->player_information && $response_code == 1) {
            $response["balance"] = $this->player_information->account_money;
            $response["player_id"] = $this->player_information->player_id;
            $response["username"] = $this->player_information->username;
            $response["nickname"] = $this->player_information->nickname;
        }

        if ($additional_response) {
            foreach ($additional_response as $key => $additional) {
                $response[$key] = $additional;
            }
        }

        $response["message"] = $message;

        if ($response_code !== 1) {
            $this->is_error = true;
        }

        $this->api_response = $response;
    }

    public function setResult($code, $message = null, $balance = 0) {
        $this->result["return_code"] = $code;
        $this->result["message"] = $message;

        if ($code == 1) {
            $this->result["balance"] = $this->member_info->money;
        }
    }

    public function getRoundResult($round_id) {
        $params["roundid"] = $round_id;
        $res = json_decode($this->sendApiRequest("/log/get", $params), true);
        return $res["data"]["link"];
    }

}
