<?php

namespace App\Models;

use CodeIgniter\Model;

class TBettingInfo extends Model
{

    protected $DBGroup = 'default';
    protected $table = 'tb_betting_info';
    protected $primaryKey = 'idx';
    protected $returnType = 'array';
    protected $useSoftDeletes = false;
    protected $allowedFields = [
        'member_idx',
        'status',
        'trx_id',
        'round_id',
        'bet_money',
        'hold_money',
        'take_money',
        'game_id',
        'product_id',
        'product_type_id',
        'provider_id',
        'product_group_id',
        'bonus_type',
        'create_dt',
        'calc_dt',
    ];
    protected $validationRules = [];
    protected $validationMessages = [];
    protected $skipValidation = false;

    public function getOrderByTransId($transactionId)
    {
        return $this->where('trx_id', $transactionId)->first();
    }

    public function getPendingBets($productId=null, $transactionId=null)
    {
        $sql = "SELECT * FROM tb_betting_info WHERE provider_id = 2 AND status = 'B' AND DATE_ADD(create_dt, INTERVAL 3 MINUTE) < NOW()";
        $qCondition = [];
        if($productId) {
            $sql .= ' AND product_id = ?';
            array_push($qCondition, $productId);
        }
        if($transactionId) {
            $sql .= ' AND trx_id = ?';
            array_push($qCondition, $transactionId);
        }

        return $this->db->query($sql, $qCondition)->getResultArray();
    }

    public function findKGOrderStatus($productId, $gameId, $transactionId)
    {
        return $this->where('provider_id', 2)
            ->where('product_id', $productId)
            ->where('game_id', $gameId)
            ->where('trx_id', $transactionId)
            ->first();
    }

    public function findOrderStatusByProvider($providerId, $productId, $gameId, $transactionId)
    {
        return $this->where('provider_id', $providerId)
            ->where('product_id', $productId)
            ->where('game_id', $gameId)
            ->where('trx_id', $transactionId)
            ->first();
    }

    public function setOrderStatus($transactionId, $gameId, $betAmount, $winAmount)
    {
        return $this->set('bet_money', $betAmount)->set('take_money', $winAmount)
            ->where('trx_id', $transactionId)
            ->where('game_id', $gameId)
            ->update();
    }
    
    public function getBettingInfoForUpdate($data){
        $sql = "select tb_betting_info.*
                from tb_betting_info 
                left join tb_static_game_id_manager on 
                tb_static_game_id_manager.game_id = tb_betting_info.game_id
                where tb_betting_info.member_idx = ? and tb_betting_info.game_id = ? and tb_betting_info.round_id = ? ";//for update";
       
         $result = $this->db->query($sql, [$data["userid"],$data["game_code"], $data["round_id"]])->getResultArray();
         
         return !isset($result) || 0 == count($result) ? null : $result[0];
            
    }

    public function getBettingInfoForUpdateGoodGaming($data, $member_idx){
        $sql = "select tb_betting_info.*
                from tb_betting_info 
                left join tb_static_game_id_manager on 
                tb_static_game_id_manager.game_id = tb_betting_info.game_id
                where tb_betting_info.member_idx = ? and tb_betting_info.game_id = ? and tb_betting_info.round_id = ? ";// for update";
       
         $result = $this->db->query($sql, [$member_idx,$data["gameCode"], $data["gameRound"]])->getResultArray();
         
         return !isset($result) || 0 == count($result) ? null : $result[0];
            
    }

    public function updateBettingInfo($take_money, $bet_status, $trans_id, $betting_idx){
        $sql = "UPDATE tb_betting_info SET take_money = take_money + ?, status = ?, trx_id = ?
                where tb_betting_info.idx = ? ";
       
         $result = $this->db->query($sql, [$take_money,$bet_status, $trans_id, $betting_idx])->getResultArray();
         
         return !isset($result) || 0 == count($result) ? null : $result[0];
    }
}