<?php

namespace App\Models;

use CodeIgniter\Model;
use App\Models\GameFavoriteModel;

class TGameIdManager extends Model
{

    protected $DBGroup = 'default';
    protected $table = 'tb_static_game_id_manager';
    protected $primaryKey = 'game_id';
    protected $returnType = 'array';
    protected $useSoftDeletes = false;
    protected $allowedFields = [
        'game_id',
        'game_short_name',
        'desc',
        'create_dt',
        'update_dt',
        'is_use',
        'status',
        'gamet_name_en',
        'game_name_kr',
        'min_bet',
        'max_bet',
        'product_id',
        'product_group_id',
        'product_type_id',
        'provider_id',
        'is_desktop',
        'is_mobile',
        'game_image',
        'game_date',
        'popularity_count',
        
    ];
    protected $validationRules = [];
    protected $validationMessages = [];
    protected $skipValidation = false;

    /**
     * Get List of Games from the Database  / 게임 및 타사 공급자 목록 가져오기
     * @param member_idx
     * @param provider_id
     * @param product_type_id
     * @param product_group_id
     * @param filter_mode
     * @param limit
     * @param skip
     * @return game_list
     */
    public function getGameInformation($game_id, $provider_id, $product_group_id)
    {
        $query = $this->where('tb_static_game_id_manager.game_id', $game_id);
        $query = $query->where('tb_static_game_id_manager.product_group_id', $product_group_id);
        $query = $query->where('tb_static_game_id_manager.provider_id', $provider_id);
        return $this->first();
    }

    public function getGameList($member_idx, $provider_id, $product_type_id, $product_group_id, $filter_mode, $limit = 100, $skip = 0,$search_val)
    {
        /* get favorite game list / 좋아하는 게임 목록 가져오기 */
        $game_favorite_model    = new GameFavoriteModel();
        $_favorite_list         = $game_favorite_model->where('member_id', $member_idx)->findAll();
        $game_array_favorite    = [];
        $game_array_game_name    = [];

        /* create array of ID for favorite games list / 좋아하는 게임 목록에 대한 ID 배열 생성 */
        foreach($_favorite_list as $favorite)
        {
            array_push($game_array_favorite, $favorite["game_id"]);
            array_push($game_array_game_name, $favorite["game_name"]);
        }

        /* get game list / 게임 목록 가져오기 */
        $this->select('tb_static_game_id_manager.*');
        $this->where('tb_static_game_id_manager.provider_id', $provider_id);

        /* additional query if filter_mode is `filtered` / filter_mode가 'filtered'인 경우 추가 쿼리 */
        if($filter_mode == 'filtered')
        {
            $this->where('tb_static_game_id_manager.product_group_id', $product_group_id);
        }

        /* additional query if filter_mode is `favorite` / filter_mode가 'favorite'인 경우 추가 쿼리 */
        if($filter_mode == 'favorite')
        {
            $this->join('game_favorite', 'game_favorite.game_id = tb_static_game_id_manager.game_id AND game_favorite.game_name = tb_static_game_id_manager.game_name_kr');
            $this->where('game_favorite.member_id', $member_idx);
        }

        /* additional query if filter_mode is `recent` / filter_mode가 'recent'인 경우 추가 쿼리 */
        if($filter_mode == 'recent')
        {
            $this->join('game_played', 'game_played.game_id = tb_static_game_id_manager.game_id');
            $this->where('game_played.member_id', $member_idx);
            $this->orderBy('game_played.last_played_datetime DESC');
        }

        /* additional query if filter_mode is `popular` / filter_mode가 'popular'인 경우 추가 쿼리 */
        if($filter_mode == 'popular')
        {
            $this->orderBy('tb_static_game_id_manager.popularity_count DESC');
            $this->where('tb_static_game_id_manager.popularity_count >', 0);
        }
        if($search_val != '')
        {
            if (preg_match('/[\x{1100}-\x{11FF}\x{3130}-\x{318F}\x{AC00}-\x{D7AF}]/u', $search_val)) {
                // Search in the Korean game name column
                $this->like('tb_static_game_id_manager.game_name_kr', $search_val, 'both');
            } else {
                // Search in the English game name column
                $this->like('tb_static_game_id_manager.gamet_name_en', $search_val, 'both');
            }
        }


        /* hide games that are not enabled in provider */
        $this->join('tb_static_switch_manager', 'tb_static_switch_manager.product_group_id = tb_static_game_id_manager.product_group_id AND tb_static_switch_manager.provider_id = tb_static_game_id_manager.provider_id', 'left');
        $this->where('tb_static_switch_manager.is_use', 'ON');
        $this->where('tb_static_game_id_manager.product_type_id', $product_type_id);
        

        /* get result of game list / 게임 목록 결과 가져오기 */
        $_game_list             = $this->findAll($limit, $skip);

        /* cross reference game_list with game_favorite to check if game is in the user's favorite list / game_list를 game_favorite와 교차 참조하여 게임이 사용자의 즐겨찾기 목록에 있는지 확인합니다. */
        foreach($_game_list as $i => $game_list)
        {
            if(in_array($game_list["game_id"], $game_array_favorite) && in_array($game_list["game_name_kr"], $game_array_game_name))
            {
                $_game_list[$i]["is_favorite"] = 1;
            }
            else
            {
                $_game_list[$i]["is_favorite"] = 0;
            }
        }

        return $_game_list;
    }
    public function increasePopularityCount($game_id, $increase_count)
    {
        $this->set('popularity_count', 'popularity_count+' . $increase_count, FALSE);
        $this->where('game_id', $game_id);
        $this->update();
    }
}
